@Echo off

REM
REM Name: PatchMDB.bat 
REM
REM Description:	Patch an MDB instance in SQL Server
REM 
REM Syntax:
REM
REM 	PatchMDB [-DBMS_INSTANCE=xxxx] [-MDB_NAME=xxxx] -MDB_PATCH=xxxx [-MDB_TARGET_DIR=<dir>] [-DBUSER=xxxx] [-DBPASSWORD=xxxx] [-DBPORT=xxxx] [-MDB_EXIT_SCRIPT=<yes or no>] [-FORCE_PATCH=<yes or no>] [-debug] [-?]
REM
REM History:
REM	17-oct-2005 mulwi01
REM	    Created
REM	29-nov-2005	mulwi01	generate signature
REM	19-jan-2005	ricle02	Star 14599933 PATCH COLLATION HANDLING
REM	11-mar-2006	mulwi01	Star 14712269 OSQL USAGE - DASH VALUE
REM     18-aug-2006     rigka01 Star 15148450, MDB21, Add more logging 
REM     4-dec-2006      todjo06 Star 15277870 Authentication change
REM     12-mar-2007     todjo06 Star 15215592 Using full path (UNC)
REM     12-mar-2007     todjo06 Star 15546020 Return 0 on reapply
REM     26-Apr-2007     todjo06 Star 15914385 DBPORT parameter
REM     27-Apr-2007     todjo06 Star 15918155 Signature problem
REM	28-Sep-2007     rigka01	Star 16377823 STAR MDB 32 check instance exists
REM 	01-Oct-2007	rigka01	Star 16377773 STAR MDB 116 remove date and time from Usage messages
REM     11-Oct-2007	rigka01 Star 16441800 MDB 119 Connection errors when applying a patch to the MDB for SQL Server Express
REM	19-Oct-2007 	rigka01 Star 16029870 MDB 121 Implement -MDB_EXIT_SCRIPT parameter same as in setupmdb
REM     24-Oct-2007     rigka01 Star 16477216 MDB 122 Hide any passwords when echoing/logging the Command line
REM     24-Oct-2007     rigka01 Fix MDB 121
REM     19-Nov-2007     rigka01 STAR 16528888, problem MDB 125, DDL script for SQL Server 2005 and higher, not 2000: ",,,2005" in manifest
REM     16-Jan-2008     rigka01 STAR 16650634, problem MDB 132, DDL script to exit immediately on encoutering error: ",,,,onErrExit" in manifest 
REM     29-Feb-2008     rigka01 STAR 16898454, problem MDB 144, add -FORCE_PATCH option to allow re-run with same value for MDB_PATCH 
REM	25-Apr-2008 	rigka01 STAR 17072813 MDB 162 patchmdb passwords fail when they contain an exclamation point
REM   22-Aug-2008    hopst01 STAR 17393114 PATCH MDB PSWORD SPECIAL CHAR, Allow special characters in password by doubld quoting the password
REM   05-dec-2008    hopst01  17727007 - FOR LOOP W CMD NOT SUPPORTED. For loops that use 
REM                           a command to populate the for loop set should be replaced. 
REM


REM
REM Start localization of environment variables in a batch file
REM
SETLOCAL ENABLEDELAYEDEXPANSION

echo %date% %time% Started PatchMDB
REM 15215592 - use full path of PatchMDB to find other
REM   batch files and executables.
set PATCHMDB_PATH=%~dp0%
set PATCHMDB_VERSION=38
echo %date% %time% PatchMDB version: %PATCHMDB_VERSION%

REM Hide DBPASSWORD if it is in the setupmdb command; keep the rest 
set COMMAND_HIDDEN_PSWDS=%~f0
set TEST_ARGS=%*
set HAS_PSWD=false
set EVEN_ARG=yes
for  %%a in (%TEST_ARGS%) do (
  if !HAS_PSWD! EQU true ( 
        set COMMAND_HIDDEN_PSWDS=!COMMAND_HIDDEN_PSWDS!=
        if !EVEN_ARG! EQU no (
          set EVEN_ARG=yes
	) else (
	  set EVEN_ARG=no
	)
        set COMMAND_HIDDEN_PSWDS=!COMMAND_HIDDEN_PSWDS!********
  	set HAS_PSWD=false
  ) else (
        if !EVEN_ARG! EQU no (
          set COMMAND_HIDDEN_PSWDS=!COMMAND_HIDDEN_PSWDS!=
          set COMMAND_HIDDEN_PSWDS=!COMMAND_HIDDEN_PSWDS!%%a
          set EVEN_ARG=yes
        )  else (
          set EVEN_ARG=no
          set COMMAND_HIDDEN_PSWDS=!COMMAND_HIDDEN_PSWDS! %%a
  	  if "%%a" == "-DBPASSWORD" ( 
	     set HAS_PSWD=true
          )
  	)
  )
)

SETLOCAL DISABLEDELAYEDEXPANSION
echo %date% %time% PatchMDB command (with full path, without passwords): %COMMAND_HIDDEN_PSWDS%

REM set DEBUGECHO=@ECHO on
set DEBUGECHO=@ECHO off
%DEBUGECHO%

set CURDIR=%CD%

REM
REM Set the default parameters
REM
set MDB_ADMIN=mdbadmin
set MDB_NAME=mdb
set DBMS_INSTANCE=
set MDB_TARGET_DIR=%PATCHMDB_PATH%
set EXIT_SCRIPT=/B
set AUTHENTICATION_TYPE=WIN
set DBUSER=
set DBPASSWORD=
set AUTH_STRING=
set AUTH_STRING_NO_PASSWORD=
set INSTANCE_ARG=
set DBPORT=
set HASPORT=0
set HASINSTANCE=0
set FORCE_PATCH=
rem set DBPASSWORD_DQ=
set MDB_RC=2
set ERR_MESSAGE=Patch MDB error

REM
REM Parse the command line parameters
REM

:PARSE_ARGS

IF "%~1"==""   GOTO NO_MORE_ARGS
IF "%~1"=="-?" GOTO USAGE

IF /I NOT "%~1"=="-DBMS_INSTANCE" GOTO NEXT_ARG1
SHIFT
if "%~1"=="local" goto DBMS_INSTANCE_LOCAL
set HASINSTANCE=1
set INSTANCE_ARG=%~1
set DBMS_INSTANCE=-S "%~1"
set DBMS_INSTANCE_PATCH=-DBMS_INSTANCE="%~1"
:DBMS_INSTANCE_LOCAL
set INSTANCE_ARG=%~1
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG1
IF /I NOT "%~1"=="-MDB_PATCH" GOTO NEXT_ARG2
SHIFT
set MDB_PATCH=%~1
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG2
IF /I NOT "%~1"=="-MDB_NAME" GOTO NEXT_ARG3
SHIFT
set MDB_NAME=%~1
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG3
IF /I NOT "%~1"=="-MDB_TARGET_DIR" GOTO NEXT_ARG4
SHIFT
set MDB_TARGET_DIR=%~1
IF EXIST "%MDB_TARGET_DIR%" GOTO TARGET_DIR_EXIST
echo %date% %time% Error: MDB_TARGET_DIR does not exist: "%MDB_TARGET_DIR%". 
GOTO DONE_ERROR 
:TARGET_DIR_EXIST
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG4

IF /I NOT "%~1"=="-debug" GOTO NEXT_ARG5 
set DEBUGECHO=@ECHO on 
%DEBUGECHO%
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG5
IF /I NOT "%~1"=="-DBUSER" GOTO NEXT_ARG6
SHIFT
set DBUSER=%~1
set AUTHENTICATION_TYPE=SQL
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG6
IF /I NOT "%~1"=="-DBPASSWORD" GOTO NEXT_ARG7
SHIFT
set DBPASSWORD_DQ="%~1"
set DBPASSWORD="%~1"
rem Escape any reserved shell characters
set DBPASSWORD=%DBPASSWORD:^=^^%
set DBPASSWORD=%DBPASSWORD:<=^<%
set DBPASSWORD=%DBPASSWORD:>=^>%
set DBPASSWORD=%DBPASSWORD:|=^|%
set DBPASSWORD=%DBPASSWORD:&=^&%
set DBPASSWORD=%DBPASSWORD:(=^(%
set DBPASSWORD=%DBPASSWORD:)=^)%
set DBPASSWORD=%DBPASSWORD:"=\"%
set AUTHENTICATION_TYPE=SQL
SHIFT
GOTO PARSE_ARGS

REM STAR 15914385 - allow port number as a separate parameter

:NEXT_ARG7
IF /I NOT "%~1"=="-DBPORT" GOTO NEXT_ARG8 
SHIFT
set HASPORT=1
set DBPORT=%~1
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG8
IF /I NOT "%~1"=="-FORCE_PATCH" GOTO NEXT_ARG9
SHIFT
IF /I NOT "%~1"=="yes" GOTO FORCE_PATCH_CHECK_NO
set FORCE_PATCH=1
SHIFT
GOTO PARSE_ARGS
:FORCE_PATCH_CHECK_NO
IF /I NOT "%~1"=="no" GOTO FORCE_PATCH_ERR
set FORCE_PATCH=
SHIFT
GOTO PARSE_ARGS
:FORCE_PATCH_ERR
echo %date% %time% Invalid -FORCE_PATCH argument "%~1", expected yes or no
set ERR_MESSAGE=Invalid -FORCE_PATCH argument "%~1", expected yes or no
GOTO USAGE

:NEXT_ARG9
IF NOT "%~1"=="-MDB_EXIT_SCRIPT" GOTO INVALID_ARG 
SHIFT
IF NOT "%~1"=="yes" GOTO CHECK_NO
set EXIT_SCRIPT=/B
SHIFT
GOTO PARSE_ARGS
:CHECK_NO
IF NOT "%~1"=="no" GOTO XSCRIPT_ERR
set EXIT_SCRIPT=
SHIFT
GOTO PARSE_ARGS
:XSCRIPT_ERR
echo %date% %time% Invalid -MDB_EXIT_SCRIPT argument "%~1", expected yes or no
set ERR_MESSAGE=Invalid -MDB_EXIT_SCRIPT argument "%~1", expected yes or no
GOTO USAGE

:INVALID_ARG
echo %date% %time% Usage error: Invalid PatchMDB argument "%~1"
GOTO USAGE

:NO_MORE_ARGS

if NOT "%AUTHENTICATION_TYPE%"=="WIN" GOTO SQL_AUTH
set AUTH_STRING=-E
set AUTH_STRING_NO_PASSWORD=-E
GOTO AUTH_DONE

:SQL_AUTH

IF "%DBUSER%"=="" GOTO AUTH_ERR
IF (%DBPASSWORD%)==() GOTO AUTH_ERR

set AUTH_STRING=-U "%DBUSER%" -P %DBPASSWORD_DQ%
set AUTH_STRING_NO_PASSWORD=-U "%DBUSER%" -P "********"
GOTO AUTH_DONE

:AUTH_ERR
echo %date% %time% Usage error: Missing argument. To use SQL Server authentication, both DBUSER and DBPASSWORD must be specified.
goto USAGE

:AUTH_DONE

if (%HASPORT%) EQU (0) GOTO NO_PORT_NUM
if ("%INSTANCE_ARG%") NEQ ("local") GOTO INSTANCE_NOT_LOCAL
echo %date% %time% Usage error: The specification of local as the value of DBMS_INSTANCE is not allowed when DBPORT is specified. 
GOTO USAGE

:INSTANCE_NOT_LOCAL
if (%HASINSTANCE%) EQU (1) GOTO HAS_INSTANCE
echo %date% %time% Usage error: Port number must be accompanied by DBMS_INSTANCE designation.  
GOTO USAGE

:HAS_INSTANCE
set DBMS_INSTANCE=-S "%INSTANCE_ARG%,%DBPORT%"
set DBMS_INSTANCE_PATCH=-DBMS_INSTANCE "%INSTANCE_ARG%,%DBPORT%"

:NO_PORT_NUM


IF NOT "%MDB_PATCH%" == "" GOTO MDBPATCH_FOUND
echo %date% %time% Usage error: Missing MDB_PATCH argument
GOTO USAGE

:MDBPATCH_FOUND

rem
rem check if SQL Server is available
rem
sqlcmd -? > nul 2>&1
IF %ERRORLEVEL% NEQ 9009 (
	set SQL_CMD=sqlcmd
	GOTO OSQL_FOUND
)
osql -? > nul 2>&1
IF %ERRORLEVEL% NEQ 9009 (
	set SQL_CMD=osql
	GOTO OSQL_FOUND
)
set ERR_MESSAGE=Cannot find osql or sqlcmd. Check that SQL Server has been installed and is in the PATH.
echo %date% %time% %ERR_MESSAGE%
GOTO DONE_ERROR
:OSQL_FOUND

set PATCH_PREFIX=%PATCHMDB_PATH%patches\p
set PATCH_MANIFEST="%PATCH_PREFIX%%MDB_PATCH%.txt"
IF EXIST %PATCH_MANIFEST% GOTO MANI_EXISTS
set ERR_MESSAGE=Error: Missing patch manifest %PATCH_MANIFEST%.
echo %date% %time% %ERR_MESSAGE%
GOTO DONE_ERROR
:MANI_EXISTS




REM
REM Set the patch log file
REM
IF (%LOGFILE%) == () set LOGFILE="%MDB_TARGET_DIR%\install_%MDB_NAME%.log"
echo . >> %LOGFILE% 2>&1
echo . >> %LOGFILE% 2>&1
echo . >> %LOGFILE% 2>&1
echo %date% %time% PatchMDB mdb log file for %MDB_PATCH% in %LOGFILE%
echo %date% %time% ====================================================================== >> %LOGFILE% 2>&1
echo %date% %time% PatchMDB mdb log file for %MDB_PATCH% in %LOGFILE% >> %LOGFILE% 2>&1

REM 
REM Log the full PatchMDB version command
REM
echo %date% %time% PatchMDB version: %PATCHMDB_VERSION% >> %LOGFILE% 2>&1
echo %date% %time% PatchMDB command: (with full path, without passwords): %COMMAND_HIDDEN_PSWDS% >> %LOGFILE% 2>&1

chdir /D "%MDB_TARGET_DIR%" >> %LOGFILE% 2>&1

REM
REM check if SQL Server instance exists
REM
if (%HASPORT%) EQU (1) GOTO HAS_PORT_MSG
echo %date% %time% Checking status of instance: %INSTANCE_ARG% ...
echo %date% %time% Checking status of instance: %INSTANCE_ARG% ... >>%LOGFILE% 2>&1
GOTO CHECK_INSTANCE


:HAS_PORT_MSG
echo %date% %time% Checking status of instance %INSTANCE_ARG% with port %DBPORT% ...
echo %date% %time% Checking status of instance %INSTANCE_ARG% with port %DBPORT% ... >>%LOGFILE% 2>&1

:CHECK_INSTANCE
echo %date% %time% %SQL_CMD% %AUTH_STRING_NO_PASSWORD% %DBMS_INSTANCE% -d master -e -b -Q "exit" >>%LOGFILE% 2>&1
%SQL_CMD% %AUTH_STRING% %DBMS_INSTANCE% -d master -e -b -Q "exit" 
IF %ERRORLEVEL% EQU 0 GOTO INSTANCE_FOUND
IF (%HASPORT%) EQU (1) GOTO ERROR_HAS_PORT_MSG
set ERR_MESSAGE=Error: Instance %INSTANCE_ARG% does not exist or is not valid.
GOTO ERROR_MESSAGE

:ERROR_HAS_PORT_MSG
set ERR_MESSAGE=Error: Instance "%INSTANCE_ARG% with Port %DBPORT%" does not exist or is not valid.
GOTO ERROR_MESSAGE

:INSTANCE_FOUND
REM
REM check if mdb already exists
REM
echo %date% %time% Checking if %MDB_NAME% already exists...
echo %date% %time% Checking if %MDB_NAME% already exists... >>%LOGFILE% 2>&1
echo %date% %time%>> %LOGFILE%
echo %date% %time% %SQL_CMD% %AUTH_STRING_NO_PASSWORD% %DBMS_INSTANCE% -d master -e -b -Q "exit(SELECT count(*) FROM master..sysdatabases WHERE name = '%MDB_NAME%')"    >>%LOGFILE% 2>&1
%SQL_CMD% %AUTH_STRING% %DBMS_INSTANCE% -d master -e -b -Q "exit(SELECT count(*) FROM master..sysdatabases WHERE name = '%MDB_NAME%')"    >>%LOGFILE% 2>&1
IF %ERRORLEVEL% NEQ 0 GOTO MDB_FOUND
set ERR_MESSAGE=Error: Database %MDB_NAME% does not exist.
GOTO ERROR_MESSAGE
:MDB_FOUND




REM
REM Check that the mdb version matches the patch requirement
REM
echo %date% %time% Checking the mdb version
echo %date% %time% Checking the mdb version >> %LOGFILE% 2>&1

FOR /F "eol=- tokens=1,2 delims=, usebackq" %%i in (%PATCH_MANIFEST%) do IF %MDB_PATCH% EQU %%i set PATCH_BUILD=%%j

echo %SQL_CMD% %AUTH_STRING_NO_PASSWORD% %DBMS_INSTANCE% -d %MDB_NAME% -e -b -Q "exit(select buildnumber from mdb)"    >>%LOGFILE% 2>&1
%SQL_CMD% %AUTH_STRING% %DBMS_INSTANCE% -d %MDB_NAME% -e -b -Q "exit(select buildnumber from mdb)"    >>%LOGFILE% 2>&1
set MDB_BUILD=%ERRORLEVEL%

if (%MDB_BUILD%) == (%PATCH_BUILD%) GOTO BUILD_EXIST
set ERR_MESSAGE=Error: Database %MDB_NAME% contains build %MDB_BUILD% but patch requires build %PATCH_BUILD%.
GOTO ERROR_MESSAGE
:BUILD_EXIST

REM 
REM Ensure auto_close off (default for SQL Server Express db is on)
REM
echo %SQL_CMD% %AUTH_STRING_NO_PASSWORD% %DBMS_INSTANCE% -d %MDB_NAME% -e -b -Q "exit(alter database %MDB_NAME% set auto_close off)" >>%LOGFILE% 2>&1
%SQL_CMD% %AUTH_STRING% %DBMS_INSTANCE% -d %MDB_NAME% -e -b -Q "exit(alter database %MDB_NAME% set auto_close off)" >>%LOGFILE% 2>&1
echo %SQL_CMD% %AUTH_STRING_NO_PASSWORD% %DBMS_INSTANCE% -d %MDB_NAME% -e -b -Q "exit(select convert(varchar(50),databasepropertyex('%MDB_NAME%','isautoclose')))" >>%LOGFILE% 2>&1
%SQL_CMD% %AUTH_STRING% %DBMS_INSTANCE% -d %MDB_NAME% -e -b -Q "exit(select convert(varchar(50),databasepropertyex('%MDB_NAME%','isautoclose')))" >>%LOGFILE% 2>&1
IF %ERRORLEVEL% EQU 0 GOTO AUTOCLOSEOFF 
echo %date% %time% Warning: Auto Close property is on for Database %MDB_NAME%.
echo %date% %time% Warning: Auto Close property is on for Database %MDB_NAME%. >> %LOGFILE% 2>&1

:AUTOCLOSEOFF
REM
REM Check if -FORCE_PATCH is specified
REM
if (%FORCE_PATCH%) EQU (1) GOTO SKIP_CHECK_MDB_PATCH

:DO_CHECK_MDB_PATCH
REM
REM Check if the patch already exists
REM
echo %date% %time% Checking if the patch already exists 
echo %date% %time% Checking if the patch already exists >> %LOGFILE%
echo %SQL_CMD% %AUTH_STRING_NO_PASSWORD% %DBMS_INSTANCE% -d %MDB_NAME% -e -b -Q "exit(select count(*) from mdb_patch where patchnumber = %MDB_PATCH%)" >>%LOGFILE% 2>&1
%SQL_CMD% %AUTH_STRING% %DBMS_INSTANCE% -d %MDB_NAME% -e -b -Q "exit(select count(*) from mdb_patch where patchnumber = %MDB_PATCH%)" >>%LOGFILE% 2>&1
IF %ERRORLEVEL% EQU 0 GOTO PATCH_NOT_EXIST
set ERR_MESSAGE=Warning: Database %MDB_NAME% already contains patch %MDB_PATCH%.
echo %date% %time% %ERR_MESSAGE%
echo %date% %time% %ERR_MESSAGE% >> %LOGFILE% 2>&1
GOTO DONE
:SKIP_CHECK_MDB_PATCH
:PATCH_NOT_EXIST




REM
REM Installing patch
REM
set MDB_RC_FILE=mdbpatcherr.txt
IF EXIST "%MDB_RC_FILE%" del "%MDB_RC_FILE%"
echo %date% %time% Installing the patch from the manifest...
echo %date% %time% Installing the patch from the manifest... >> %LOGFILE%
REM Star 14599933
FOR /F "eol=- tokens=1,2,3,4,5,6,7 delims=, usebackq" %%i in (%PATCH_MANIFEST%) do call "%PATCHMDB_PATH%\installmdbpatchsql" %MDB_NAME% "%PATCH_PREFIX%%%i.sql" %%i %%k %%l %%m



REM
REM Check that the patch was installed
REM
%SQL_CMD% %AUTH_STRING% %DBMS_INSTANCE% -d %MDB_NAME% -e -b -Q "exit(select count(*) from mdb_patch where patchnumber = %MDB_PATCH%)" >>%LOGFILE% 2>&1
IF %ERRORLEVEL% NEQ 0 GOTO PATCH_OK
set ERR_MESSAGE=Error: Patch %MDB_PATCH% failed on database %MDB_NAME%.
GOTO ERROR_MESSAGE
:PATCH_OK



REM
REM Create the mdb signature for asset/patch management
REM

REM 15918155 - do not send DBUSER and DBPASSWORD to CreateMDBSignature if
REM  they are not being used

if "%AUTHENTICATION_TYPE%"=="WIN" GOTO CREATE_WIN_SIG

call CreateMDBSignature %DBMS_INSTANCE_PATCH% -MDB_NAME=%MDB_NAME% -DBUSER=%DBUSER% -DBPASSWORD=%DBPASSWORD_DQ%	>>%LOGFILE% 2>&1
GOTO COMPLETED

:CREATE_WIN_SIG

call CreateMDBSignature %DBMS_INSTANCE_PATCH% -MDB_NAME=%MDB_NAME% >>%LOGFILE% 2>&1
	

:COMPLETED
echo %date% %time% 
echo %date% %time% >> %LOGFILE%
echo %date% %time% Patch %MDB_PATCH% installed in database %MDB_NAME%
echo %date% %time% Patch %MDB_PATCH% installed in database %MDB_NAME% >> %LOGFILE%
echo %date% %time% 
echo %date% %time% >> %LOGFILE%
GOTO DONE




:USAGE
echo. 
echo Usage: 
echo. 
echo PatchMDB [-DBMS_INSTANCE=xxxx] [-MDB_NAME=xxxx] -MDB_PATCH=xxxx [-MDB_TARGET_DIR=dir] [-DBUSER=xxxx] [-DBPASSWORD=xxxx] [-DBPORT=xxxx] [-MDB_EXIT_SCRIPT=yes or no] [-FORCE_PATCH=yes or no] [-debug] [-?]
echo. 
echo where: 
echo. 
echo       DBMS_INSTANCE     SQL Server instance name or local 
echo 				(optional with default local)
echo       DBPORT            port number of SQL Server instance 
echo 				(optional with default SQL Server default port)
echo       MDB_NAME          database name        
echo				(optional with default mdb)
echo       MDB_PATCH         patch number         
echo 				(required)
echo       MDB_TARGET_DIR    target MDB directory 
echo 				(optional with default current directory)
echo       DBUSER            SQL Server user 	  
echo 				(required for SQL Server authentication)
echo       DBPASSWORD        password for SQL Server user 
echo 				(required for SQL Server authentication)
echo       MDB_EXIT_SCRIPT   yes to exit the script with /B or no 
echo 				(optional with default yes)
echo       FORCE_PATCH       yes to force processing of manifest for MDB_PATCH
echo                            (optional with default no)
echo       debug             specify this argument to echo output 
echo 				(optional with default no debug message echoing)
echo 	   ?		     specify this argument to echo this usage message 
echo				(optional with default no usage echoing)
echo. 

endlocal
echo %date% %time% Usage error: rc=99 
EXIT /B 99


:ERROR_MESSAGE
echo %date% %time% %ERR_MESSAGE%
echo %date% %time% %ERR_MESSAGE% >> %LOGFILE% 2>&1
echo %date% %time% Error rc=1 >> %LOGFILE% 2>&1
GOTO DONE_ERROR


:DONE_ERROR
echo %date% %time% Setting directory back to "%CURDIR%" 
chdir /D "%CURDIR%" 
echo %date% %time% Error rc=1 
EXIT %EXIT_SCRIPT% 1

:DONE
echo %date% %time% Setting directory back to "%CURDIR%" >> %LOGFILE% 2>&1
chdir /D "%CURDIR%" >> %LOGFILE% 2>&1
echo %date% %time% Successful rc=0 
echo %date% %time% Successful rc=0 >> %LOGFILE% 2>&1
EXIT %EXIT_SCRIPT% 0

endlocal

