SET LOCK_TIMEOUT 300000
GO
SET DEADLOCK_PRIORITY LOW
GO
SET XACT_ABORT ON
GO
SET TRANSACTION ISOLATION LEVEL SERIALIZABLE
GO
BEGIN TRANSACTION 
GO

/****************************************************************************************/
/*                                                                                      */
/* Patch Star 15653570 Autosys: update ujo_det_rep, ujo_jobst, add ujo_jobrow		*/
/*                                                                                      */
/****************************************************************************************/

/* lock objects...  */
 
/* Start of lines added to convert to online lock */
 
 
Select top 1 1 from mdb_patch with (tablockx)
GO
 
/* Start of locks for dependent tables */
 
Select top 1 1 from mdb_service_pack with (tablockx)
GO
 
Select top 1 1 from mdb with (tablockx)
GO
 
Select top 1 1 from mdb_checkpoint with (tablockx)
GO
 
Select top 1 1 from ujo_job_status with (tablockx)
GO

Select top 1 1 from ujo_job2 with (tablockx)
GO

Select top 1 1 from ujo_job with (tablockx)
GO

/* End of lines added to convert to online lock */

/* 14910277 Add evt_num to job order sequence */

if exists (select * from sysobjects where name = "ujo_det_rep" and type = 'P')
begin
	drop procedure ujo_det_rep
end
go

create procedure ujo_det_rep
   @joid int,
   @run_num int

AS

select  status,
        alarm,
        event,
        event_time_gmt,
        ntry,
        que_status,
        MONTH(que_status_stamp),
    	DAY(que_status_stamp),
    	YEAR(que_status_stamp),
    	DATEPART(hh,que_status_stamp),
    	DATEPART(mi,que_status_stamp),
    	DATEPART(ss,que_status_stamp),
    	text,
    	machine,
    	evt_num,
    	(select isnull(max(over_num),0) from ujo_overjob o1 
     	 where o1.joid = j1.joid and o1.run_num = j1.run_num)
from  ujo_eventvu j1
where joid = @joid and run_num = @run_num

union

select  status,
        alarm,
        event,
        event_time_gmt,
        ntry,
        que_status,
        MONTH(que_status_stamp),
    	DAY(que_status_stamp),
    	YEAR(que_status_stamp),
    	DATEPART(hh,que_status_stamp),
    	DATEPART(mi,que_status_stamp),
    	DATEPART(ss,que_status_stamp),
        text,
        machine,
        evt_num,
        (select isnull(max(over_num),0) from ujo_overjob o2 
	 	 where o2.joid = j.joid and o2.run_num = j.run_num)
		 from ujo_proc_eventvu j
where joid = @joid and run_num = @run_num
order by event_time_gmt, evt_num

return -999
go

grant execute on ujo_det_rep to ujoadmin
GO

/* 15653570 - speed up autorep */
if exists (select * from sysobjects where name = "ujo_jobrow" and type = 'V')
begin
	drop view ujo_jobrow
end
go

CREATE VIEW ujo_jobrow AS
SELECT  j.joid, j.job_name,j.job_type,j.owner,j.permission,
        j.box_joid,j.machine,j.n_retrys,
        j.date_conditions,j.days_of_week,j.run_calendar,j.exclude_calendar,
        j.start_times,j.start_mins,j.run_window,
        j.command,
        j.description, j.term_run_time,
        j.box_terminator,
        j.job_terminator,j.std_in_file,j.std_out_file,j.std_err_file,
        j.watch_file,j.watch_file_min_size,j.watch_interval,
        j.min_run_alarm, j.max_run_alarm, j.alarm_if_fail,
        j.chk_files,j.free_procs,j.profile,j.heartbeat_interval,
        j.auto_hold,
        j.job_load,j.priority, j.auto_delete, j.numero,
        j.max_exit_success, j.box_success, j.box_failure,
        status,status_time,last_start,last_end,next_start,run_window_end,
        exit_code,run_machine,que_name,jc_pid,pid,
        run_num, ntry, appl_ntry, last_heartbeat,
        run_priority, next_priority, evt_num, over_num,
        b.job_name box_name,j2.command2, j2.external_app, j2.timezone,
        j2.as_applic,j2.as_group,j.send_notification, j.service_desk
FROM  ujo_job_status s , ujo_job2 j2, ujo_job j 
LEFT OUTER JOIN ujo_job b ON j.box_joid = b.joid
WHERE j.joid = j2.joid AND s.joid = j.joid
go

grant select on ujo_jobrow to ujoadmin 
go

if exists (select * from sysobjects where name = "ujo_jobst" and type = 'V')
begin
	drop view ujo_jobst
end
go

CREATE VIEW  ujo_jobst
AS
SELECT j.*, status, status_time, run_num, ntry, appl_ntry, last_start,
	last_end,next_start,exit_code,run_machine, que_name,
	run_priority, next_priority, pid, jc_pid,time_ok,
	last_heartbeat, j2.timezone,j2.as_applic,j2.as_group
FROM ujo_job j, ujo_job_status s, ujo_job2 j2
WHERE j.joid = s.joid and j.joid = j2.joid
go

grant select on ujo_jobst to ujoadmin
go


/****************************************************************************************/
/*                                                                                      */
/* Register patch                                                               	*/
/*                                                                                      */
/****************************************************************************************/
insert into mdb_patch ( patchnumber, installdate,  mdbmajorversion, mdbminorversion, description)
 values ( 15653570, getdate(), 1, 4, 'Patch Star 15653570 (MODIFIED v2) Autosys: update ujo_det_rep, ujo_jobst, add ujo_jobrow' )
GO

COMMIT TRANSACTION 
GO

