SET LOCK_TIMEOUT 300000
GO
SET DEADLOCK_PRIORITY LOW
GO
SET XACT_ABORT ON
GO
SET TRANSACTION ISOLATION LEVEL SERIALIZABLE
GO
BEGIN TRANSACTION 
GO

/****************************************************************************************/
/*                                                                                      */
/* Star 16255152 DSM: add urc_computer_addr for IPV6					*/
/*                                                                                      */
/****************************************************************************************/

/* lock objects...  */
 
/* Start of lines added to convert to online lock */
 
 
Select top 1 1 from mdb_patch with (tablockx)
GO
 
/* Start of locks for dependent tables */
 
Select top 1 1 from mdb_service_pack with (tablockx)
GO
 
Select top 1 1 from mdb with (tablockx)
GO
 
Select top 1 1 from mdb_checkpoint with (tablockx)
GO
 
/* End of lines added to convert to online lock */

/****** Object:  Table [dbo].[urc_computer_addr]    Script Date: 07/25/2007 10:45:58 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO
CREATE TABLE [dbo].[urc_computer_addr](
	[uuid] [binary](16) NOT NULL,
	[domain_uuid] [binary](16) NOT NULL,
	[computer_uuid] [binary](16) NOT NULL,
	[protocol] [int] NOT NULL,
	[flags] [int] NOT NULL,

	/* [strAddress] = [ipaddress],[subnet mask],[mac address],[scalability server name] */
	[strAddress] [nvarchar](500)  COLLATE  !!insensitive NOT NULL,
	[auto_rep_version] [timestamp] NULL
) ON [PRIMARY]
GO

ALTER TABLE [dbo].[urc_computer_addr] WITH NOCHECK ADD 
	CONSTRAINT [XPKurc_comp_addr] PRIMARY KEY  CLUSTERED 
	(
		[uuid]
	)  ON [PRIMARY] 
GO
CREATE INDEX [urc_computer_addr_x0] ON [dbo].[urc_computer_addr]([computer_uuid],[domain_uuid]) ON [PRIMARY]
GO


ALTER TABLE urc_computer_addr
       ADD CONSTRAINT urc_computer_addr_fk0
              FOREIGN KEY (computer_uuid)
                             REFERENCES urc_computer  (uid)
GO
ALTER TABLE urc_computer_addr
       ADD CONSTRAINT urc_computer_addr_fk1
              FOREIGN KEY (domain_uuid)
                             REFERENCES ca_n_tier  (domain_uuid)
GO

GRANT  SELECT, insert, delete, update  ON [dbo].[urc_computer_addr]  TO [ca_itrm_group]
go
GRANT  SELECT  ON [dbo].[urc_computer_addr]  TO [upmuser_group]
go
GRANT  SELECT  ON [dbo].[urc_computer_addr]  TO [ca_itrm_group_ams]
go




/****** Object:  Stored Procedure dbo.p_urc_computer_deleted    Script Date: 12/1/2005 11:50:11 PM ******/



/*
 ***************************************************
 after deletion of an urc_computer, 
 can only happen at the domain for domain entries
 ***************************************************
 */

alter procedure [dbo].[p_urc_computer_deleted]
	@uid binary(16)
as
begin

	declare	@own_domain_uuid binary(16);
	declare @domain_uuid binary(16);
	
	/* retrieve own domain_uuid */
	set @own_domain_uuid = (select set_val_uuid
		  					from ca_settings
							where set_id = 1)

	/*
	Delete the corresponding urc_ab_computer entry
	*/	
	delete from urc_ab_computer where (uid = @uid and domain_uuid = @own_domain_uuid); 


	/*
	Delete the corresponding entry in urc_computer_addr
	*/
	delete from urc_computer_addr where (computer_uuid = @uid);

	/*	
	Update the parents:
	*/
	execute p_urc_ab_update_parents @uid, 0x00000000000000000000000000000000, 1, @own_domain_uuid;
	
	/*
	Delete the corresponding member entries,
	after the update of the parents !!
	else they won't be anymore there
	*/	
	delete from urc_ab_group_member where (member_uuid = @uid and member_domain_uuid = @own_domain_uuid);

end
GO



/****** Object:  Stored Procedure dbo.p_urc_computer_updated    Script Date: 12/1/2005 11:50:19 PM ******/



/*
 ***************************************************
 after update (streamaddress) on urc_computer
 used to track changes / handover of changes made 
 during the registration of an urc computer
 ***************************************************
 */

alter procedure [dbo].[p_urc_computer_updated]
	@uid binary(16),
	@streamaddress binary(510)
as
begin

	declare	@own_domain_uuid binary(16);
	declare @domain_uuid binary(16);
	
	/* retrieve own domain_uuid */
	set @own_domain_uuid = (select set_val_uuid
		  					from ca_settings
							where set_id = 1)

	/* retrieve the computer's domain uuid */
	set @domain_uuid = (select domain_uuid from ca_discovered_hardware where dis_hw_uuid = @uid);

	/* execute procedure only for own domain_uuid */
	if (@own_domain_uuid = @domain_uuid)
	begin

		/*
		Update the corresponding urc_ab_computer entry
		*/	
		update urc_ab_computer set connection_information = @streamaddress 
							   where (uid = @uid and domain_uuid = @own_domain_uuid); 
		/*	
		update the parents
		*/
		execute p_urc_ab_update_parents @uid, 0x00000000000000000000000000000000, 1, @own_domain_uuid;

	end
	
end
GO




/****** Object:  Trigger dbo.r_u_urc_computeraddr ******/


/*
 **********************************************************
	trigger on after update (streamaddress)
	on urc_computer_addr to track all changes made 
	during the registration of an urc computer
 **********************************************************
 */

CREATE trigger [r_u_urc_computer_addr] 
	
on [dbo].[urc_computer_addr]
	after insert,update
as
begin
	declare @computer_uid binary(16);
	declare @streamaddress binary(510);

	declare cursor_urc_computer_addr cursor local for 
		select computer_uuid from inserted

	open cursor_urc_computer_addr;
	fetch next from cursor_urc_computer_addr into @computer_uid; -- get first 
	while @@fetch_status = 0
	begin
		set @streamaddress = 0x00;
		execute p_urc_computer_updated @computer_uid, @streamaddress;
		fetch next from cursor_urc_computer_addr into @computer_uid; -- get next

	end; -- while
	close cursor_urc_computer_addr;
	deallocate cursor_urc_computer_addr;

end
GO

CREATE trigger [r_d_urc_computer_addr] 
	
on [dbo].[urc_computer_addr]
	after delete
as
begin
	declare @computer_uid binary(16);
	declare @streamaddress binary(510);

	declare cursor_urc_computer_addr cursor local for 
		select computer_uuid from deleted

	open cursor_urc_computer_addr;
	fetch next from cursor_urc_computer_addr into @computer_uid; -- get first 
	while @@fetch_status = 0
	begin
		set @streamaddress = 0x00;
		execute p_urc_computer_updated @computer_uid, @streamaddress;
		fetch next from cursor_urc_computer_addr into @computer_uid; -- get next

	end; -- while
	close cursor_urc_computer_addr;
	deallocate cursor_urc_computer_addr;

end
GO



/****************************************************************************************/
/*                                                                                      */
/* Register patch                                                               	*/
/*                                                                                      */
/****************************************************************************************/
insert into mdb_patch ( patchnumber, installdate,  mdbmajorversion, mdbminorversion, description)
 values ( 16255152, getdate(), 1, 4, 'Star 16255152 DSM: add urc_computer_addr for IPV6' )
GO

COMMIT TRANSACTION 
GO

