
/*
 **************************************************
 **************************************************
 * install URC specific database definitions
 * for an E N T E R P R I S E manager only !!
 **************************************************
 **************************************************
 */

/*
 ***************************************************
	an urc_ab_computer entry has been replicated up
 ***************************************************
 */

drop procedure p_urc_ep_urc_ab_computer_repl
go

create procedure p_urc_ep_urc_ab_computer_repl 
	@uid binary(16),
	@domain_uuid binary(16) 
as
begin

	declare @group_uuid binary(16);
	declare @objCnt integer;
	declare @own_domain_uuid binary(16);


	/* retrieve own domain_uuid */
	set @own_domain_uuid = (select set_val_uuid
							from ca_settings
							where set_id = 1)

	/* execute procedure only for other domain_uuid's */
	if (@own_domain_uuid != @domain_uuid)
	begin

		/*
		take all group_uuids to the member_uuid = uid and 
		group_domain_uuid = _own_domain_uuid in ca_group_member table
		
		check if the group_uuid exists in urc_ab_group_def
		if yes then 
		create urc_ab_computer entry for the enterprise
		*/
		
		declare cursor_ca_group_member cursor local
		for 
			select group_uuid
			from ca_group_member
			where member_uuid = @uid and group_domain_uuid = @own_domain_uuid
			
		open cursor_ca_group_member
		-- get group_uuid
		fetch next from cursor_ca_group_member into @group_uuid
		while @@fetch_status = 0
		begin
			
			/* check if the group_uuid exists in urc_ab_group_def */
			set @objCnt = 0;
			set rowcount 1;
			set @objCnt = (select count(*) from urc_ab_group_def where uid = @group_uuid);
			set rowcount 0;
			
			if (@objCnt > 0) 
			begin
				/* create urc_ab_computer entry with the enterprise domain_uuid */
				insert into urc_ab_computer (uid, domain_uuid, label, 
									host_name, connection_information, host_uuid)
									(select uid, @own_domain_uuid, label, 
									host_name, connection_information, host_uuid 
									from urc_ab_computer
									where uid = @uid and domain_uuid = @domain_uuid);
				/* --> p_urc_ab_computer_created is executed */
				/* --> that will add the member entry and    */
				/*     update the parents                    */
				
				break;
				
			end

			fetch next from cursor_ca_group_member into @group_uuid
		end	
		close cursor_ca_group_member
		deallocate cursor_ca_group_member

	end
end
go

grant execute on p_urc_ep_urc_ab_computer_repl to ca_itrm_group


/*
 ***********************************************************
 trigger: an urc_ab_computer entry has been replicated up
 ***********************************************************
 */

drop trigger r_urc_ep_i_urc_ab_computer
go

create trigger r_urc_ep_i_urc_ab_computer
	on urc_ab_computer
	after insert
as
begin
    declare @uid binary(16);
    declare @domain_uuid binary(16);
    
    select @uid = (select uid from inserted);
    select @domain_uuid = (select domain_uuid from inserted);
    
	execute p_urc_ep_urc_ab_computer_repl @uid, @domain_uuid;
end;												    
go


/*
 ***********************************************
 an entry in urc_ab_group_member is deleted	
 ***********************************************
 */

drop procedure p_urc_ep_urc_ab_group_member_del
go
													   
CREATE procedure p_urc_ep_urc_ab_group_member_del
	@member_uuid binary(16),
	@group_uuid binary(16),
	@group_domain_uuid binary(16),
	@member_type integer
											      
as
begin

    declare @urcObjCnt integer;

	/* check if there exists another urc_ab_group_member entry for the member_uuid */
	set @urcObjCnt = 0;
	set rowcount 1;
	set @urcObjCnt = (select count(*) from urc_ab_group_member 
		                              where member_uuid = @member_uuid);
	set rowcount 0;
		
	if( @urcObjCnt = 0)
	begin 
		/* no other member entry exists */
		delete from urc_ab_computer where uid = @member_uuid and domain_uuid = @group_domain_uuid;
		delete from urc_computer_addr where computer_uuid=@member_uuid and domain_uuid = @group_domain_uuid;
	end;

end;
go
grant execute on p_urc_ep_urc_ab_group_member_del to ca_itrm_group
go


/*
 ***********************************************
 trigger: an entry in urc_ab_group_def is deleted	
 ***********************************************
 */

drop trigger r_urc_ep_d_urc_ab_group_member
go

create trigger r_urc_ep_d_urc_ab_group_member
	 on urc_ab_group_member
	 after delete
as
begin 
    declare @member_uuid binary(16);
	declare @group_uuid  binary(16);
	declare @group_domain_uuid binary(16);
	declare @member_type integer;

	declare cursor_urc_ab_group_member cursor local for 
		select member_uuid, group_uuid, group_domain_uuid, member_type from deleted

	open cursor_urc_ab_group_member;
	fetch next from cursor_urc_ab_group_member into @member_uuid, @group_uuid, @group_domain_uuid, @member_type; -- get first 
	while @@fetch_status = 0
	begin
	
		execute  ca_itrm.p_urc_ep_urc_ab_group_member_del @member_uuid, @group_uuid, @group_domain_uuid, @member_type;
			
		fetch next from cursor_urc_ab_group_member into @member_uuid, @group_uuid, @group_domain_uuid, @member_type;

	end; -- while
	close cursor_urc_ab_group_member;
	deallocate cursor_urc_ab_group_member;
	
end;	
go


/*
 ********************************************************
 an entry in urc_ab_computer is deleted	(via replication)
 ********************************************************
 */

drop procedure p_urc_ep_urc_ab_computer_deleted
go
														   
create procedure p_urc_ep_urc_ab_computer_deleted 
	@uid binary(16),
	@domain_uuid binary(16)
											      
as
begin

	declare	@own_domain_uuid binary(16);

	/* retrieve own domain_uuid */
	select @own_domain_uuid = (select set_val_uuid from ca_settings	where set_id = 1);

	/* execute procedure only for replicated computer entries, */
	/* these are the entries with another domain_uuid          */
	if (@own_domain_uuid != @domain_uuid)
	begin
		/*
		Delete the corresponding urc_ab_computer entry of the enterprise
		*/	
		delete from urc_ab_computer
			   where uid = @uid and domain_uuid =@own_domain_uuid; 
		/*	
		Update the parents:
		*/
		execute p_urc_ab_update_parents @uid, 0x00000000000000000000000000000000, 1, @own_domain_uuid;
		
		/*
		Delete the corresponding member entries,
		after the update of the parents !!
		else they won't be anymore there
		*/	
		delete from urc_ab_group_member
			where member_uuid = @uid and member_domain_uuid = @own_domain_uuid;

	end;

end;
go

grant execute on p_urc_ep_urc_ab_computer_deleted to ca_itrm_group
go


/*
 *******************************************************************
 trigger: an entry in urc_ab_computer is deleted (via replication)
 *******************************************************************
 */

drop trigger r_urc_ep_d_urc_ab_computer
go

create trigger r_urc_ep_d_urc_ab_computer
	 on urc_ab_computer
	 after delete
as
begin 
    declare @uid binary(16);
    declare @domain_uuid binary(16);
												       
	declare cursor_urc_ab_computer cursor local for 
		select uid, domain_uuid from deleted

	open cursor_urc_ab_computer;
	fetch next from cursor_urc_ab_computer into @uid, @domain_uuid; -- get first 
	while @@fetch_status = 0
	begin
	
		execute  ca_itrm.p_urc_ep_urc_ab_computer_deleted @uid, @domain_uuid;
			
		fetch next from cursor_urc_ab_computer into @uid, @domain_uuid;

	end; -- while
	close cursor_urc_ab_computer;
	deallocate cursor_urc_ab_computer;

end;
go



/*
 ********************************************************
 an entry in urc_ab_computer is updated	(via replication)
 ********************************************************
 */
														   
drop procedure p_urc_ep_urc_ab_computer_updated
go

create procedure p_urc_ep_urc_ab_computer_updated 
	@uid binary(16),
    @domain_uuid binary(16)
											      
as
begin

    declare	@own_domain_uuid binary(16);
	declare	@label nvarchar(255);
	declare	@host_name nvarchar(255);
	declare	@connection_information binary(510);
	declare	@host_uuid char(36);


	/* retrieve own domain_uuid */
	select @own_domain_uuid = (select set_val_uuid
		                       from ca_settings
		                       where set_id = 1);

	/* execute procedure only for replicated computer entries, */
	/* these are the entries with another domain_uuid          */
	if (@own_domain_uuid != @domain_uuid)
	begin

		/*
		Update the urc_ab_computer entry of the enterprise
		with the values of the domain entry
		*/	
		

        set @label = (select label from urc_ab_computer
                	  where uid = @uid and domain_uuid = @domain_uuid);

        set @host_name = (select host_name from urc_ab_computer
                	 	  where uid = @uid and domain_uuid = @domain_uuid);

        set @connection_information = (select  connection_information  from urc_ab_computer
                					   where uid = @uid and domain_uuid = @domain_uuid);

        set @host_uuid= (select host_uuid from urc_ab_computer
                		 where uid = @uid and domain_uuid = @domain_uuid);


		update urc_ab_computer set label = @label, 
	  							   host_name = @host_name,
								   connection_information = @connection_information,
								   host_uuid = @host_uuid
								   where uid = @uid and domain_uuid =@own_domain_uuid;
		/*	
		update the parents
		*/
		execute  p_urc_ab_update_parents @uid, 0x00000000000000000000000000000000, 1, @own_domain_uuid;
												  
	end;

end;
go

grant execute on p_urc_ep_urc_ab_computer_updated to ca_itrm_group
go


/*
 *******************************************************************
 trigger: an entry in urc_ab_computer is updated (via replication)
 *******************************************************************
 */

/**
 * trigger after update urc_ab_computer
 * after update (label, host_name, connection_information, securityproviders, host_uuid)
 */

drop trigger r_urc_ep_u_urc_ab_computer
go

create trigger r_urc_ep_u_urc_ab_computer
	  on urc_ab_computer
	  after update
as
begin
	 -- we need to specify the columns, else it will be updated in a loop because of the
	 -- auto_rep_version update 
	   
    if update(label) or update(host_name) or update (connection_information)
        or update(securityproviders) or update(host_uuid)
    begin    
        declare @uid binary(16);
    	declare @domain_uuid binary(16);
    	  
		declare cursor_urc_ab_computer cursor local for 
			select uid, domain_uuid from inserted
	    
		open cursor_urc_ab_computer;
		fetch next from cursor_urc_ab_computer into @uid, @domain_uuid; -- get first 
		while @@fetch_status = 0
		begin
		
    		execute p_urc_ep_urc_ab_computer_updated @uid, @domain_uuid ;
			fetch next from cursor_urc_ab_computer into @uid, @domain_uuid;

		end; -- while
		close cursor_urc_ab_computer;
		deallocate cursor_urc_ab_computer;

    end;
end
go

													   
