@Echo off

REM
REM Name: PatchMDB.bat 
REM
REM Description:	Patch an MDB instance in Oracle
REM 
REM Syntax:
REM
REM 	PatchMDB -ORA_CONNECT_ID=xxxx [-MDB_NAME=xxxx] -MDB_ADMIN_PSWD=xxxx 
REM				[-ORA_INDEX_TBLSPACE_NAME=xxxx] -MDB_PATCH=xxxx 
REM                             [-ORA_SYS_USER=xxxx] [-ORA_SYS_PSWD=xxxx]
REM				[-MDB_TARGET_DIR=<dir>] [-ORA_BACKUP=xxxx]
REM  				[-FORCE_PATCH=<yes or no>] [-debug] [-?]
REM
REM History:
REM	23-jan-2006 mulwi01
REM	    Created
REM	24-feb-2006 mulwi01
REM	    added undo to remove patches; also undo patch on error
REM	 3-mar-2006 mulwi01
REM	    added RMAN backup option
REM	17-mar-2006 ricle02
REM	    A check is now performed for the existence of the index tablespace
REM	    before the patch proceeds.
REM	19-apr-2006 ricle02
REM	   Disabled the UNDO functionality
REM     18-aug-2006 rigka01, STAR issue 15148450, problem MDB21
REM	   Log the PatchMDB command 
REM	20-Mar-2007 rigka01, STAR issue 14729038, STAR problem MDB 65
REM	   Replace use of relative path
REM     22-Mar-2007 todjo06, connect to the database using a privileged
REM                          user instead of mdbadmin in specific cases
REM     1-May-2007 todjo06 adjusted usage to include ORA_SYS_USER and
REM                          ORA_SYS_PSWD
REM     9-May-2007 todjo06, removed references to ORA_NOSYSDBA
REM    10-May-2007 todjo06, move ORA_SYS_USER/ORA_SYS_PSWD message
REM                           to after log initialization
REM    2-Jul-2007 todjo06, improve message about archivelog mode
REM    21-Sep-2007 STAR 16382371, MDB 84 - blank in patch path fails
REM    24-Oct-2007 rigka01 Star 16477216 MDB 122 Hide any passwords when echoing/logging the Command line
REM    09-Nov-2007 rigka01 Star 16529704 MDB 123 Allow RMAN to work for Oracle net authorization not just OS authorization
REM    05-Feb-2008 rigka01 STAR 16815138 MDB 137 Remove echoes that write
REM                        MDB_ADMIN_PSWD to the install_mdb.log
REM    03-Mar-2008 rigka01 STAR 16898454 MDB 144 Add -FORCE_PATCH option to
REM                        allow re-run with savem value for MDB_PATCH
REM    04-Apr-2008 rigka01 Fix up messages
REM    25-Apr-2008 rigka01 STAR 17072813 MDB 162 patchmdb passwords fail when
REM                        they contain an exclamation point
REM    20-May-2008 rigka01 STAR 17131540 MDB 167 - MDB_TARGET_DIR path exists but path contains special characters 
REM    06-Jun-2008 rigka01 Back-out STAR 17131540 MDB 167 due to problem it introduced when MDB_TARGET_DIR includes a blank 
REM    06-Jun-2008 rigka01 New fix for STAR 17131540 MDB 167
REM    22-Aug-2008 hopst01 STAR 17393114 PATCH MDB PSWORD SPECIAL CHAR, Allow special characters in password by double quoting the password
REM    09-Sep-2008 rigka01 and hopst01 STAR 17393114, MDB 182 simplify prev fix
REM    05-dec-2008 hopst01 17727007 - FOR LOOP W CMD NOT SUPPORTED. For loops that use 
REM                        a command to populate the for loop set should be replaced. 
REM
REM Start localization of environment variables in a batch file
REM
SETLOCAL

echo %date% %time% Started PatchMDB
set PATCHMDB_COMMAND=%~f0 %*
set PATCHMDB_VERSION=45
set PATCHMDB_PATH=%~d0%~p0
echo %date% %time% PatchMDB version: %PATCHMDB_VERSION% 

SETLOCAL ENABLEDELAYEDEXPANSION

REM set DEBUGECHO=@ECHO on
set DEBUGECHO=@ECHO off
%DEBUGECHO%

set PATH_CMD_NO_ARGS=%~f0
REM Hide MDB_ADMIN_PSWD if it is in the command; keep the rest as-is
set COMMAND_HIDDEN_PSWDS=
set TEST_ARGS_FOR_MDB_ADMIN_PSWD=%*
set HAS_PSWD=false
set EVEN_ARG=yes
for  %%a in (%TEST_ARGS_FOR_MDB_ADMIN_PSWD%) do (
  if !HAS_PSWD! EQU true ( 
        set COMMAND_HIDDEN_PSWDS=!COMMAND_HIDDEN_PSWDS!=
        if !EVEN_ARG! EQU no (
          set EVEN_ARG=yes
	) else (
	  set EVEN_ARG=no
	)
        set COMMAND_HIDDEN_PSWDS=!COMMAND_HIDDEN_PSWDS!xxxxxxxx
  	set HAS_PSWD=false
  ) else (
        if !EVEN_ARG! EQU no (
          set COMMAND_HIDDEN_PSWDS=!COMMAND_HIDDEN_PSWDS!=
          set COMMAND_HIDDEN_PSWDS=!COMMAND_HIDDEN_PSWDS!%%a
          set EVEN_ARG=yes
        )  else (
          set EVEN_ARG=no
          set COMMAND_HIDDEN_PSWDS=!COMMAND_HIDDEN_PSWDS! %%a
  	  if "%%a" == "-MDB_ADMIN_PSWD" ( 
	     set HAS_PSWD=true
          )
  	)
  )
)
REM Hide ORA_SYS_PSWD if it is in the command; keep the rest as-is
set TEST_ARGS_FOR_ORA_SYS_PSWD=%COMMAND_HIDDEN_PSWDS%
set COMMAND_HIDDEN_PSWDS=
set HAS_PSWD=false
set EVEN_ARG=yes
for  %%a in (%TEST_ARGS_FOR_ORA_SYS_PSWD%) do (
  if !HAS_PSWD! EQU true ( 
        set COMMAND_HIDDEN_PSWDS=!COMMAND_HIDDEN_PSWDS!=
        if !EVEN_ARG! EQU no (
          set EVEN_ARG=yes
	) else (
	  set EVEN_ARG=no
	)
        set COMMAND_HIDDEN_PSWDS=!COMMAND_HIDDEN_PSWDS!xxxxxxxx
  	set HAS_PSWD=false
  ) else (
        if !EVEN_ARG! EQU no (
          set COMMAND_HIDDEN_PSWDS=!COMMAND_HIDDEN_PSWDS!=
          set COMMAND_HIDDEN_PSWDS=!COMMAND_HIDDEN_PSWDS!%%a
          set EVEN_ARG=yes
        )  else (
          set EVEN_ARG=no
          set COMMAND_HIDDEN_PSWDS=!COMMAND_HIDDEN_PSWDS! %%a
  	  if "%%a" == "-ORA_SYS_PSWD" ( 
	     set HAS_PSWD=true
          )
  	)
  )
)
SETLOCAL DISABLEDELAYEDEXPANSION

echo %date% %time% PatchMDB command (with full path, without passwords): %PATH_CMD_NO_ARGS% %COMMAND_HIDDEN_PSWDS%

set CURDIR=%CD%

REM
REM Set the default parameters
REM
set ORA_CONNECT_ID=
set MDB_NAME=MDB
set MDB_ADMIN_PSWD=
set MDB_PATCH=
set MDB_TARGET_DIR=%CURDIR%
set UNDO_PATCH=
set ORA_BACKUP=yes
set ORA_SYS_USER=
set ORA_SYS_PSWD=
set USE_OS_AUTH=0
set ORA_SID_ERR=0
set ORA_SYS_PATCH=0
set MDB_RC=2
set ERR_MESSAGE=Patch MDB error
set ORA_AS_SYSDBA= as sysdba

set PATCH_PREFIX=%PATCHMDB_PATH%\patches\p


REM
REM Parse the command line parameters
REM

:PARSE_ARGS

IF "%~1"==""   GOTO NO_MORE_ARGS
IF "%~1"=="-?" GOTO USAGE

IF /I NOT "%~1"=="-ORA_CONNECT_ID" GOTO NEXT_ARG1
SHIFT
set ORA_CONNECT_ID=%~1
SHIFT
if defined ORACLE_SID GOTO PARSE_ARGS
if (%ORA_CONNECT_ID%) NEQ (%%ORACLE_SID%%) GOTO PARSE_ARGS
set ORA_SID_ERR=1
GOTO SET_LOG

:NEXT_ARG1
IF /I NOT "%~1"=="-MDB_PATCH" GOTO NEXT_ARG2
SHIFT
set MDB_PATCH=%~1
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG2
IF /I NOT "%~1"=="-MDB_NAME" GOTO NEXT_ARG3
SHIFT
set MDB_NAME=%~1
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG3
IF /I NOT "%~1"=="-MDB_ADMIN_PSWD" GOTO NEXT_ARG4
SHIFT
set MDB_ADMIN_PSWD="%~1"
rem undo "call" stmt
rem set MDB_ADMIN_PSWD=%MDB_ADMIN_PSWD:^^=^%
rem Escape any reserved shell characters
set MDB_ADMIN_PSWD=%MDB_ADMIN_PSWD:^=^^%
set MDB_ADMIN_PSWD=%MDB_ADMIN_PSWD:<=^<%
set MDB_ADMIN_PSWD=%MDB_ADMIN_PSWD:>=^>%
set MDB_ADMIN_PSWD=%MDB_ADMIN_PSWD:|=^|%
set MDB_ADMIN_PSWD=%MDB_ADMIN_PSWD:&=^&%
set MDB_ADMIN_PSWD=%MDB_ADMIN_PSWD:(=^(%
set MDB_ADMIN_PSWD=%MDB_ADMIN_PSWD:)=^)%
set MDB_ADMIN_PSWD=%MDB_ADMIN_PSWD:"=\"%
set MDB_ADMIN_PSWD_DQ="%~1"
rem undo "call" stmt
rem set MDB_ADMIN_PSWD_DQ=%MDB_ADMIN_PSWD_DQ:^^=^%
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG4
IF /I NOT "%~1"=="-MDB_TARGET_DIR" GOTO NEXT_ARG5
SHIFT
set MDB_TARGET_DIR=%~1
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG5
IF /I NOT "%~1"=="-ORA_INDEX_TBLSPACE_NAME" GOTO NEXT_ARG6
SHIFT
set ORA_INDEX_TBLSPACE_NAME="%~1"
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG6
IF /I NOT "%~1"=="-UNDO" GOTO NEXT_ARG7
echo %date% %time% ERROR -UNDO option no longer functional
GOTO USAGE
REM set UNDO_PATCH=undo
REM SHIFT

:NEXT_ARG7
IF /I NOT "%~1"=="-ORA_BACKUP" GOTO NEXT_ARG8
SHIFT
set ORA_BACKUP=%~1
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG8
IF /I NOT "%~1"=="-ORA_SYS_USER" GOTO NEXT_ARG9
SHIFT
set ORA_SYS_USER=%~1
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG9
IF /I NOT "%~1"=="-ORA_SYS_PSWD" GOTO NEXT_ARG10
SHIFT
set ORA_SYS_PSWD="%~1"
set ORA_SYS_PSWD_DQ="%~1"
rem Escape any reserved shell characters
set ORA_SYS_PSWD=%ORA_SYS_PSWD:^=^^%
set ORA_SYS_PSWD=%ORA_SYS_PSWD:<=^<%
set ORA_SYS_PSWD=%ORA_SYS_PSWD:>=^>%
set ORA_SYS_PSWD=%ORA_SYS_PSWD:|=^|%
set ORA_SYS_PSWD=%ORA_SYS_PSWD:&=^&%
set ORA_SYS_PSWD=%ORA_SYS_PSWD:(=^(%
set ORA_SYS_PSWD=%ORA_SYS_PSWD:)=^)%
set ORA_SYS_PSWD=%ORA_SYS_PSWD:"=\"%
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG10
IF /I NOT "%~1"=="-debug" GOTO NEXT_ARG11
set DEBUGECHO=@ECHO ON
%DEBUGECHO%
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG11
IF /I NOT "%~1"=="-FORCE_PATCH" GOTO INVALID_ARG
SHIFT
IF /I NOT "%~1"=="yes" GOTO FORCE_PATCH_CHECK_NO
set FORCE_PATCH=1
SHIFT
GOTO PARSE_ARGS
:FORCE_PATCH_CHECK_NO
IF /I NOT "%~1"=="no" GOTO FORCE_PATCH_ERR
set FORCE_PATCH=
SHIFT
GOTO PARSE_ARGS
:FORCE_PATCH_ERR
echo %date% %time% ERROR: Invalid -FORCE_PATCH argument "%~1", expected yes or no
set ERR_MESSAGE=Invalid -FORCE_PATCH argument "%~1", expected yes or no
GOTO USAGE

:INVALID_ARG
echo %date% %time% ERROR: Invalid argument "%~1"
set ERR_MESSAGE=Invalid PatchMdb argument "%~1"
GOTO USAGE

:NO_MORE_ARGS

rem
rem check required arguments
rem

IF (%ORA_CONNECT_ID%) NEQ () GOTO REQ_ARG1
echo %date% %time% ERROR: Missing required argument: ORA_CONNECT_ID 
set ERR_MESSAGE=Missing required argument: ORA_CONNECT_ID
GOTO USAGE

:REQ_ARG1
IF (%MDB_PATCH%) NEQ () GOTO REQ_ARG2
echo %date% %time% ERROR: Missing required argument: MDB_PATCH 
set ERR_MESSAGE=Missing required argument: MDB_PATCH
GOTO USAGE

:REQ_ARG2
IF (%MDB_ADMIN_PSWD%) NEQ () GOTO REQ_ARG3
echo %date% %time% ERROR: Missing required argument: MDB_ADMIN_PSWD 
set ERR_MESSAGE=Missing required argument: MDB_ADMIN_PSWD
GOTO USAGE

:REQ_ARG3
IF (%ORA_BACKUP%) EQU (yes) GOTO REQ_ARG4
IF (%ORA_BACKUP%) EQU (no)  GOTO REQ_ARG4
echo %date% %time% ERROR: Invalid ORA_BACKUP argument: %ORA_BACKUP% , expected yes or no
set ERR_MESSAGE=Invalid ORA_BACKUP argument: %ORA_BACKUP% , expected yes or no
GOTO USAGE

:REQ_ARG4
if (%ORA_SYS_USER%) EQU () GOTO NO_ORA_USER
if (%ORA_SYS_PSWD%) NEQ () GOTO REQ_ARG5
echo %date% %time% ERROR: Missing required argument: ORA_SYS_PSWD
set ERR_MESSAGE=Missing required argument: ORA_SYS_PSWD
GOTO USAGE

:NO_ORA_USER
set ORA_SYS_USER=sys

:REQ_ARG5
if (%ORA_SYS_PSWD%) NEQ () GOTO SET_DEFAULTS
set USE_OS_AUTH=1

:ARG5_MORE 
set ORACLE_SID=%ORA_CONNECT_ID%
set ORA_SYS_PSWD=xxxx

 
:SET_DEFAULTS
rem
rem set defaults
rem
IF (%ORA_INDEX_TBLSPACE_NAME%) NEQ () GOTO IDX_TBLSPACE_SET
set ORA_INDEX_TBLSPACE_NAME=%MDB_NAME%_INDEX

:IDX_TBLSPACE_SET
set MDB_INDEX_TBLSPACE_NAME=%ORA_INDEX_TBLSPACE_NAME%
rem
rem check if Oracle is available
rem
sqlplus -V > nul 2>&1
IF %ERRORLEVEL% NEQ 9009 (
    set SQL_CMD=sqlplus
    GOTO OSQL_FOUND
)
set MDB_RC=10
set ERR_MESSAGE=Cannot find sqlplus. Check that Oracle has been installed and in the PATH
GOTO DONE_EXIT
:OSQL_FOUND



set PATCH_MANIFEST="%PATCH_PREFIX%%MDB_PATCH%.txt"
IF EXIST %PATCH_MANIFEST% GOTO MANI_EXISTS
set MDB_RC=50
set ERR_MESSAGE=Missing patch manifest %PATCH_MANIFEST%
echo %date% %time% ERROR: %ERR_MESSAGE%
GOTO DONE_EXIT
:MANI_EXISTS

:SET_LOG

REM
REM Set the patch log file
REM
IF (%LOGFILE%) == () set LOGFILE="%MDB_TARGET_DIR%\install_%MDB_NAME%.log"
echo . >> %LOGFILE% 2>&1
echo . >> %LOGFILE% 2>&1
echo . >> %LOGFILE% 2>&1
echo %date% %time% PatchMDB mdb log file for %MDB_PATCH% in %LOGFILE%
echo %date% %time% ====================================================================== >> %LOGFILE% 2>&1
echo %date% %time% PatchMDB mdb log file for %MDB_PATCH% in %LOGFILE% >> %LOGFILE% 2>&1

REM
REM Log the full PatchMDB command
REM
echo %date% %time% PatchMDB command (with full path, without passwords): %PATH_CMD_NO_ARGS% %COMMAND_HIDDEN_PSWDS% >> %LOGFILE% 2>&1
echo %date% %time% PatchMDB version: %PATCHMDB_VERSION% >> %LOGFILE% 2>&1

if (%ORA_SID_ERR%) EQU (0) GOTO SID_OK 
set MDB_RC=55
set ERR_MESSAGE=Invalid ORA_CONNECT_ID, ORACLE_SID variable not set
GOTO DONE_EXIT

:SID_OK

echo quit | %SQL_CMD% -L "MDBADMIN/%MDB_ADMIN_PSWD%@%ORA_CONNECT_ID%" > nul 2>&1
if (%ERRORLEVEL%) EQU (0) GOTO MDBADMIN_OK
echo quit | %SQL_CMD% -L "MDBADMIN/%MDB_ADMIN_PSWD%@%ORA_CONNECT_ID%"
echo quit | %SQL_CMD% -L "MDBADMIN/%MDB_ADMIN_PSWD%@%ORA_CONNECT_ID%" >> %LOGFILE%
set MDB_RC=20
set ERR_MESSAGE=Could not connect to Oracle using mdbadmin user
GOTO DONE_EXIT

:MDBADMIN_OK
FOR /F "eol=- tokens=1,2,3,4,5,6 delims=, usebackq" %%i in (%PATCH_MANIFEST%) do IF "%%k" == "sys" set ORA_SYS_PATCH=1

if (%ORA_SYS_PATCH%) EQU (0) GOTO AUTH_OK
if (%USE_OS_AUTH%) EQU (0) GOTO AUTH_CHECK2

echo quit | %SQL_CMD% -L "/ as sysdba" > nul 2>&1
if (%ERRORLEVEL%) EQU (0) GOTO AUTH_OK

echo quit | %SQL_CMD% -L "/" > nul 2>&1
if (%ERRORLEVEL%) EQU (0) GOTO SET_AUTH2

set ERR_MESSAGE=This patch requires the use of the ORA_SYS_USER and ORA_SYS_PSWD parameters
set MDB_RC=30
GOTO DONE_EXIT

:AUTH_CHECK2

if (%ORA_SYS_USER%) EQU (SYS) GOTO SYS_USER
if (%ORA_SYS_USER%) EQU (sys) GOTO SYS_USER
set ORA_AS_SYSDBA=

:SYS_USER

echo quit | %SQL_CMD% -L "%ORA_SYS_USER%/%ORA_SYS_PSWD%@%ORA_CONNECT_ID% %ORA_AS_SYSDBA%" > nul 2>&1
if (%ERRORLEVEL%) EQU (0) GOTO AUTH_OK
echo quit | %SQL_CMD% -L "%ORA_SYS_USER%/%ORA_SYS_PSWD%@%ORA_CONNECT_ID% %ORA_AS_SYSDBA%" 
echo quit | %SQL_CMD% -L "%ORA_SYS_USER%/%ORA_SYS_PSWD%@%ORA_CONNECT_ID%" >> %LOGFILE%
set MDB_RC=40
set ERR_MESSAGE=Could not connect to Oracle using %ORA_SYS_USER% user
GOTO DONE_EXIT

:SET_AUTH2
set USE_OS_AUTH=2

:AUTH_OK

if (%USE_OS_AUTH%) EQU (0) GOTO CONSTR1
if (%USE_OS_AUTH%) EQU (1) GOTO CONSTR2
set CONNECT_STRING="/"
GOTO ALM_CHECK

:CONSTR1
set CONNECT_STRING="%ORA_SYS_USER%/%ORA_SYS_PSWD%@%ORA_CONNECT_ID%%ORA_AS_SYSDBA%"
GOTO ALM_CHECK

:CONSTR2
set CONNECT_STRING="/ as sysdba"

:ALM_CHECK
%SQL_CMD% -L "%CONNECT_STRING%" @patches/mdb_alm.sql > nul 2>&1
FOR /F %%i in (alm.lst) do IF (%%i) EQU (NOARCHIVELOG) GOTO BAD_ALM
GOTO ALM_OK

:BAD_ALM
if (%ORA_BACKUP%) EQU (yes) GOTO ALM_MESG1
echo %date% %time% WARNING: It is recommended that the database be in archivelog mode.
echo %date% %time% WARNING: It is recommended that the database be in archivelog mode. >> %LOGFILE%
GOTO ALM_OK

:ALM_MESG1
echo %date% %time% ERROR: The database instance must be in archivelog mode in order to backup.
echo %date% %time% Consult the Oracle documentation on how to put the database into Archivelog mode.
echo %date% %time% -ORA_BACKUP=no may be used to avoid taking a backup but is not recommended.
set MDB_RC=45
set ERR_MESSAGE=The database instance must be in archivelog mode in order to backup.
GOTO DONE_EXIT

:ALM_OK

REM
REM Check that the mdb version matches the patch requirement
REM
echo %date% %time% Checking the mdb version
echo %date% %time% Checking the mdb version >> %LOGFILE% 2>&1

FOR /F "eol=- tokens=1,2 delims=, usebackq" %%i in (%PATCH_MANIFEST%) do IF %MDB_PATCH% EQU %%i set PATCH_BUILD=%%j

%SQL_CMD% -L "MDBADMIN/%MDB_ADMIN_PSWD%@%ORA_CONNECT_ID%" @"%PATCHMDB_PATH%\patches\mdb_build.sql" >>%LOGFILE% 2>&1
set MDB_BUILD=%ERRORLEVEL%
IF (%MDB_BUILD%) EQU (%PATCH_BUILD%0000) GOTO BUILD_EXIST
set MDB_RC=60
set ERR_MESSAGE=Error %MDB_BUILD%: database %ORA_CONNECT_ID% build mismatch. Patch requires build %PATCH_BUILD%
GOTO DONE_EXIT
:BUILD_EXIST

REM
REM Check if index tablespace exists
REM
echo %date% %time% Checking if index tablspace %MDB_INDEX_TBLSPACE_NAME% exists
echo %date% %time% Checking if index tablspace %MDB_INDEX_TBLSPACE_NAME% exists >> %LOGFILE% 2>&1
%SQL_CMD% -L "MDBADMIN/%MDB_ADMIN_PSWD%@%ORA_CONNECT_ID%" @"%PATCHMDB_PATH%\patches\mdb_tablespace.sql" %MDB_INDEX_TBLSPACE_NAME% >>%LOGFILE% 2>&1
set RC=%ERRORLEVEL%
IF %RC% EQU 0 GOTO MDB_TBLSPACE_NOT_FOUND
IF %RC% EQU 1 GOTO MDB_TBLSPACE_EXISTS
set MDB_RC=100
set ERR_MESSAGE=%SQL_CMD% command error, ERRORLEVEL=%RC%, see logfile for details
GOTO DONE_EXIT

:MDB_TBLSPACE_NOT_FOUND
set MDB_RC=102
set ERR_MESSAGE=Tablespace %MDB_INDEX_TBLSPACE_NAME% does not exist. Use -MDB_NAME or -ORA_INDEX_TBLSPACE_NAME to specify non default names.
GOTO DONE_EXIT

:MDB_TBLSPACE_EXISTS
REM
REM Check if -FORCE_PATCH is specified
REM
if (%FORCE_PATCH%) EQU (1) GOTO SKIP_CHECK_MDB_PATCH

:DO_CHECK_MDB_PATCH
REM
REM Check if the patch already exists
REM
echo %date% %time% Checking if the patch already exists 
echo %date% %time% Checking if the patch already exists >> %LOGFILE%

%SQL_CMD% -L "MDBADMIN/%MDB_ADMIN_PSWD%@%ORA_CONNECT_ID%" @"%PATCHMDB_PATH%\patches\mdb_patch.sql" %MDB_PATCH% >>%LOGFILE% 2>&1
set RC=%ERRORLEVEL%
IF (%RC%) GEQ (10000) GOTO PATCH_EXISTS
IF (%RC%) EQU (0)     GOTO PATCH_NOT_EXIST
set MDB_RC=70
set ERR_MESSAGE=Return code %RC%: Unable to determine if database %ORA_CONNECT_ID% contains patch %MDB_PATCH% 
GOTO DONE_EXIT

:PATCH_EXISTS
IF (%UNDO_PATCH%) EQU (undo) GOTO REMOVE_PATCH
set MDB_RC=0
set ERR_MESSAGE=Database %ORA_CONNECT_ID% already contains patch %MDB_PATCH%
GOTO DONE_EXIT

:PATCH_NOT_EXIST
IF (%UNDO_PATCH%) EQU () GOTO BACKUP_DB
set MDB_RC=75
set ERR_MESSAGE=Database %ORA_CONNECT_ID% does not contain patch %MDB_PATCH%
GOTO DONE_EXIT

:SKIP_CHECK_MDB_PATCH

:BACKUP_DB
IF (%ORA_BACKUP%) NEQ (yes) GOTO END_BACKUP
echo %date% %time% Backing up the database...
echo %date% %time% Backing up the database... >> %LOGFILE%
if (%USE_OS_AUTH%) EQU (0) GOTO NO_OS_AUTH
set RMAN_CONNECT_STRING="/@%ORA_CONNECT_ID%"
echo %date% %time% rman target=%RMAN_CONNECT_STRING% cmdfile="%PATCHMDB_PATH%\patches\mdb_backup.sql" >> %LOGFILE%
GOTO RUN_RMAN
:NO_OS_AUTH
set RMAN_CONNECT_STRING="%ORA_SYS_USER%/%ORA_SYS_PSWD%@%ORA_CONNECT_ID%"
echo %date% %time% rman target=%ORA_SYS_USER%/xxxxxxxx@%ORA_CONNECT_ID% cmdfile="%PATCHMDB_PATH%\patches\mdb_backup.sql" >> %LOGFILE%
:RUN_RMAN
rman target=%RMAN_CONNECT_STRING% cmdfile="%PATCHMDB_PATH%\patches\mdb_backup.sql" >>%LOGFILE% 2>&1
set RC=%ERRORLEVEL%
IF (%RC%) EQU (0)     GOTO END_BACKUP
set MDB_RC=78
set ERR_MESSAGE=Return code %RC%: RMAN backup failed on database %ORA_CONNECT_ID%, check %LOGFILE%
GOTO DONE_EXIT
:END_BACKUP




REM
REM Installing patch
REM
set MDB_RC_FILE=mdbpatcherr.txt
IF EXIST "%MDB_RC_FILE%" del "%MDB_RC_FILE%"
echo %date% %time% Installing the patch from the manifest...
echo %date% %time% Installing the patch from the manifest... >> %LOGFILE%

FOR /F "eol=- tokens=1,2,3,4,5,6 delims=, usebackq" %%i in (%PATCH_MANIFEST%) do call "%PATCHMDB_PATH%\installmdbpatchora" %ORA_CONNECT_ID% %MDB_ADMIN_PSWD_DQ%  %MDB_INDEX_TBLSPACE_NAME% "%PATCH_PREFIX%%%i.sql" %%i %USE_OS_AUTH% %ORA_SYS_USER% %ORA_SYS_PSWD_DQ% %%k



REM
REM Check that the patch was installed
REM
%SQL_CMD% -L "MDBADMIN/%MDB_ADMIN_PSWD%@%ORA_CONNECT_ID%" @"%PATCHMDB_PATH%\patches\mdb_patch.sql" %MDB_PATCH% >>%LOGFILE% 2>&1
set RC=%ERRORLEVEL%
IF (%RC%) EQU (10000) GOTO MDB_SIGNATURE
set UNDO_PATCH=error
set MDB_RC=90
set ERR_MESSAGE=Patch %MDB_PATCH% failed on database %MDB_NAME% 
echo %date% %time% ERROR: Return code %MDB_RC%, %ERR_MESSAGE% >> %LOGFILE% 2>&1
IF EXIST "%PATCH_PREFIX%%MDB_PATCH%_undo.sql" GOTO REMOVE_PATCH_ERR
GOTO DONE_EXIT





:MDB_SIGNATURE
REM
REM Create the mdb signature for asset/patch management
REM
chdir /D "%MDB_TARGET_DIR%"
IF EXIST "%PATCHMDB_PATH%\CreateMDBSignature.bat" call "%PATCHMDB_PATH%\CreateMDBSignature" -ORA_CONNECT_ID=%ORA_CONNECT_ID% -MDB_ADMIN_PSWD=%MDB_ADMIN_PSWD_DQ% -MDB_NAME=%MDB_NAME% -MDB_PATCH_DDL_DIR="%PATCHMDB_PATH%\patches" >>%LOGFILE% 2>&1
IF (%UNDO_PATCH%) EQU () GOTO COMPLETED
GOTO DONE_EXIT

	

:COMPLETED
set MDB_RC=0
set ERR_MESSAGE=Patch %MDB_PATCH% installed in database %ORA_CONNECT_ID% 
GOTO DONE_EXIT




:REMOVE_PATCH
set MDB_RC=0
set ERR_MESSAGE=Patch %MDB_PATCH% uninstalled in database %ORA_CONNECT_ID% 
:REMOVE_PATCH_ERR

call "%PATCHMDB_PATH%\installmdbpatchora" %ORA_CONNECT_ID% %MDB_ADMIN_PSWD_DQ%  %MDB_INDEX_TBLSPACE_NAME% "%PATCH_PREFIX%%MDB_PATCH%_undo.sql" %MDB_PATCH% undo
set RC=%ERRORLEVEL%
IF (%RC%) EQU (0) GOTO MDB_SIGNATURE
set MDB_RC=95
set ERR_MESSAGE=Undo patch %MDB_PATCH% failed on database %MDB_NAME% 
GOTO MDB_SIGNATURE




:USAGE
echo.
echo Usage: 
echo.
echo PatchMDB -ORA_CONNECT_ID=xxxx [-MDB_NAME=xxxx] -MDB_ADMIN_PSWD=xxxx -MDB_PATCH=xxxx [-MDB_TARGET_DIR=dir] [-ORA_BACKUP=xxxx] [-ORA_SYS_USER=xxxx] [-ORA_SYS_PSWD=xxxx] [-FORCE_PATCH=yes or no] [-debug] [-?]
echo. 
echo where: 
echo.
echo      ORA_CONNECT_ID    connection identifier (required)
echo      MDB_NAME          database name (optional with default mdb)
echo      MDB_ADMIN_PSWD    mdbadmin user password (required)
echo      ORA_INDEX_TBLSPACE_NAME name of the index tablespace (optional with default)	
echo      MDB_PATCH         patch number (required)
echo      MDB_TARGET_DIR    target mdb directory (optional with default current directory)
echo      ORA_BACKUP        yes for a backup or no (optional with default yes)
echo      ORA_SYS_USER      for patching using privileged user (optional with default sys) 
echo      ORA_SYS_PSWD      password of privileged user (required if not OS authorized)
echo      FORCE_PATCH       yes to force processing of manifest for MDB_PATCH (optional with default no)
echo      debug             echo output (optional with default no debug message echoing)
echo      ?                 echo this Usage message (optional with default no usage echoing)
echo. 
echo.

set MDB_RC=99
set ERR_MESSAGE=Usage error rc=99
GOTO DONE_EXIT


:DONE_EXIT
echo %date% %time% 
echo %date% %time% Return code %MDB_RC%, %ERR_MESSAGE%

IF (%LOGFILE%) == () goto NO_LOG
echo %date% %time% >> %LOGFILE% 2>&1
echo %date% %time% Return code %MDB_RC%, %ERR_MESSAGE% >> %LOGFILE% 2>&1
:NO_LOG

echo %date% %time% Setting directory back to "%CURDIR%" 
chdir /D "%CURDIR%"

EXIT /B %MDB_RC%
