SET ECHO ON;
WHENEVER OSERROR EXIT FAILURE;
WHENEVER SQLERROR EXIT SQL.SQLCODE;

SET AUTOCOMMIT OFF;


/****************************************************************************************/
/*                                                                                      */
/* Star 15032461 AUTOSYS: Duplicate EVT_NUM values					*/
/*                                                                                      */
/****************************************************************************************/

CREATE OR REPLACE PACKAGE ujo_det_rep_pkg
AS
    FUNCTION ujo_det_rep( 
    i_joid NUMBER ,
    i_run_num NUMBER,
    i_ostatus OUT NUMBER,
    i_oalarm OUT NUMBER,
    i_oevent OUT NUMBER,
    i_oevent_time_gmt OUT NUMBER,
    i_ontry OUT NUMBER, 
    i_oque_status OUT NUMBER, 
    i_omonth OUT NUMBER,
    i_oday OUT NUMBER,
    i_oyear OUT NUMBER,
    i_ohours OUT NUMBER,
    i_ominutes OUT NUMBER,
    i_oseconds OUT NUMBER,
    i_otext OUT VARCHAR2, 
    i_omachine OUT VARCHAR2,
    i_oevt_num OUT NUMBER,
    done_fetch OUT NUMBER,
    found OUT NUMBER) RETURN INTEGER;
END;
/


CREATE OR REPLACE PACKAGE BODY ujo_det_rep_pkg AS

    CURSOR ujo_get_det_rep(i_joid IN NUMBER, i_run_num IN NUMBER) IS
        SELECT status, 
            alarm, 
            event, 
            event_time_gmt, 
            ntry, 
            que_status,
            TO_NUMBER(TO_CHAR(que_status_stamp, 'MM')),
            TO_NUMBER(TO_CHAR(que_status_stamp, 'DD')),
            TO_NUMBER(TO_CHAR(que_status_stamp, 'YYYY')),
            TO_NUMBER(TO_CHAR(que_status_stamp, 'HH24')),
            TO_NUMBER(TO_CHAR(que_status_stamp, 'MI')),
            TO_NUMBER(TO_CHAR(que_status_stamp, 'SS')),
            text, 
            machine,
            evt_num
        FROM ujo_eventvu 
        WHERE joid = i_joid 
        AND run_num = i_run_num 
        UNION
        SELECT status, 
            alarm, 
            event, 
            event_time_gmt, 
            ntry, 
            que_status,
            TO_NUMBER(TO_CHAR(que_status_stamp, 'MM')),
            TO_NUMBER(TO_CHAR(que_status_stamp, 'DD')),
            TO_NUMBER(TO_CHAR(que_status_stamp, 'YYYY')),
            TO_NUMBER(TO_CHAR(que_status_stamp, 'HH24')),
            TO_NUMBER(TO_CHAR(que_status_stamp, 'MI')),
            TO_NUMBER(TO_CHAR(que_status_stamp, 'SS')), 
            text, 
            machine,
            evt_num
        FROM ujo_proc_eventvu 
        WHERE joid = i_joid 
        AND run_num = i_run_num 
        ORDER BY event_time_gmt, evt_num;
    
    FUNCTION ujo_det_rep( 
        i_joid NUMBER ,
        i_run_num NUMBER,
        i_ostatus OUT NUMBER,
        i_oalarm OUT NUMBER,
        i_oevent OUT NUMBER,
        i_oevent_time_gmt OUT NUMBER,
        i_ontry OUT NUMBER, 
        i_oque_status OUT NUMBER,
        i_omonth OUT NUMBER,
        i_oday OUT NUMBER,
        i_oyear OUT NUMBER,
        i_ohours OUT NUMBER,
        i_ominutes OUT NUMBER,
        i_oseconds OUT NUMBER,
        i_otext OUT VARCHAR2, 
        i_omachine OUT VARCHAR2,
        i_oevt_num OUT NUMBER,
        done_fetch OUT NUMBER,
        found OUT NUMBER)
    RETURN INTEGER AS
        temp_done_fetch INTEGER;
        temp_found INTEGER;
        temp_evt_num INTEGER;
    BEGIN
        IF NOT ujo_get_det_rep%ISOPEN THEN
            OPEN ujo_get_det_rep(i_joid, i_run_num);
        END IF;

        temp_done_fetch := 1;
        temp_found := 0;

        FETCH ujo_get_det_rep
        INTO i_ostatus,
            i_oalarm,
            i_oevent,
            i_oevent_time_gmt,
            i_ontry,
            i_oque_status,
            i_omonth,
            i_oday,
            i_oyear,
            i_ohours,
            i_ominutes,
            i_oseconds,
            i_otext,
            i_omachine,
            i_oevt_num;
        IF ujo_get_det_rep%NOTFOUND THEN
            CLOSE ujo_get_det_rep;
            temp_done_fetch := 0;
        ELSE
            temp_found := temp_found + 1;
            temp_done_fetch := 1;
        END IF;

        done_fetch := temp_done_fetch; 
        found := temp_found;
        RETURN 4;
    END;
END;
/

grant execute on ujo_det_rep_pkg to ujoadmin;



CREATE OR REPLACE FUNCTION ujo_event_state (
    i_eoid VARCHAR2 ,
    i_que_status NUMBER ,
    i_evt_num NUMBER)
RETURN INTEGER AS

    l_evt_num INTEGER;

BEGIN

    IF (i_que_status > 1 AND i_que_status < 5) THEN
        UPDATE ujo_next_oid set oid = oid + 1 where field = 'evt_proc_num'
        RETURNING oid - 1 INTO l_evt_num;
        IF SQL%ROWCOUNT = 0 THEN
            INSERT INTO ujo_next_oid (oid, field) values (1, 'evt_proc_num');
            l_evt_num := 1;
        END IF;
    ELSE
        l_evt_num := i_evt_num + 1;
    END IF;
    
    UPDATE ujo_event
    SET que_status = i_que_status,
        que_status_stamp = SYSDATE,
        evt_num = l_evt_num
    WHERE eoid = i_eoid;

    IF SQL%ROWCOUNT != 1 THEN
        raise_application_error(-20000, 'Invalid event (' || i_eoid || ')');
        RETURN 0;
    END IF;

/* If i_que_status > 1 (EVENT_DONE, HDLR_ERROR, UN_SENT), move to proc_event */
	IF (i_que_status > 1 AND i_que_status < 5) THEN
		INSERT INTO ujo_proc_event(eoid, joid, job_name, box_name, AUTOSERV, priority,
                               event, status, alarm, event_time_gmt, exit_code, machine,
                               pid, jc_pid, run_num, ntry, text, que_priority, stamp,
							   evt_num, que_status, que_status_stamp)
		SELECT *
		FROM ujo_event
		WHERE eoid = i_eoid;

		IF SQL%ROWCOUNT = 1 THEN
			DELETE FROM ujo_event WHERE eoid = i_eoid;
		END IF;

	END IF;

    RETURN 1;

END;
/

grant execute on ujo_event_state to ujoadmin;


CREATE OR REPLACE FUNCTION ujo_send_event (
    i_eoid VARCHAR2,
    i_joid NUMBER,
    i_job_name VARCHAR2,
    i_box_name VARCHAR2,
    i_AUTOSERV VARCHAR2,
    i_priority NUMBER,
    i_event NUMBER,
    i_status NUMBER,
    i_alarm NUMBER,
    i_event_time_gmt NUMBER,
    i_exit_code NUMBER,
    i_machine VARCHAR2,
    i_pid NUMBER,
    i_jc_pid NUMBER,
    i_run_num NUMBER,
    i_ntry NUMBER,
    i_text VARCHAR2,
    i_que_priority NUMBER)

RETURN INTEGER AS

    i_eoid_count INTEGER;
    i_eoid_proc_count INTEGER;
    i_eoid_total INTEGER; 
    i_evt_num NUMBER; 
  	i_counter VARCHAR2(7);
    i_run_num2 NUMBER;

    l_event NUMBER;
    l_instance VARCHAR2(4);
    l_offset NUMBER(11);
    l_event_time_gmt NUMBER(11);
    l_job_type CHAR(1);
    l_count NUMBER;
    l_priority NUMBER;

    l_charpos NUMBER;
    l_nod_name VARCHAR2(65);
    l_nod_ckpt VARCHAR2(16);
    l_nod_boot VARCHAR2(16);
    l_num_nodes NUMBER;

BEGIN
    /* STAR 11631087 */
    IF (i_event = 101 AND
        (i_status = 1 OR i_status = 4 OR i_status = 5 OR i_status = 6 OR i_status = 10)) THEN
        l_priority := 10;
    ELSE
        l_priority := i_priority;
    END IF;

    SELECT str_val INTO l_instance FROM ujo_alamode WHERE type = 'AUTOSERV';

    UPDATE ujo_next_oid
    SET oid = oid + 1
    WHERE field = 'evt_num'
    RETURNING oid INTO i_evt_num;

    l_event := i_event;
    IF (l_instance != i_AUTOSERV OR (i_event = 101 AND (i_status = 1 OR i_status = 3 OR i_status = 4 OR i_status = 5 OR i_status = 6 OR i_status = 10)) ) THEN
        SELECT int_val INTO l_offset FROM ujo_alamode
        WHERE type = 'gmt_offset';

        SELECT ((sysdate - to_date('01-JAN-1970','DD-MON-YYYY')) * 24 * 60 * 60) 
        INTO l_event_time_gmt FROM dual;
        l_event_time_gmt := l_event_time_gmt + l_offset;
        
        IF (i_event = 101 AND i_status = 10 ) THEN
            l_event_time_gmt := l_event_time_gmt + 2;
        END IF; 
    ELSE
        l_event_time_gmt := i_event_time_gmt;
    END IF;

    /* Update the asbnode table if necessary */
    IF (SUBSTR(i_eoid, 4, 1) = 'y') THEN
    
        l_charpos := INSTR(i_text, '$$UJM_NODE$$');
        IF (l_charpos != 0) THEN
            l_nod_name := SUBSTR(i_text, l_charpos + 12, 65);
            l_nod_name := UPPER(l_nod_name);
            l_nod_ckpt := SUBSTR(i_text, l_charpos + 77, 16);
            l_nod_boot := SUBSTR(i_text, l_charpos + 93, 16);

            SELECT count('x') INTO l_num_nodes FROM ujo_asbnode WHERE nod_name = l_nod_name;
            IF (l_num_nodes = 0) THEN
                INSERT INTO ujo_asbnode (nod_name, nod_ckpt, nod_boot)
                    VALUES (l_nod_name, l_nod_ckpt, l_nod_boot);
            ELSE
                UPDATE ujo_asbnode SET nod_ckpt=l_nod_ckpt, nod_boot=l_nod_boot
                    WHERE nod_name=l_nod_name;
            END IF;

        END IF;
    END IF;

    /* Begin - Added for issue number 10582854 - Xia Lin */
	IF (i_event = 12345)
	THEN
	   l_event := 101;

	   IF (i_status = 1 OR i_status = 3 OR i_status = 4 OR i_status = 5 OR i_status = 6 OR i_status = 8)
	   THEN
		  l_event_time_gmt := i_event_time_gmt;
       END IF;
    END IF;
    /* End - Added for issue number 10582854 - Xia Lin */

    SELECT count('x') INTO i_eoid_count FROM ujo_event
    WHERE eoid = i_eoid;

    SELECT count('x') INTO i_eoid_proc_count FROM ujo_proc_event
    WHERE eoid = i_eoid;

    i_eoid_total := i_eoid_count + i_eoid_proc_count;

    IF i_eoid_total > 0 THEN
        SELECT count('x') INTO i_eoid_count FROM ujo_event
        WHERE eoid = i_eoid
        AND job_name = i_job_name 
        AND event = l_event 
        AND status = i_status
        AND ntry = i_ntry;  

        SELECT count('x') INTO i_eoid_proc_count FROM ujo_proc_event
        WHERE eoid = i_eoid
        AND job_name = i_job_name 
        AND event = l_event 
        AND status = i_status
        AND ntry = i_ntry; 

        i_eoid_total := i_eoid_count + i_eoid_proc_count;

        IF i_eoid_total > 0 THEN
            dbms_output.put_line('This Event was already here');
            RETURN 1;
        END IF;

        SELECT counter INTO i_counter FROM ujo_last_Eoid_counter;

        INSERT INTO ujo_event2
            (eoid, stamp, joid, job_name, box_name, 
            AUTOSERV, priority, event, status, alarm,
            event_time_gmt, exit_code, machine, pid, jc_pid, 
            run_num, ntry, text, que_priority, evt_num, 
            que_status, que_status_stamp, counter)
        VALUES
            (i_eoid, SYSDATE, i_joid, i_job_name, i_box_name, 
            i_AUTOSERV, l_priority, l_event, i_status, i_alarm, 
            l_event_time_gmt, i_exit_code, i_machine, i_pid, i_jc_pid, 
            i_run_num, i_ntry, i_text, i_que_priority, i_evt_num, 
            0, SYSDATE, i_counter);

        dbms_output.put_line('Detected duplicate event.') ;
        RETURN -1; 
    END IF;

    IF i_run_num = 0 AND i_joid != 0 THEN
        SELECT nvl(run_num, 0) INTO i_run_num2
        FROM ujo_job_status WHERE joid = i_joid;
    ELSE
        i_run_num2 := i_run_num;
    END IF;


    INSERT INTO ujo_event
        (eoid, stamp, joid, job_name, box_name, 
        AUTOSERV, priority, event, status, alarm, 
        event_time_gmt, exit_code, machine, pid, jc_pid, 
        run_num, ntry, text, que_priority, 
        evt_num, que_status, que_status_stamp)
    VALUES
        (i_eoid, SYSDATE, i_joid, i_job_name, i_box_name, 
        i_AUTOSERV, l_priority, l_event, i_status, i_alarm, 
        l_event_time_gmt, i_exit_code, i_machine, i_pid, i_jc_pid, 
        i_run_num2, i_ntry, i_text, i_que_priority, 
        i_evt_num, 0, SYSDATE);        
   
    IF SQL%ROWCOUNT = 0 THEN
        RETURN -2;
    END IF;

    IF l_event = 106 THEN     /* ALARM! - stub out alarm table */
        INSERT INTO ujo_alarm
            (eoid, alarm, alarm_time, job_name, joid,
            state, state_time, evt_num, event_comment, len, response)
        VALUES
            (i_eoid, i_alarm, l_event_time_gmt, i_job_name, i_joid,
            43, l_event_time_gmt, 0, i_text, 1, ' ');   /*43=OPEN*/
    END IF;
	COMMIT;
    RETURN 1;
END;
/

grant execute on ujo_send_event to ujoadmin;


/****************************************************************************************/
/*                                                                                      */
/* Register patch                                                                       */
/*                                                                                      */
/****************************************************************************************/
INSERT INTO mdb_patch ( patchnumber, installdate,  mdbmajorversion, mdbminorversion, description)
 VALUES ( 15032461, systimestamp, 1, 4, 'Star 15032461 AUTOSYS: Duplicate EVT_NUM values' );

COMMIT;

EXIT SUCCESS;
