SET ECHO ON;
WHENEVER OSERROR EXIT FAILURE;
WHENEVER SQLERROR EXIT SQL.SQLCODE;

SET AUTOCOMMIT OFF;


/****************************************************************************************/
/*                                                                                      */
/* Star 16526919 remove case sensitivity enforcement on name column of ca_resource_family */
/*                                                                                      */
/****************************************************************************************/
/* 16584925 MDB auto-build CRPatch */ 
CREATE OR REPLACE TRIGGER t_ci_ca_resource_family
BEFORE INSERT OR UPDATE ON MDBADMIN.ca_resource_family
FOR EACH ROW

/*  declare variables  */
DECLARE
	i INTEGER;   /*  integer to check for existence of duplicate index values  */

	/*  changing fields that are not part of the index causes a mutating table error (ORA-4091)  */
	/*  created an exception variable and initialized it to -4091 to trap and igonore the error  */
	mutating_table EXCEPTION;
	PRAGMA EXCEPTION_INIT (mutating_table, -4091);

/*  main logic  */
BEGIN

	/*  check for duplicate index values in the table  */
	SELECT COUNT(*) INTO i FROM MDBADMIN.ca_resource_family
		 WHERE name = :new.name;

	/*  if duplicates found, raise error  */
	IF i > 0 THEN
		RAISE DUP_VAL_ON_INDEX;
	END IF;

/*  exception handler  */
EXCEPTION

	/*  Duplicate Error - raise error to calling application  */
	WHEN DUP_VAL_ON_INDEX THEN
		RAISE DUP_VAL_ON_INDEX;

	/*  Mutating Table Error - ignore  */
	WHEN mutating_table THEN
		i:= 0;

END;
/
/****************************************************************************************/
/*                                                                                      */
/* Register patch                                                                       */
/*                                                                                      */
/****************************************************************************************/
INSERT INTO mdb_patch ( patchnumber, installdate,  mdbmajorversion, mdbminorversion, description)
 VALUES ( 16526919, systimestamp, 1, 4, 'Star 16526919 remove case sensitivity enforcement on name column of ca_resource_family' );

COMMIT;

EXIT SUCCESS;
