SET LOCK_TIMEOUT 300000
GO
SET DEADLOCK_PRIORITY LOW
GO
SET XACT_ABORT ON
GO
SET TRANSACTION ISOLATION LEVEL SERIALIZABLE
GO
BEGIN TRANSACTION 
GO

/****************************************************************************************/
/*                                                                                      */
/* STAR 16443075 UAPM r11.3.4 support							*/
/*                                                                                      */
/****************************************************************************************/

/* lock objects...  */
 
/* Start of lines added to convert to online lock */
 
 
Select top 1 1 from mdb_patch with (tablockx)
GO
 
/* Start of locks for dependent tables */
 
Select top 1 1 from mdb_service_pack with (tablockx)
GO
 
Select top 1 1 from mdb with (tablockx)
GO
 
Select top 1 1 from mdb_checkpoint with (tablockx)
GO
 
Select top 1 1 from ca_resource_family with (tablockx)
GO
 
Select top 1 1 from ca_discovered_hardware with (tablockx)
GO
 
/* End of lines added to convert to online lock */

ALTER TABLE ca_resource_family ADD is_software int NULL DEFAULT 0
GO

UPDATE ca_resource_family SET is_software = 0
GO

UPDATE ca_resource_family SET is_software = 1 
WHERE id = 601
GO

ALTER TABLE ca_discovered_hardware 
DROP CONSTRAINT DF__ca_discov__disco__17E28260
GO

ALTER TABLE ca_discovered_hardware 
ADD CONSTRAINT DF__ca_discov__disco__17E28260 DEFAULT (1) FOR discovery_changes_switch
GO

UPDATE ca_discovered_hardware SET discovery_changes_switch = 1 WHERE discovery_changes_switch <> 1
GO

CREATE TRIGGER t_u_ca_agent ON ca_agent
FOR UPDATE AS

/*  Only run if last_run_date has been updated. */

if update (last_run_date)
  begin
  
    /*  Only run if UAPM is installed. */
  
    declare @count integer
    
    set @count = (SELECT count(*) 
                    FROM ca_application_registration 
                   WHERE product_code = 2002)
                   
    if @count = 0
        return
  
    declare @object_uuid binary(16)
    declare change_list cursor for select inserted.object_uuid
                                     from inserted, deleted
                                    where inserted.object_uuid = deleted.object_uuid
                                      and inserted.last_run_date <> deleted.last_run_date
    open change_list
    
    fetch next from change_list into @object_uuid
    
    while @@fetch_status = 0
      begin
      
          begin
    
            /* Set the discovery_changes_switch "on". */
    
            update ca_discovered_hardware
    	       set discovery_changes_switch = 1
             where dis_hw_uuid = @object_uuid
               and discovery_changes_switch <> 1
               
          end
    
        fetch next from change_list into @object_uuid
            
      end
    	
    close change_list
    deallocate change_list

  end
GO 

IF OBJECT_ID ('t_u_inv_generalinventory_item','TR') IS NOT NULL
   DROP TRIGGER t_u_inv_generalinventory_item
GO

CREATE TRIGGER t_u_inv_generalinventory_item ON inv_generalinventory_item
FOR UPDATE AS

/* Only execute if item_value_text, item_value_double or item_value_long has been updated. */

if update (item_value_text) or update (item_value_double) or update (item_value_long)
  begin
  
    /*  Only run if UAPM is installed. */
  
    declare @count integer
    
    set @count = (SELECT count(*) 
                    FROM ca_application_registration 
                   WHERE product_code = 2002)
                          
    if @count = 0
        return  
  
    declare @object_uuid binary(16)
    
    declare change_list cursor for 
      select object_uuid from inserted
        where (item_parent_name_id = 2  and item_name_id = 7   and item_value_double <> item_previous_value_double)
           or (item_parent_name_id = 2  and item_name_id = 9   and item_value_long <> item_previous_value_long)
           or (item_parent_name_id = 5  and item_name_id = 17  and item_value_text <> item_previous_value_text)
           or (item_parent_name_id = 6  and item_name_id = 1   and item_value_text <> item_previous_value_text)
           or (item_parent_name_id = 6  and item_name_id = 13  and item_value_long <> item_previous_value_long)
           or (item_parent_name_id = 31 and item_name_id = 132 and item_value_double <> item_previous_value_double)
           
    open change_list    

    fetch next from change_list into @object_uuid    

    while @@fetch_status = 0
      begin

        /*  Update the discovery_changes_switch for the  */
        /*  associated ca_discovered_hardware record,    */
        /*  only if the switch is "off".                 */
         
        update ca_discovered_hardware
           set discovery_changes_switch = 1
         where dis_hw_uuid = @object_uuid
           and discovery_changes_switch <> 1
          
        fetch next from change_list into @object_uuid
        
      end

    close change_list    
    deallocate change_list
      
  end
GO

/****************************************************************************************/
/*                                                                                      */
/* Register patch                                                               	*/
/*                                                                                      */
/****************************************************************************************/
insert into mdb_patch ( patchnumber, installdate,  mdbmajorversion, mdbminorversion, description)
 values ( 16443075, getdate(), 1, 4, 'STAR 16443075 UAPM r11.3.4 support' )
GO

COMMIT TRANSACTION 
GO

