SET ECHO ON;
WHENEVER OSERROR EXIT FAILURE;
WHENEVER SQLERROR EXIT SQL.SQLCODE;

SET AUTOCOMMIT OFF;


/****************************************************************************************/
/*                                                                                      */
/* Star 14651279 NEED TO ADD TRIGGERS							*/
/*                                                                                      */
/****************************************************************************************/


/*******************************************/
/*******************************************/
/* Change Category */
/*******************************************/
/*******************************************/
CREATE OR REPLACE TRIGGER t_ci_chgcat
BEFORE INSERT OR UPDATE ON MDBADMIN.chgcat
FOR EACH ROW

/*  declare variables  */
DECLARE
	i INTEGER;   /*  integer to check for existence of duplicate index values  */

	/*  changing fields that are not part of the index causes a mutating table error (ORA-4091)  */
	/*  created an exception variable and initialized it to -4091 to trap and igonore the error  */
	mutating_table EXCEPTION;
	PRAGMA EXCEPTION_INIT (mutating_table, -4091);

/*  main logic  */
BEGIN

	/*  check for duplicate index values in the table  */
	SELECT COUNT(*) INTO i FROM MDBADMIN.chgcat
		 WHERE NLS_UPPER(code) = NLS_UPPER(:new.code);

	/*  if duplicates found, raise error  */
	IF i > 0 THEN
		RAISE DUP_VAL_ON_INDEX;
	END IF;

/*  exception handler  */
EXCEPTION

	/*  Duplicate Error - raise error to calling application  */
	WHEN DUP_VAL_ON_INDEX THEN
		RAISE DUP_VAL_ON_INDEX;

	/*  Mutating Table Error - ignore  */
	WHEN mutating_table THEN
		i:= 0;

END;
/

/*******************************************/
/*******************************************/
/* Change_Status */
/*******************************************/
/*******************************************/
CREATE OR REPLACE TRIGGER t_ci_chgstat
BEFORE INSERT OR UPDATE ON MDBADMIN.chgstat
FOR EACH ROW

/*  declare variables  */
DECLARE
	i INTEGER;   /*  integer to check for existence of duplicate index values  */

	/*  changing fields that are not part of the index causes a mutating table error (ORA-4091)  */
	/*  created an exception variable and initialized it to -4091 to trap and igonore the error  */
	mutating_table EXCEPTION;
	PRAGMA EXCEPTION_INIT (mutating_table, -4091);

/*  main logic  */
BEGIN

	/*  check for duplicate index values in the table  */
	SELECT COUNT(*) INTO i FROM MDBADMIN.chgstat
		 WHERE NLS_UPPER(code) = NLS_UPPER(:new.code);

	/*  if duplicates found, raise error  */
	IF i > 0 THEN
		RAISE DUP_VAL_ON_INDEX;
	END IF;

/*  exception handler  */
EXCEPTION

	/*  Duplicate Error - raise error to calling application  */
	WHEN DUP_VAL_ON_INDEX THEN
		RAISE DUP_VAL_ON_INDEX;

	/*  Mutating Table Error - ignore  */
	WHEN mutating_table THEN
		i:= 0;

END;
/

/*******************************************/
/*******************************************/
/* Cr_Stored_Queries */
/*******************************************/
/*******************************************/
CREATE OR REPLACE TRIGGER t_ci_crsq
BEFORE INSERT OR UPDATE ON MDBADMIN.crsq
FOR EACH ROW

/*  declare variables  */
DECLARE
	i INTEGER;   /*  integer to check for existence of duplicate index values  */

	/*  changing fields that are not part of the index causes a mutating table error (ORA-4091)  */
	/*  created an exception variable and initialized it to -4091 to trap and igonore the error  */
	mutating_table EXCEPTION;
	PRAGMA EXCEPTION_INIT (mutating_table, -4091);

/*  main logic  */
BEGIN

	/*  check for duplicate index values in the table  */
	SELECT COUNT(*) INTO i FROM MDBADMIN.crsq
		 WHERE NLS_UPPER(code) = NLS_UPPER(:new.code);

	/*  if duplicates found, raise error  */
	IF i > 0 THEN
		RAISE DUP_VAL_ON_INDEX;
	END IF;

/*  exception handler  */
EXCEPTION

	/*  Duplicate Error - raise error to calling application  */
	WHEN DUP_VAL_ON_INDEX THEN
		RAISE DUP_VAL_ON_INDEX;

	/*  Mutating Table Error - ignore  */
	WHEN mutating_table THEN
		i:= 0;

END;
/

/*******************************************/
/*******************************************/
/* Timezone */
/*******************************************/
/*******************************************/
CREATE OR REPLACE TRIGGER t_ci_tz
BEFORE INSERT OR UPDATE ON MDBADMIN.tz
FOR EACH ROW

/*  declare variables  */
DECLARE
	i INTEGER;   /*  integer to check for existence of duplicate index values  */

	/*  changing fields that are not part of the index causes a mutating table error (ORA-4091)  */
	/*  created an exception variable and initialized it to -4091 to trap and igonore the error  */
	mutating_table EXCEPTION;
	PRAGMA EXCEPTION_INIT (mutating_table, -4091);

/*  main logic  */
BEGIN

	/*  check for duplicate index values in the table  */
	SELECT COUNT(*) INTO i FROM MDBADMIN.tz
		 WHERE NLS_UPPER(code) = NLS_UPPER(:new.code);

	/*  if duplicates found, raise error  */
	IF i > 0 THEN
		RAISE DUP_VAL_ON_INDEX;
	END IF;

/*  exception handler  */
EXCEPTION

	/*  Duplicate Error - raise error to calling application  */
	WHEN DUP_VAL_ON_INDEX THEN
		RAISE DUP_VAL_ON_INDEX;

	/*  Mutating Table Error - ignore  */
	WHEN mutating_table THEN
		i:= 0;

END;
/

/*******************************************/
/*******************************************/
/* Mgs_Status */
/*******************************************/
/*******************************************/
CREATE OR REPLACE TRIGGER t_ci_mgsstat
BEFORE INSERT OR UPDATE ON MDBADMIN.mgsstat
FOR EACH ROW

/*  declare variables  */
DECLARE
	i INTEGER;   /*  integer to check for existence of duplicate index values  */

	/*  changing fields that are not part of the index causes a mutating table error (ORA-4091)  */
	/*  created an exception variable and initialized it to -4091 to trap and igonore the error  */
	mutating_table EXCEPTION;
	PRAGMA EXCEPTION_INIT (mutating_table, -4091);

/*  main logic  */
BEGIN

	/*  check for duplicate index values in the table  */
	SELECT COUNT(*) INTO i FROM MDBADMIN.mgsstat
		 WHERE NLS_UPPER(code) = NLS_UPPER(:new.code);

	/*  if duplicates found, raise error  */
	IF i > 0 THEN
		RAISE DUP_VAL_ON_INDEX;
	END IF;

/*  exception handler  */
EXCEPTION

	/*  Duplicate Error - raise error to calling application  */
	WHEN DUP_VAL_ON_INDEX THEN
		RAISE DUP_VAL_ON_INDEX;

	/*  Mutating Table Error - ignore  */
	WHEN mutating_table THEN
		i:= 0;

END;
/

/*******************************************/
/*******************************************/
/* Call_Req_Type */
/*******************************************/
/*******************************************/
CREATE OR REPLACE TRIGGER t_ci_crt
BEFORE INSERT OR UPDATE ON MDBADMIN.crt
FOR EACH ROW

/*  declare variables  */
DECLARE
	i INTEGER;   /*  integer to check for existence of duplicate index values  */

	/*  changing fields that are not part of the index causes a mutating table error (ORA-4091)  */
	/*  created an exception variable and initialized it to -4091 to trap and igonore the error  */
	mutating_table EXCEPTION;
	PRAGMA EXCEPTION_INIT (mutating_table, -4091);

/*  main logic  */
BEGIN

	/*  check for duplicate index values in the table  */
	SELECT COUNT(*) INTO i FROM MDBADMIN.crt
		 WHERE NLS_UPPER(code) = NLS_UPPER(:new.code);

	/*  if duplicates found, raise error  */
	IF i > 0 THEN
		RAISE DUP_VAL_ON_INDEX;
	END IF;

/*  exception handler  */
EXCEPTION

	/*  Duplicate Error - raise error to calling application  */
	WHEN DUP_VAL_ON_INDEX THEN
		RAISE DUP_VAL_ON_INDEX;

	/*  Mutating Table Error - ignore  */
	WHEN mutating_table THEN
		i:= 0;

END;
/

/*******************************************/
/*******************************************/
/* Act_Type_Assoc */
/*******************************************/
/*******************************************/
CREATE OR REPLACE TRIGGER t_ci_atyp_asc
BEFORE INSERT OR UPDATE ON MDBADMIN.atyp_asc
FOR EACH ROW

/*  declare variables  */
DECLARE
	i INTEGER;   /*  integer to check for existence of duplicate index values  */

	/*  changing fields that are not part of the index causes a mutating table error (ORA-4091)  */
	/*  created an exception variable and initialized it to -4091 to trap and igonore the error  */
	mutating_table EXCEPTION;
	PRAGMA EXCEPTION_INIT (mutating_table, -4091);

/*  main logic  */
BEGIN

	/*  check for duplicate index values in the table  */
	SELECT COUNT(*) INTO i FROM MDBADMIN.atyp_asc
		 WHERE NLS_UPPER(code) = NLS_UPPER(:new.code);

	/*  if duplicates found, raise error  */
	IF i > 0 THEN
		RAISE DUP_VAL_ON_INDEX;
	END IF;

/*  exception handler  */
EXCEPTION

	/*  Duplicate Error - raise error to calling application  */
	WHEN DUP_VAL_ON_INDEX THEN
		RAISE DUP_VAL_ON_INDEX;

	/*  Mutating Table Error - ignore  */
	WHEN mutating_table THEN
		i:= 0;

END;
/

/*******************************************/
/*******************************************/
/* Cr_Status */
/*******************************************/
/*******************************************/
CREATE OR REPLACE TRIGGER t_ci_cr_stat
BEFORE INSERT OR UPDATE ON MDBADMIN.cr_stat
FOR EACH ROW

/*  declare variables  */
DECLARE
	i INTEGER;   /*  integer to check for existence of duplicate index values  */

	/*  changing fields that are not part of the index causes a mutating table error (ORA-4091)  */
	/*  created an exception variable and initialized it to -4091 to trap and igonore the error  */
	mutating_table EXCEPTION;
	PRAGMA EXCEPTION_INIT (mutating_table, -4091);

/*  main logic  */
BEGIN

	/*  check for duplicate index values in the table  */
	SELECT COUNT(*) INTO i FROM MDBADMIN.cr_stat
		 WHERE NLS_UPPER(code) = NLS_UPPER(:new.code);

	/*  if duplicates found, raise error  */
	IF i > 0 THEN
		RAISE DUP_VAL_ON_INDEX;
	END IF;

/*  exception handler  */
EXCEPTION

	/*  Duplicate Error - raise error to calling application  */
	WHEN DUP_VAL_ON_INDEX THEN
		RAISE DUP_VAL_ON_INDEX;

	/*  Mutating Table Error - ignore  */
	WHEN mutating_table THEN
		i:= 0;

END;
/



/*******************************************/
/*******************************************/
/* Act_Type */
/*******************************************/
/*******************************************/
CREATE OR REPLACE TRIGGER t_ci_act_type
BEFORE INSERT OR UPDATE ON MDBADMIN.act_type
FOR EACH ROW

/*  declare variables  */
DECLARE
	i INTEGER;   /*  integer to check for existence of duplicate index values  */

	/*  changing fields that are not part of the index causes a mutating table error (ORA-4091)  */
	/*  created an exception variable and initialized it to -4091 to trap and igonore the error  */
	mutating_table EXCEPTION;
	PRAGMA EXCEPTION_INIT (mutating_table, -4091);

/*  main logic  */
BEGIN

	/*  check for duplicate index values in the table  */
	SELECT COUNT(*) INTO i FROM MDBADMIN.act_type
		 WHERE NLS_UPPER(code) = NLS_UPPER(:new.code);

	/*  if duplicates found, raise error  */
	IF i > 0 THEN
		RAISE DUP_VAL_ON_INDEX;
	END IF;

/*  exception handler  */
EXCEPTION

	/*  Duplicate Error - raise error to calling application  */
	WHEN DUP_VAL_ON_INDEX THEN
		RAISE DUP_VAL_ON_INDEX;

	/*  Mutating Table Error - ignore  */
	WHEN mutating_table THEN
		i:= 0;

END;
/

/*******************************************/
/*******************************************/
/* Service_Desc */
/*******************************************/
/*******************************************/
CREATE OR REPLACE TRIGGER t_ci_srv_desc
BEFORE INSERT OR UPDATE ON MDBADMIN.srv_desc
FOR EACH ROW

/*  declare variables  */
DECLARE
	i INTEGER;   /*  integer to check for existence of duplicate index values  */

	/*  changing fields that are not part of the index causes a mutating table error (ORA-4091)  */
	/*  created an exception variable and initialized it to -4091 to trap and igonore the error  */
	mutating_table EXCEPTION;
	PRAGMA EXCEPTION_INIT (mutating_table, -4091);

/*  main logic  */
BEGIN

	/*  check for duplicate index values in the table  */
	SELECT COUNT(*) INTO i FROM MDBADMIN.srv_desc
		 WHERE NLS_UPPER(code) = NLS_UPPER(:new.code);

	/*  if duplicates found, raise error  */
	IF i > 0 THEN
		RAISE DUP_VAL_ON_INDEX;
	END IF;

/*  exception handler  */
EXCEPTION

	/*  Duplicate Error - raise error to calling application  */
	WHEN DUP_VAL_ON_INDEX THEN
		RAISE DUP_VAL_ON_INDEX;

	/*  Mutating Table Error - ignore  */
	WHEN mutating_table THEN
		i:= 0;

END;
/

/*******************************************/
/*******************************************/
/* Remote_Ref */
/*******************************************/
/*******************************************/
CREATE OR REPLACE TRIGGER t_ci_rem_ref
BEFORE INSERT OR UPDATE ON MDBADMIN.rem_ref
FOR EACH ROW

/*  declare variables  */
DECLARE
	i INTEGER;   /*  integer to check for existence of duplicate index values  */

	/*  changing fields that are not part of the index causes a mutating table error (ORA-4091)  */
	/*  created an exception variable and initialized it to -4091 to trap and igonore the error  */
	mutating_table EXCEPTION;
	PRAGMA EXCEPTION_INIT (mutating_table, -4091);

/*  main logic  */
BEGIN

	/*  check for duplicate index values in the table  */
	SELECT COUNT(*) INTO i FROM MDBADMIN.rem_ref
		 WHERE NLS_UPPER(code) = NLS_UPPER(:new.code);

	/*  if duplicates found, raise error  */
	IF i > 0 THEN
		RAISE DUP_VAL_ON_INDEX;
	END IF;

/*  exception handler  */
EXCEPTION

	/*  Duplicate Error - raise error to calling application  */
	WHEN DUP_VAL_ON_INDEX THEN
		RAISE DUP_VAL_ON_INDEX;

	/*  Mutating Table Error - ignore  */
	WHEN mutating_table THEN
		i:= 0;

END;
/

/*******************************************/
/*******************************************/
/* Sequence_Control */
/*******************************************/
/*******************************************/
CREATE OR REPLACE TRIGGER t_ci_seqctl
BEFORE INSERT OR UPDATE ON MDBADMIN.seqctl
FOR EACH ROW

/*  declare variables  */
DECLARE
	i INTEGER;   /*  integer to check for existence of duplicate index values  */

	/*  changing fields that are not part of the index causes a mutating table error (ORA-4091)  */
	/*  created an exception variable and initialized it to -4091 to trap and igonore the error  */
	mutating_table EXCEPTION;
	PRAGMA EXCEPTION_INIT (mutating_table, -4091);

/*  main logic  */
BEGIN

	/*  check for duplicate index values in the table  */
	SELECT COUNT(*) INTO i FROM MDBADMIN.seqctl
		 WHERE NLS_UPPER(code) = NLS_UPPER(:new.code);

	/*  if duplicates found, raise error  */
	IF i > 0 THEN
		RAISE DUP_VAL_ON_INDEX;
	END IF;

/*  exception handler  */
EXCEPTION

	/*  Duplicate Error - raise error to calling application  */
	WHEN DUP_VAL_ON_INDEX THEN
		RAISE DUP_VAL_ON_INDEX;

	/*  Mutating Table Error - ignore  */
	WHEN mutating_table THEN
		i:= 0;

END;
/

/*******************************************/
/*******************************************/
/* Issue_Category */
/*******************************************/
/*******************************************/
CREATE OR REPLACE TRIGGER t_ci_isscat
BEFORE INSERT OR UPDATE ON MDBADMIN.isscat
FOR EACH ROW

/*  declare variables  */
DECLARE
	i INTEGER;   /*  integer to check for existence of duplicate index values  */

	/*  changing fields that are not part of the index causes a mutating table error (ORA-4091)  */
	/*  created an exception variable and initialized it to -4091 to trap and igonore the error  */
	mutating_table EXCEPTION;
	PRAGMA EXCEPTION_INIT (mutating_table, -4091);

/*  main logic  */
BEGIN

	/*  check for duplicate index values in the table  */
	SELECT COUNT(*) INTO i FROM MDBADMIN.isscat
		 WHERE NLS_UPPER(code) = NLS_UPPER(:new.code);

	/*  if duplicates found, raise error  */
	IF i > 0 THEN
		RAISE DUP_VAL_ON_INDEX;
	END IF;

/*  exception handler  */
EXCEPTION

	/*  Duplicate Error - raise error to calling application  */
	WHEN DUP_VAL_ON_INDEX THEN
		RAISE DUP_VAL_ON_INDEX;

	/*  Mutating Table Error - ignore  */
	WHEN mutating_table THEN
		i:= 0;

END;
/

/*******************************************/
/*******************************************/
/* Issue_Status */
/*******************************************/
/*******************************************/
CREATE OR REPLACE TRIGGER t_ci_issstat
BEFORE INSERT OR UPDATE ON MDBADMIN.issstat
FOR EACH ROW

/*  declare variables  */
DECLARE
	i INTEGER;   /*  integer to check for existence of duplicate index values  */

	/*  changing fields that are not part of the index causes a mutating table error (ORA-4091)  */
	/*  created an exception variable and initialized it to -4091 to trap and igonore the error  */
	mutating_table EXCEPTION;
	PRAGMA EXCEPTION_INIT (mutating_table, -4091);

/*  main logic  */
BEGIN

	/*  check for duplicate index values in the table  */
	SELECT COUNT(*) INTO i FROM MDBADMIN.issstat
		 WHERE NLS_UPPER(code) = NLS_UPPER(:new.code);

	/*  if duplicates found, raise error  */
	IF i > 0 THEN
		RAISE DUP_VAL_ON_INDEX;
	END IF;

/*  exception handler  */
EXCEPTION

	/*  Duplicate Error - raise error to calling application  */
	WHEN DUP_VAL_ON_INDEX THEN
		RAISE DUP_VAL_ON_INDEX;

	/*  Mutating Table Error - ignore  */
	WHEN mutating_table THEN
		i:= 0;

END;
/

/*******************************************/
/*******************************************/
/* Spell_Macro_Type */
/*******************************************/
/*******************************************/
CREATE OR REPLACE TRIGGER t_ci_splmactp
BEFORE INSERT OR UPDATE ON MDBADMIN.splmactp
FOR EACH ROW

/*  declare variables  */
DECLARE
	i INTEGER;   /*  integer to check for existence of duplicate index values  */

	/*  changing fields that are not part of the index causes a mutating table error (ORA-4091)  */
	/*  created an exception variable and initialized it to -4091 to trap and igonore the error  */
	mutating_table EXCEPTION;
	PRAGMA EXCEPTION_INIT (mutating_table, -4091);

/*  main logic  */
BEGIN

	/*  check for duplicate index values in the table  */
	SELECT COUNT(*) INTO i FROM MDBADMIN.splmactp
		 WHERE NLS_UPPER(code) = NLS_UPPER(:new.code);

	/*  if duplicates found, raise error  */
	IF i > 0 THEN
		RAISE DUP_VAL_ON_INDEX;
	END IF;

/*  exception handler  */
EXCEPTION

	/*  Duplicate Error - raise error to calling application  */
	WHEN DUP_VAL_ON_INDEX THEN
		RAISE DUP_VAL_ON_INDEX;

	/*  Mutating Table Error - ignore  */
	WHEN mutating_table THEN
		i:= 0;

END;
/


/*******************************************/
/*******************************************/
/* Task_Type */
/*******************************************/
/*******************************************/
CREATE OR REPLACE TRIGGER t_ci_tskty
BEFORE INSERT OR UPDATE ON MDBADMIN.tskty
FOR EACH ROW

/*  declare variables  */
DECLARE
	i INTEGER;   /*  integer to check for existence of duplicate index values  */

	/*  changing fields that are not part of the index causes a mutating table error (ORA-4091)  */
	/*  created an exception variable and initialized it to -4091 to trap and igonore the error  */
	mutating_table EXCEPTION;
	PRAGMA EXCEPTION_INIT (mutating_table, -4091);

/*  main logic  */
BEGIN

	/*  check for duplicate index values in the table  */
	SELECT COUNT(*) INTO i FROM MDBADMIN.tskty
		 WHERE NLS_UPPER(code) = NLS_UPPER(:new.code);

	/*  if duplicates found, raise error  */
	IF i > 0 THEN
		RAISE DUP_VAL_ON_INDEX;
	END IF;

/*  exception handler  */
EXCEPTION

	/*  Duplicate Error - raise error to calling application  */
	WHEN DUP_VAL_ON_INDEX THEN
		RAISE DUP_VAL_ON_INDEX;

	/*  Mutating Table Error - ignore  */
	WHEN mutating_table THEN
		i:= 0;

END;
/

/*******************************************/
/*******************************************/
/* Task_Status */
/*******************************************/
/*******************************************/
CREATE OR REPLACE TRIGGER t_ci_tskstat
BEFORE INSERT OR UPDATE ON MDBADMIN.tskstat
FOR EACH ROW

/*  declare variables  */
DECLARE
	i INTEGER;   /*  integer to check for existence of duplicate index values  */

	/*  changing fields that are not part of the index causes a mutating table error (ORA-4091)  */
	/*  created an exception variable and initialized it to -4091 to trap and igonore the error  */
	mutating_table EXCEPTION;
	PRAGMA EXCEPTION_INIT (mutating_table, -4091);

/*  main logic  */
BEGIN

	/*  check for duplicate index values in the table  */
	SELECT COUNT(*) INTO i FROM MDBADMIN.tskstat
		 WHERE NLS_UPPER(code) = NLS_UPPER(:new.code);

	/*  if duplicates found, raise error  */
	IF i > 0 THEN
		RAISE DUP_VAL_ON_INDEX;
	END IF;

/*  exception handler  */
EXCEPTION

	/*  Duplicate Error - raise error to calling application  */
	WHEN DUP_VAL_ON_INDEX THEN
		RAISE DUP_VAL_ON_INDEX;

	/*  Mutating Table Error - ignore  */
	WHEN mutating_table THEN
		i:= 0;

END;
/



/****************************************************************************************/
/*                                                                                      */
/* Register patch                                                                       */
/*                                                                                      */
/****************************************************************************************/
INSERT INTO mdb_patch ( patchnumber, installdate,  mdbmajorversion, mdbminorversion, description)
 VALUES ( 14651279, systimestamp, 1, 4, 'Star 14651279 NEED TO ADD TRIGGERS' );

COMMIT;

EXIT SUCCESS;
