SET ECHO ON;
WHENEVER OSERROR EXIT FAILURE;
WHENEVER SQLERROR EXIT SQL.SQLCODE;

SET AUTOCOMMIT OFF;


/****************************************************************************************/
/*                                                                                      */
/* Patch Star 16231032 Unicenter Insight Performance Monitor: add dbh_sql_mss		*/
/*                                                                                      */
/****************************************************************************************/

CREATE TABLE DBH_SQL_MSS
(SQL_ID	NUMBER(8,0) NOT NULL,
 CAPTURE_ID	NUMBER(8,0),
 DBH_INSTANCE	VARCHAR2(80) NOT NULL,
 SNAPSHOT_TIMESTAMP	DATE NOT NULL,
 SCAN_TYPE	CHAR(1) NOT NULL,
 SQL_OWNER	VARCHAR2(256),
 SQL_PARSER	VARCHAR2(256),
 SESSION_ID	NUMBER(8,0),
 HOST_NAME	VARCHAR2(256),
 APPL_NAME	VARCHAR2(256),
 OS_USER	VARCHAR2(256),
 SQL_HANDLE	VARCHAR2(100),
 STATEMENT_START_OFFSET	NUMBER(12,0),
 PLAN_HANDLE	VARCHAR2(100),
 CUR_LOG_READ	NUMBER(12,0),
 CUR_LOG_WRITE	NUMBER(12,0),
 CUR_PHY_READ	NUMBER(12,0),
 CUR_CPU	NUMBER(16,6),
 CUR_ELAPSED_TIME	NUMBER(16,6),
 PLAN_GENERATION_NUM	NUMBER(8,0),
 EXECUTION_COUNT	NUMBER(8,0),
 PLAN_CREATION_TIME	DATE,
 LAST_EXEC_TIME	DATE,
 LAST_LOG_READ	NUMBER(12,0),
 LAST_LOG_WRITE	NUMBER(12,0),
 LAST_PHY_READ	NUMBER(12,0),
 LAST_CPU	NUMBER(16,6),
 LAST_CLR_TIME	NUMBER(16,6),
 LAST_ELAPSED_TIME	NUMBER(16,6),
 MIN_LOG_READ	NUMBER(12,0),
 MIN_LOG_WRITE	NUMBER(12,0),
 MIN_PHY_READ	NUMBER(12,0),
 MIN_CPU	NUMBER(16,6),
 MIN_CLR_TIME	NUMBER(16,6),
 MIN_ELAPSED_TIME	NUMBER(16,6),
 MAX_LOG_READ	NUMBER(12,0),
 MAX_LOG_WRITE	NUMBER(12,0),
 MAX_PHY_READ	NUMBER(12,0),
 MAX_CPU	NUMBER(16,6),
 MAX_CLR_TIME	NUMBER(16,6),
 MAX_ELAPSED_TIME	NUMBER(16,6),
 TOTAL_LOG_READ	NUMBER(12,0),
 TOTAL_LOG_WRITE	NUMBER(12,0),
 TOTAL_PHY_READ	NUMBER(12,0),
 TOTAL_CPU	NUMBER(16,6),
 TOTAL_CLR_TIME	NUMBER(16,6),
 TOTAL_ELAPSED_TIME	NUMBER(16,6),
 DIFF_LOG_READ	NUMBER(12,0),
 DIFF_LOG_WRITE	NUMBER(12,0),
 DIFF_PHY_READ	NUMBER(12,0),
 DIFF_IO	NUMBER(12,0),
 DIFF_CPU	NUMBER(16,6),
 STMT_TEXT	CLOB NOT NULL,
 TAG	VARCHAR2(50),
 FOLDER	VARCHAR2(50),
 DESCRIPTION	VARCHAR2(2000),
 PLAN_OWNER	VARCHAR2(30),
 DB_CONNECT_STRING	VARCHAR2(400),
 DB_NAME	VARCHAR2(256),
 DB_VERSION	VARCHAR2(30),
 CREATE_DATE	DATE,
 VERIFY_DATE	DATE,
 START_CAPTURE_TIME	DATE,
 END_CAPTURE_TIME	DATE,
 CAPTURE_INTERVAL	NUMBER(12,2),
 VAR1	VARCHAR2(400),
 VAR2	VARCHAR2(400),
 VAR3	VARCHAR2(400),
 VAR4	VARCHAR2(400),
 VAR5	VARCHAR2(400),
 TEXT1	CLOB,
 DEC1	NUMBER(12,2),
 DEC2	NUMBER(12,2),
 DEC3	NUMBER(12,2),
 DEC4	NUMBER(16,6),
 DEC5	NUMBER(16,6),
 FLOAT1	FLOAT,
 FLOAT2	FLOAT,
 FLOAT3	FLOAT,
 DATE1	DATE,
 DATE2	DATE,
 DATE3	DATE
);

ALTER TABLE DBH_SQL_MSS ADD CONSTRAINT DBH_SQL_MSS_PK
  PRIMARY KEY (SQL_ID)
;
ALTER TABLE DBH_SQL_MSS ADD CONSTRAINT DBH_SQL_MSS_CAPTURE_FK
  FOREIGN KEY (CAPTURE_ID) REFERENCES DBH_CAPTURES(CAPTURE_ID) ON DELETE CASCADE
;
CREATE INDEX DBH_SQL_MSS_SNAPSHOT_TS_IDX ON DBH_SQL_MSS (CAPTURE_ID, SNAPSHOT_TIMESTAMP)
;
CREATE INDEX DBH_SQL_MSS_INST_SNAPTS_IDX ON DBH_SQL_MSS (DBH_INSTANCE, SCAN_TYPE, SNAPSHOT_TIMESTAMP)
;
CREATE SEQUENCE DBH_SQL_MSS_ID_SEQ START WITH 1
;
GRANT SELECT,INSERT,DELETE,UPDATE ON MDBADMIN.DBH_SQL_MSS TO UDMADMIN_GROUP
;
GRANT SELECT ON MDBADMIN.DBH_SQL_MSS_ID_SEQ TO UDMADMIN_GROUP
;

/****************************************************************************************/
/*                                                                                      */
/* Register patch                                                                       */
/*                                                                                      */
/****************************************************************************************/
INSERT INTO mdb_patch ( patchnumber, installdate,  mdbmajorversion, mdbminorversion, description)
 VALUES ( 16231032, systimestamp, 1, 4, 'Patch Star 16231032 Unicenter Insight Performance Monitor: add dbh_sql_mss' );

COMMIT;

EXIT SUCCESS;
