rem @Echo off
REM
REM Name: setupmdb_pre.bat 
REM
REM Description: 
REM    internal pre-process script for mdb on Oracle
REM 
REM Syntax:
REM
REM    setupmdb_pre 
REM
REM History:
REM    27-jul-2006  mulwi01  created 
REM    28-aug-2006  mulwi01  allow existing mdbadmin 
REM    31-aug-2007  johri05  16212586  27Jul07 MODIFICATIONS TO UPGRADE PROC
REM                           If database is 1.0.4 then run CumulativePatch.
REM    15-sep-2007  devjo01  16212586 Add body of code to appply cumulative patch. 
REM    18-sep-2007  johri05  16212586 
REM    19-sep-2007  hopst01  Check for mdb version before applying cumulative patch
REM    25-sep-2007  hopst01  Add additional precheck functionality
REM    25-oct-2007  hopst01  Add call to precheck_extention
REM    31-oct-2007  hopst01  b1638 Add precheck messages re tblspace and mdbadmin creation
REM                          b1631 tblspace_path only necessary if creating tablespaces
REM     6-nov-2007  hopst01  b1636 Add precheck call to Java to setupmdb.  
REM                                Remove printing of ERR_MESSAGE as java 
REM                                check in setupmdb might change the value
REM     7-nov-2007  hopst01  b1637 Integrate windows script with status log
REM    21-nov-2007  hopst01  b1527 Check if use has "SYS" privileges
REM    29-nov-2007  johri05  b1628 Consistency of logged messages time stamps
REM                      Add :GET_DT--Get Date/Time func to fmt logged timestamp
REM                      to match Java Utilities and LinuxUnix install scripts.
REM    29-nov-2007  hopst01        Remove quotes around precheck message output
REM    10-dec-2007  hopst01  b1631 Add missing done_exit label to tblspace check
REM    14-dec-2007  johri05  b1628 Consistency of logged messages time stamps
REM                      Copy fixed :GET_DT func from setupmdb.bat
REM                      Fixed :GET_DT--"Get Date/Time" func, to always use 24-hr fmt
REM                      Also fixed to use %TMPDIR% for temporary (writable) directory.
REM    21-dec-2007  hopst01  16587331 - Update precheck messages
REM    19-feb-2008  hopst01  Ensure that debug level is restore after call to PatchMDB
REM    20-feb-2008  hopst01  p16776291 - ORACLE 9I OUTER JOIN BUG, split role query
REM    07-apr-2008  hopst01  b1740 - quote oracle tablespace path since parsed value no longer contains quotes
REM    14-feb-2008  hopst01  b1787 bugmail - Windows OMF
REM    14-jul-2008  hopst01  b1766 - PRECHECK FAILURE (Star 17044308). Always copy precheck files.
REM    18-jul-2008  hopst01  Star 17294367 DBPASSWORD WITH "," FAILED. Treat DBPASSWORD like 
REM                          MDB_ADMIN_PSWD.  Use temp variable when comparing it to an 
REM                          empty string.
REM    06-aug-2008  hopst01  STAR 17177592 - Conversion unnecessary.  If MDB 1.0.4 present only install
REM                          MDB Base 1.5 schema. If 1.5 database is 1.0.4 compatible don't 
REM                          process any XML files. Add mdb_version check and setting of 
REM                          flags MDB_VERSION_EXIST and IS_104_COMPATIBLE     
REM    08-aug-2008  johri05  b1882-17149528-MDBTOOLS DATE TIME ERROR LOG
REM                          Fix to use only minimal short format to prefix logged messages.
REM                          Long format is to be used once in the install log header.
REM                             long fmt:  yyyy-MMM-dd,HH:mm:ss-zzzz -ex:2008-jul-04,15:30:00-0400
REM                             short fmt:       MM-dd,HH:mm:ss      -ex:      07-04,15:30:00
REM                          Del :GET_DT function
REM                          Chg calls to :GET_DT label to be calls to mdbFuncs with GET_DT argument
REM    11-aug-2008  hopst01  b1882-17149528-MDBTOOLS DATE TIME ERROR LOG. Target Directory needs to 
REM                          be quoted. Don't pipe calls to mdbFuncs to nul as it makes debug impossible.
REM    12-aug-2008  hopst01  STAR 17398516 - UNVER MDB PSWD DURING PRECHK. Verify mdbadmin connection if
REM                          mdbadmin exists.
REM    13-aug-2008  hopst01  Star 17401377-WRONG MSG DURING PRCHK FAIL. Handle negative return codes from
REM                          SQLServer command line utility.
REM    18-aug-2008  hopst01  STAR 17402077 - Add synonym permission to mdbadmin 
REM                          and run synonym permission check
REM    20-aug-2008  hopst01  STAR 17419427 - ERROR IN PATCHMDB COMMAND. 
REM                          Remove redundant double quotes around LOGFILE variable
REM    21-aug-2008  hopst01  STAR 17425189 - QA:INSTALL AND PRECHECK FAILS. 
REM                          Fix mdbadmin user existence check.
REM    05-dec-2008  hopst01  17727007 - FOR LOOP W CMD NOT SUPPORTED. For loops that use 
REM                          a command to populate the for loop set should be replaced. 
REM    06-sep-2010  kansa04  19536759 - setupmdb.bat fails if it is passed an Oracle tablespace path 
REM			     containing a space for Oracle 11g
REM   
set MDB_RC=210
SET MESSAGE_NAME=
SET MESSAGE_TEXT=

rem
rem No log file while pre-checking.
rem
IF /I  "%PRECHECK%" == "yes"   set LOGFILE=nul

set ORA_CONNECT_ID=%DBNAME%
set ORA_SYS_PSWD=%DBPASSWORD%
set T_ORA_SYS_PSWD=x%DBPASSWORD%x
set T_ORA_SYS_PSWD="%T_ORA_SYS_PSWD:"=%"
set ORA_SYS_USER=sys
IF (%DBUSER%) NEQ () set ORA_SYS_USER=%DBUSER%


rem
rem Set a writable TMPDIR for general use. (use tgt dir for pre-script)
rem Pre-script requires a validated tgt dir.
rem  (also requires current dir to be the tgt dir)
rem
set TMPDIR=%MDB_TARGET_DIR%


rem
rem check required arguments
rem

IF (%ORA_CONNECT_ID%) NEQ () GOTO REQ_ARG1
set ERR_MESSAGE=Missing required argument: DBNAME (Oracle connection id)
GOTO DONE_EXIT

:REQ_ARG1
IF (%T_ORA_SYS_PSWD%) NEQ ("xx") GOTO REQ_ARG2
set ERR_MESSAGE=Missing required argument: DBPASSWORD (Oracle SYS password)
GOTO DONE_EXIT

:REQ_ARG2

:REQ_ARG3
SET T_MDB_ADMIN_PSWD=
SET T_MDB_ADMIN_PSWD=x%MDB_ADMIN_PSWD%x
SET T_MDB_ADMIN_PSWD="%T_MDB_ADMIN_PSWD:"=%"
IF (%T_MDB_ADMIN_PSWD%) NEQ ("xx") GOTO EXC_ARG
set ERR_MESSAGE=Missing required argument: MDB_ADMIN_PSWD
GOTO DONE_EXIT

:EXC_ARG

rem
rem copy for precheck
rem
IF /I "%PRECHECK%" == "yes" IF /I NOT "%MDB_TARGET_DIR%" == "%MDB_SOURCE_DIR%" xcopy "%MDB_SOURCE_DIR%\%DBVENDOR%\precheck_extension.bat" .\ /Q /Y /R > nul 2>&1

rem
rem check user tablespace arguments
rem
IF (%ORA_DATA_TBLSPACE_NAME%) EQU () IF (%ORA_INDEX_TBLSPACE_NAME%) EQU () GOTO USE_DEF_TBLS
GOTO CHECK_TBLS_NAME
:USE_DEF_TBLS
set ORA_DATA_TBLSPACE_NAME=MDB_DATA
set ORA_INDEX_TBLSPACE_NAME=MDB_INDEX

:CHECK_TBLS_NAME
IF (%ORA_DATA_TBLSPACE_NAME%) NEQ () IF (%ORA_INDEX_TBLSPACE_NAME%) EQU () GOTO DATA_NO_INDEX_ERROR
GOTO TBL_CHECK2
:DATA_NO_INDEX_ERROR
set ERR_MESSAGE=Error: ORA_DATA_TBLSPACE is specified but ORA_INDEX_TBLSPACE is missing
GOTO DONE_EXIT

:TBL_CHECK2
IF (%ORA_INDEX_TBLSPACE_NAME%) NEQ () IF (%ORA_DATA_TBLSPACE_NAME%) EQU () GOTO INDEX_NO_DATA_ERROR
GOTO TBL_CHECK3
:INDEX_NO_DATA_ERROR
set ERR_MESSAGE=Error: ORA_INDEX_TBLSPACE is specified but ORA_DATA_TBLSPACE is missing
GOTO DONE_EXIT

:TBL_CHECK3
set MISSING_TBLSPACE=
IF (%ORA_DATA_TBLSPACE_NAME%)  EQU () SET MISSING_TBLSPACE=ORA_DATA_TBLSPACE
IF (%ORA_INDEX_TBLSPACE_NAME%) EQU () SET MISSING_TBLSPACE=ORA_INDEX_TBLSPACE
IF (%MISSING_TBLSPACE%) EQU () GOTO CHECK_CLIENT_AVAILABLE
set ERR_MESSAGE=Missing required arguemnt %MISSING_TBLSPACE%
GOTO DONE_EXIT

:CHECK_CLIENT_AVAILABLE
rem
rem check if Oracle is available
rem
sqlplus -V > nul 2>&1
IF %ERRORLEVEL% NEQ 9009 (
    set SQL_CMD=sqlplus
    GOTO OSQL_FOUND
)
set MDB_RC=10
set ERR_MESSAGE=Cannot find sqlplus. Check that Oracle has been installed and in the PATH
GOTO DONE_EXIT
:OSQL_FOUND



REM 
REM check oracle connectivity
REM

echo quit | %SQL_CMD% -L "%ORA_SYS_USER%/%ORA_SYS_PSWD%@%ORA_CONNECT_ID%%ORA_AS_SYSDBA%" > nul 2>&1
if (%ERRORLEVEL%) EQU (0) GOTO CONN_OK

echo quit | %SQL_CMD% -L "%ORA_SYS_USER%/%ORA_SYS_PSWD%@%ORA_CONNECT_ID%%ORA_AS_SYSDBA%" >> %LOGFILE% 2>&1
set MDB_RC=104
set ERR_MESSAGE=Oracle connectivity error, see logfile for details
IF /I  "%PRECHECK%" == "yes"   set ERR_MESSAGE=Oracle connectivity error, unable to connect as user %ORA_SYS_USER% 
GOTO DONE_EXIT

:CONN_OK
REM
REM check to see if user has "SYS" privileges
REM
SET IS_DBA=no
:CHECK_CAT_PRIVS
%SQL_CMD% -L "%ORA_SYS_USER%/%ORA_SYS_PSWD%@%ORA_CONNECT_ID%%ORA_AS_SYSDBA%" @.\DDL\check_cat_privs.sql >>%LOGFILE% 2>&1
set RC=%ERRORLEVEL%
IF %RC% EQU 0 GOTO CHECK_ROLE_PRIVS
SET ERR_MESSAGE=User missing privilege -- SELECT ON DBA_ROLES 
IF /I NOT  "%PRECHECK%" == "yes" SET ERR_MESSAGE=%ERR_MESSAGE%, see logfile for details.
set MDB_RC=18
GOTO DONE_EXIT

:CHECK_ROLE_PRIVS
%SQL_CMD% -L "%ORA_SYS_USER%/%ORA_SYS_PSWD%@%ORA_CONNECT_ID%%ORA_AS_SYSDBA%" @.\DDL\check_role_privs_b.sql >>%LOGFILE% 2>&1
set RC=%ERRORLEVEL%
IF %RC% NEQ 0 SET IS_DBA=yes&& GOTO CHECK_SYN_PRIVS

:CHECK_ROLE_PRIVS_B
%SQL_CMD% -L "%ORA_SYS_USER%/%ORA_SYS_PSWD%@%ORA_CONNECT_ID%%ORA_AS_SYSDBA%" @.\DDL\check_role_privs.sql >>%LOGFILE% 2>&1
set RC=%ERRORLEVEL%
IF %RC% NEQ 0 GOTO CHECK_SYN_PRIVS
set ERR_MESSAGE=User missing privilege -- CREATE ROLE
IF /I NOT  "%PRECHECK%" == "yes" SET ERR_MESSAGE=%ERR_MESSAGE%, see logfile for details.
set MDB_RC=19
GOTO DONE_EXIT

:CHECK_SYN_PRIVS
IF "%IS_DBA%" EQU "yes" GOTO ROLE_PRIVS_CONFIMRED 
%SQL_CMD% -L "%ORA_SYS_USER%/%ORA_SYS_PSWD%@%ORA_CONNECT_ID%%ORA_AS_SYSDBA%" @.\DDL\check_syn_privs.sql >>%LOGFILE% 2>&1
set RC=%ERRORLEVEL%
IF %RC% EQU 3 GOTO ROLE_PRIVS_CONFIMRED
set ERR_MESSAGE=User missing synonym management privileges 
IF /I NOT  "%PRECHECK%" == "yes" SET ERR_MESSAGE=%ERR_MESSAGE%, see logfile for details.
set MDB_RC=128
GOTO DONE_EXIT
:ROLE_PRIVS_CONFIMRED

REM
REM Determine if OMF files are being used
REM
:CHECK_OMF_FILE
%SQL_CMD% -L "%ORA_SYS_USER%/%ORA_SYS_PSWD%@%ORA_CONNECT_ID%%ORA_AS_SYSDBA%" @.\DDL\check_omf.sql >>%LOGFILE% 2>&1
set RC=%ERRORLEVEL%
IF %RC% EQU 0 GOTO NO_OMF
IF %RC% LSS 0 GOTO CHECK_OMF_FILE_ERROR
if %RC% NEQ 2 GOTO SET_OMF

:CHECK_OMF_FILE_ERROR
set MDB_RC=110
set ERR_MESSAGE=Problem accessing Oracle data, could be related to lack of user privileges
IF /I NOT  "%PRECHECK%" == "yes" set ERR_MESSAGE=%ERR_MESSAGE%, see logfile for details.
GOTO DONE_EXIT

:SET_OMF 
set OMF_ENABLED=1

IF ("%ORA_TBLSPACE_PATH%") NEQ () GOTO USE_CMT
set CREATE_MDB_TBLSPACE_SQL=create_mdb_tblspace_omf.sql
GOTO CHECK_OMF_FILE_COMPLETED

:NO_OMF
set OMF_ENABLED=0

:USE_CMT
set CREATE_MDB_TBLSPACE_SQL=create_mdb_tblspace.sql

:CHECK_OMF_FILE_COMPLETED

REM check if the mdbadmin user exists 
REM
IF /I  "%PRECHECK%" == "yes" GOTO CHECK_USER
call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_DT  "%MDB_TARGET_DIR%"
echo %DT% >> %LOGFILE%
echo %DT% Checking if mdbadmin exists...
echo %DT% Checking if mdbadmin exists... >>%LOGFILE% 2>&1
echo %DT% >> %LOGFILE%
:CHECK_USER
%SQL_CMD% -L "%ORA_SYS_USER%/%ORA_SYS_PSWD%@%ORA_CONNECT_ID%%ORA_AS_SYSDBA%" @.\DDL\find_oracle_user.sql mdbadmin >>%LOGFILE% 2>&1
set RC=%ERRORLEVEL%
IF %RC% EQU 0 GOTO CREATE_MDBADMIN_USER
set CREATE_MDBADMIN=no
IF %RC% EQU 1 GOTO VERIFY_MDBADMIN_CONN
set MDB_RC=103
set ERR_MESSAGE=%SQL_CMD% command error, ERRORLEVEL=%RC%, see logfile for details
IF /I  "%PRECHECK%" == "yes"  set ERR_MESSAGE=%SQL_CMD% command error searching for mdbadmin user, ERRORLEVEL=%RC%
GOTO DONE_EXIT

:CREATE_MDBADMIN_USER
set CREATE_MDBADMIN=yes
GOTO CHECK_DATA_EXIST

REM 
REM check oracle connectivity for mdbadmin
REM
:VERIFY_MDBADMIN_CONN
echo quit | %SQL_CMD% -L "mdbadmin/%MDB_ADMIN_PSWD%@%ORA_CONNECT_ID%" > nul 2>&1
if (%ERRORLEVEL%) EQU (0) GOTO CHECK_DATA_EXIST

echo quit | %SQL_CMD% -L "mdbadmin/%MDB_ADMIN_PSWD%@%ORA_CONNECT_ID%" >> %LOGFILE% 2>&1
set MDB_RC=127
set ERR_MESSAGE=Oracle connectivity error connecting as mdbadmin, see logfile for details
IF /I  "%PRECHECK%" == "yes"   set ERR_MESSAGE=Oracle connectivity error, unable to connect as user mdbadmin 
GOTO DONE_EXIT



:CHECK_DATA_EXIST

REM
REM check if the mdb data tablespace exists 
REM
IF /I  "%PRECHECK%" == "yes" GOTO CHECK_DATA
call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_DT  "%MDB_TARGET_DIR%"
echo %DT% >> %LOGFILE%
echo %DT% Checking if %ORA_DATA_TBLSPACE_NAME% exists...
echo %DT% Checking if %ORA_DATA_TBLSPACE_NAME% exists... >>%LOGFILE% 2>&1
echo %DT% >> %LOGFILE%
:CHECK_DATA
%SQL_CMD% -L "%ORA_SYS_USER%/%ORA_SYS_PSWD%@%ORA_CONNECT_ID%%ORA_AS_SYSDBA%" @.\DDL\mdb_tablespace.sql %ORA_DATA_TBLSPACE_NAME% >>%LOGFILE% 2>&1
set RC=%ERRORLEVEL%
IF %RC% EQU 0 GOTO CREATE_DATA
IF %RC% EQU 1 GOTO CHECK_INDEX_EXIST
set MDB_RC=100
set ERR_MESSAGE=%SQL_CMD% command error, ERRORLEVEL=%RC%, see logfile for details
IF /I  "%PRECHECK%" == "yes" set ERR_MESSAGE=%SQL_CMD% command error search for tablespace %ORA_DATA_TBLSPACE_NAME%, ERRORLEVEL=%RC%
GOTO DONE_EXIT

:CREATE_DATA
set CREATE_DATA_TBLSPACE=yes




:CHECK_INDEX_EXIST
REM
REM check if the mdb index tablespace exists 
REM
IF /I  "%PRECHECK%" == "yes" GOTO CHECK_INDEX
call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_DT  "%MDB_TARGET_DIR%"
echo %DT% >> %LOGFILE%
echo %DT% Checking if %ORA_INDEX_TBLSPACE_NAME% exists...
echo %DT% Checking if %ORA_INDEX_TBLSPACE_NAME% exists... >>%LOGFILE% 2>&1
echo %DT% >> %LOGFILE%
:CHECK_INDEX
%SQL_CMD% -L "%ORA_SYS_USER%/%ORA_SYS_PSWD%@%ORA_CONNECT_ID%%ORA_AS_SYSDBA%" @.\DDL\mdb_tablespace.sql %ORA_INDEX_TBLSPACE_NAME% >>%LOGFILE% 2>&1
set RC=%ERRORLEVEL%
IF %RC% EQU 0 GOTO CREATE_INDEX
IF %RC% EQU 1 GOTO CHECK_TBLSPACE_PATH
set MDB_RC=300
set ERR_MESSAGE=%SQL_CMD% command error, ERRORLEVEL=%RC%, see logfile for details
IF /I  "%PRECHECK%" == "yes" set ERR_MESSAGE=%SQL_CMD% command error search for tablespace %ORA_DATA_TBLSPACE_NAME%, ERRORLEVEL=%RC%
GOTO DONE_EXIT

:CREATE_INDEX
set CREATE_INDEX_TBLSPACE=yes

:CHECK_TBLSPACE_PATH
rem 
rem determine if we need the tablespace path as a part of install
rem only required if we need to create a tablespace
rem
IF ("%ORA_TBLSPACE_PATH%")   NEQ ("") GOTO LOAD_MDB_DDL_PRECHECK
IF (%CREATE_DATA_TBLSPACE%)  EQU (yes) GOTO CHECK_CREATE_FILE
IF (%CREATE_INDEX_TBLSPACE%) EQU (yes) GOTO CHECK_CREATE_FILE
GOTO LOAD_MDB_DDL_PRECHECK
:CHECK_CREATE_FILE
IF (%CREATE_MDB_TBLSPACE_SQL%) EQU (create_mdb_tblspace_omf.sql) GOTO LOAD_MDB_DDL_PRECHECK 
set ERR_MESSAGE=Missing required argument: ORA_TBLSPACE_PATH
set MDB_RC=210
GOTO DONE_EXIT

:LOAD_MDB_DDL_PRECHECK
rem
rem if the mdbadmin user isnt installed then pre-check can exist successfully
rem if the mdbadmin user is installed, check the version of the mdb
rem
IF /I NOT "%PRECHECK%" == "yes" GOTO LOAD_MDB_DDL

rem
rem print tablespace information
rem
:PRINT_TBLSPACE_INFO
IF (%CREATE_DATA_TBLSPACE%) EQU (no) GOTO PRINT_NEW_INDEX_INFO
:PRINT_NEW_DATA_INFO
SET MESSAGE_NAME=CREATING_DATA_TBLSPACE_NAME
SET MESSAGE_TEXT=%ORA_DATA_TBLSPACE_NAME%
SET MESSAGE_TEXT=###%MESSAGE_TEXT%###
SET MESSAGE_TEXT=%MESSAGE_TEXT:"###=%
SET MESSAGE_TEXT=%MESSAGE_TEXT:###"=%
SET MESSAGE_TEXT=%MESSAGE_TEXT:###=%
CALL :PRINT_MSG

SET MESSAGE_NAME=CREATING_DATA_TBLSPACE_SIZE
SET MESSAGE_TEXT=%MDBDATA_SIZE%
CALL :PRINT_MSG

:PRINT_NEW_INDEX_INFO
IF (%CREATE_INDEX_TBLSPACE%) EQU (no) GOTO DONE_PRINT_TBLSPACE_INFO
SET MESSAGE_NAME=CREATING_INDEX_TBLSPACE_NAME
SET MESSAGE_TEXT=%ORA_INDEX_TBLSPACE_NAME%
SET MESSAGE_TEXT=###%MESSAGE_TEXT%###
SET MESSAGE_TEXT=%MESSAGE_TEXT:"###=%
SET MESSAGE_TEXT=%MESSAGE_TEXT:###"=%
SET MESSAGE_TEXT=%MESSAGE_TEXT:###=%
CALL :PRINT_MSG

SET MESSAGE_NAME=CREATING_INDEX_TBLSPACE_SIZE
SET MESSAGE_TEXT=%MDBINDEX_SIZE%
CALL :PRINT_MSG

:DONE_PRINT_TBLSPACE_INFO
IF (%CREATE_MDBADMIN%) EQU (no) GOTO CHECK_MDB_VERSION_SKIP_ECHO
SET MESSAGE_NAME=CREATING_MDBADMIN
SET MESSAGE_TEXT=yes
CALL :PRINT_MSG
set MDB_RC=0
set ERR_MESSAGE=Pre-check OKAY ( will install new MDB database ) 

GOTO DONE_EXIT


:LOAD_MDB_DDL

IF (%CREATE_DATA_TBLSPACE%) EQU (no) GOTO END_DATA_CREATED
REM
REM Create mdb data tablespace
REM
call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_DT  "%MDB_TARGET_DIR%"
echo %DT% >> %LOGFILE%
echo %DT% Creating %ORA_DATA_TBLSPACE_NAME% tablespace in the %ORA_CONNECT_ID% database...
echo %DT% Creating %ORA_DATA_TBLSPACE_NAME% tablespace in the %ORA_CONNECT_ID% database... >>%LOGFILE%
echo %DT% >> %LOGFILE%
echo %DT% %SQL_CMD% -L "%ORA_SYS_USER%/########@%ORA_CONNECT_ID%%ORA_AS_SYSDBA%" @.\DDL\%CREATE_MDB_TBLSPACE_SQL% %ORA_DATA_TBLSPACE_NAME% "%ORA_TBLSPACE_PATH%" %MDBDATA_SIZE% >>%LOGFILE% 2>&1
%SQL_CMD% -L "%ORA_SYS_USER%/%ORA_SYS_PSWD%@%ORA_CONNECT_ID%%ORA_AS_SYSDBA%" @.\DDL\%CREATE_MDB_TBLSPACE_SQL% %ORA_DATA_TBLSPACE_NAME% "%ORA_TBLSPACE_PATH%" %MDBDATA_SIZE% >>%LOGFILE% 2>&1
IF %ERRORLEVEL% EQU 0 GOTO DATA_CREATED
set MDB_RC=150
set ERR_MESSAGE=Failed to create the %ORA_DATA_TBLSPACE_NAME% tablespace (errorlevel=%ERRORLEVEL%)
GOTO DONE_EXIT
:DATA_CREATED
CALL :INCREMENT_STATUS 1
set DB_CREATED=true
:END_DATA_CREATED



IF (%CREATE_INDEX_TBLSPACE%) EQU (no) GOTO END_INDEX_CREATED
REM
REM Create mdb index tablespace
REM
call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_DT  "%MDB_TARGET_DIR%"
echo %DT% >> %LOGFILE%
echo %DT% Creating %ORA_INDEX_TBLSPACE_NAME% tablespace in the %ORA_CONNECT_ID% database...
echo %DT% Creating %ORA_INDEX_TBLSPACE_NAME% tablespace in the %ORA_CONNECT_ID% database... >>%LOGFILE%
echo %DT% >> %LOGFILE%
echo %DT% %SQL_CMD% -L "%ORA_SYS_USER%/########@%ORA_CONNECT_ID%%ORA_AS_SYSDBA%" @.\DDL\%CREATE_MDB_TBLSPACE_SQL% %ORA_INDEX_TBLSPACE_NAME% "%ORA_TBLSPACE_PATH%" %MDBINDEX_SIZE% >>%LOGFILE% 2>&1
%SQL_CMD% -L "%ORA_SYS_USER%/%ORA_SYS_PSWD%@%ORA_CONNECT_ID%%ORA_AS_SYSDBA%" @.\DDL\%CREATE_MDB_TBLSPACE_SQL% %ORA_INDEX_TBLSPACE_NAME% "%ORA_TBLSPACE_PATH%" %MDBINDEX_SIZE% >>%LOGFILE% 2>&1
IF %ERRORLEVEL% EQU 0 GOTO INDEX_CREATED
set MDB_RC=350
set ERR_MESSAGE=Failed to create the %ORA_INDEX_TBLSPACE_NAME% tablespace (errorlevel=%ERRORLEVEL%)
GOTO DONE_EXIT
:INDEX_CREATED
CALL :INCREMENT_STATUS 1
set DB_CREATED=true
set DB_CREATED_INDEX=true
:END_INDEX_CREATED



set DB_CREATED_USER=false
IF (%CREATE_MDBADMIN%) EQU (no) GOTO END_MDBADMIN
REM
REM create mdbadmin user
REM
call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_DT  "%MDB_TARGET_DIR%"
echo %DT% >> %LOGFILE%
echo %DT% Creating mdbadmin user in the %ORA_CONNECT_ID% database...
echo %DT% Creating mdbadmin user in the %ORA_CONNECT_ID% database... >>%LOGFILE%
echo %DT% >> %LOGFILE%
echo %DT% %SQL_CMD% -L "%ORA_SYS_USER%/########@%ORA_CONNECT_ID%%ORA_AS_SYSDBA%" @.\DDL\create_mdb_oracle.sql ######## %ORA_DATA_TBLSPACE_NAME% >>%LOGFILE% 2>&1
%SQL_CMD% -L "%ORA_SYS_USER%/%ORA_SYS_PSWD%@%ORA_CONNECT_ID%%ORA_AS_SYSDBA%" @.\DDL\create_mdb_oracle.sql %MDB_ADMIN_PSWD_DQ% %ORA_DATA_TBLSPACE_NAME% %ORA_INDEX_TBLSPACE_NAME% >>%LOGFILE% 2>&1
IF %ERRORLEVEL% EQU 0 GOTO MDBADMIN_CREATED
set MDB_RC=400
set ERR_MESSAGE=Failed to create the mdbadmin user (errorlevel=%ERRORLEVEL%)
GOTO DONE_EXIT
:MDBADMIN_CREATED
CALL :INCREMENT_STATUS 1
set DB_CREATED_USER=true

:END_MDBADMIN
call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_DT  "%MDB_TARGET_DIR%"
echo %DT% >> %LOGFILE%
echo %DT% Granting mdbadmin synonym management permissions in %ORA_CONNECT_ID% database... >>%LOGFILE%
echo %DT% >> %LOGFILE%
echo %DT% %SQL_CMD% -L "%ORA_SYS_USER%/########@%ORA_CONNECT_ID%%ORA_AS_SYSDBA%" @.\DDL\mdb_synonym_perm.sql  >>%LOGFILE% 2>&1
%SQL_CMD% -L "%ORA_SYS_USER%/%ORA_SYS_PSWD%@%ORA_CONNECT_ID%%ORA_AS_SYSDBA%" @.\DDL\mdb_synonym_perm.sql >>%LOGFILE% 2>&1
IF %ERRORLEVEL% EQU 0 GOTO END_MDBADMIN_SYN_PERM 
set MDB_RC=129
set ERR_MESSAGE=Unable to grant mdbadmin synonym management permissions (errorlevel=%ERRORLEVEL%)
GOTO DONE_EXIT
:END_MDBADMIN_SYN_PERM

REM
REM Apply MDB 1.0.4 Cumulative Patch if target database is MDB 1.0.4.
REM Check Version, if 1.0.4 then apply patch.
REM
IF (%DB_CREATED_USER%) EQU (true) GOTO END_CUMULATIVE_PATCH

:CHECK_MDB_VERSION
REM
REM determine if the mdb_version table exists
REM 
%SQL_CMD% -L "%ORA_SYS_USER%/%ORA_SYS_PSWD%@%ORA_CONNECT_ID%%ORA_AS_SYSDBA%" @.\DDL\mdb_version_tbl_check.sql >>%LOGFILE% 2>&1
set RC=%ERRORLEVEL%
IF EXIST version.txt del version.txt
IF %RC% GTR 100000 SET MDB_VERSION_EXIST=yes
SET LOCAL_VERSION=

REM
REM check the existing mdb version
REM
call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_DT  "%MDB_TARGET_DIR%"
echo %DT% >> %LOGFILE%
echo %DT% Checking the existing %DBNAME% version...
echo %DT% Checking the existing %DBNAME% version... >>%LOGFILE% 2>&1
echo %DT% >> %LOGFILE%
:CHECK_MDB_VERSION_SKIP_ECHO
SET MESSAGE_NAME=CREATING_MDBADMIN
SET MESSAGE_TEXT=no
IF /I "%PRECHECK%" == "yes" CALL :PRINT_MSG
%SQL_CMD% -L "%ORA_SYS_USER%/%ORA_SYS_PSWD%@%ORA_CONNECT_ID%%ORA_AS_SYSDBA%" @.\DDL\mdb_version.sql >>%LOGFILE% 2>&1
set RC=%ERRORLEVEL%
IF NOT EXIST version.txt echo none; > version.txt
FOR /F "tokens=1 delims=;" %%a in (version.txt) do (
  set LOCAL_VERSION=%%a
)
IF EXIST version.txt del version.txt
IF %RC% EQU 104000   SET IS_104_COMPATIBLE=yes&& GOTO MDB_104_EXISTS
IF %RC% EQU 942      GOTO MDB_TABLE_NOT_EXIST
IF %RC% LEQ 100000   GOTO MDB_VER_ERROR
IF %RC% LSS 105000   GOTO NO_AVAILABLE_UPGRADE
IF %RC% LSS 0        GOTO MDB_VER_ERROR
REM
REM must have version 1.05 or greater
REM
GOTO NO_UPGRADE

REM
REM Check for a cumulative patch
REM Apply MDB 1.0.4 Cumulative Patch if target database is MDB 1.0.4.
REM
:MDB_104_EXISTS
IF /I  NOT "%PRECHECK%" == "yes" GOTO END_PRECHECK
set MDB_RC=0
set ERR_MESSAGE=Pre-check OKAY ( will upgrade MDB which has version-1.0.4 )
GOTO DONE_EXIT
:END_PRECHECK

CALL :INCREMENT_STATUS 1
call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_DT  "%MDB_TARGET_DIR%"
echo %DT% >> %LOGFILE%
for /f "delims=," %%a in ("%CD%") do set SAVEPATCHDIR=%%a
IF (%MDB_PATCH_DIR%) EQU () set MDB_PATCH_DIR=CumulativePatch

IF NOT EXIST "%MDB_PATCH_DIR%" GOTO END_CUMULATIVE_PATCH
IF NOT EXIST "%MDB_PATCH_DIR%\GetPatchNumber.bat" GOTO NO_PATCH_NUM
IF NOT EXIST "%MDB_PATCH_DIR%\PatchMDB.bat" GOTO NO_PATCH_MDB

REM
REM Apply cumulative patch
REM

call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_DT  "%MDB_TARGET_DIR%"
echo %DT% Applying cumulative patch...                >> %LOGFILE% 2>&1


chdir /D "%MDB_PATCH_DIR%" >> %LOGFILE% 2>&1

call ".\GetPatchNumber" MDB_CUMULATIVE_PATCH	             >> %LOGFILE% 2>&1

call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_DT  "%MDB_TARGET_DIR%"
echo %DT% call PatchMDB -ORA_CONNECT_ID=%ORA_CONNECT_ID% -MDB_ADMIN_PSWD=###### -MDB_NAME=%DBNAME% -ORA_INDEX_TBLSPACE_NAME=%ORA_INDEX_TBLSPACE_NAME% -MDB_PATCH=%MDB_CUMULATIVE_PATCH% -MDB_TARGET_DIR="%MDB_TARGET_DIR%\oracle" -ORA_BACKUP=no  %MDB_PATCH_OPTIONS%                           >> %LOGFILE% 2>&1
call PatchMDB  -ORA_CONNECT_ID=%ORA_CONNECT_ID% -ORA_SYS_USER=%ORA_SYS_USER% -ORA_SYS_PSWD=%DBPASSWORD_DQ% -MDB_ADMIN_PSWD=%MDB_ADMIN_PSWD_DQ% -MDB_NAME=%DBNAME% -ORA_INDEX_TBLSPACE_NAME=%ORA_INDEX_TBLSPACE_NAME% -MDB_PATCH=%MDB_CUMULATIVE_PATCH% -MDB_TARGET_DIR="%MDB_TARGET_DIR%\oracle" -ORA_BACKUP=no
set T_RC=%ERRORLEVEL%
%DEBUGECHO%
IF %T_RC% EQU 0 GOTO PATCH_OK
set MDB_RC=%ERRORLEVEL%
set ERR_MESSAGE=Failed to apply patch %MDB_CUMULATIVE_PATCH% on database %DBNAME% (errorlevel=%ERRORLEVEL%)
chdir /D "%SAVEPATCHDIR%"                                    >> %LOGFILE% 2>&1
GOTO DONE_EXIT

:NO_PATCH_NUM

set MDB_RC=351
set ERR_MESSAGE=Warning: cannot apply cumulative patch because "%MDB_TARGET_DIR%\CumulativePatch\GetPatchNumber.bat" is not found 
chdir /D "%SAVEPATCHDIR%"                                    >> %LOGFILE% 2>&1
GOTO DONE_EXIT

:PATCH_OK
chdir /D "%SAVEPATCHDIR%"                                    >> %LOGFILE% 2>&1
CALL :INCREMENT_STATUS 5

:END_CUMULATIVE_PATCH

set ERR_MESSAGE=setupmdb_pre completed for the %ORA_CONNECT_ID% database on %DBHOST%
set MDB_RC=0
GOTO DONE_EXIT

:MDB_TABLE_NOT_EXIST
set ERR_MESSAGE=No MDB found in database %ORA_CONNECT_ID%
set MDB_RC=0
GOTO DONE_EXIT

:MDB_VER_ERROR
set MDB_RC=201
set ERR_MESSAGE=%SQL_CMD% command error checking MDB version, ERRORLEVEL=%RC%
IF /I NOT "%PRECHECK%" == "yes" set ERR_MESSAGE=%ERR_MESSAGE%, see logfile for details
GOTO DONE_EXIT

:NO_AVAILABLE_UPGRADE
set MDB_RC=200
set ERR_MESSAGE=MDB detected in database %ORA_CONNECT_ID% ... NO upgrade available.
GOTO DONE_EXIT

:NO_UPGRADE
set MDB_RC=0
set ERR_MESSAGE=Pre-check OKAY ( will upgrade MDB version-1.5 )
GOTO DONE_EXIT


:DONE_EXIT
REM
REM Handle precheck output.
REM
IF /I  NOT "%PRECHECK%" == "yes"   goto NORM_DONE_EXIT
IF %MDB_RC% EQU 0 CALL :PRECHECK_EXT
GOTO FINI

:NORM_DONE_EXIT
chdir /D "%MDB_TARGET_DIR%"                                    >> %LOGFILE% 2>&1
call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_DT  "%MDB_TARGET_DIR%"
echo %DT% %ERR_MESSAGE%
echo %DT% %ERR_MESSAGE%                               >> %LOGFILE% 2>&1
echo %DT% Setupmdb_pre exit /B return code=%MDB_RC%   >> %LOGFILE% 2>&1

:FINI
EXIT /B %MDB_RC%

rem
rem Additional prechecks that are not a part of the install process are
rem performed here.  The script sets both the return code and the err_message
rem
:PRECHECK_EXT
IF NOT EXIST precheck_extension.bat GOTO PRECHECK_EXT_ERR
call precheck_extension
GOTO DONE_PRECHECK_EXT
:PRECHECK_EXT_ERR
set MDB_RC=3
set ERR_MESSAGE=Missing pre-install script: %CD%\precheck_extension.bat
:DONE_PRECHECK_EXT
GOTO :EOF

rem
rem Prints out message in format message_name="message_text"
rem
:PRINT_MSG
IF (%PRECHECK_MSGFILE%) EQU ()  goto PRINT_STDOUT
echo %MESSAGE_NAME%=%MESSAGE_TEXT%       >> %PRECHECK_MSGFILE%
GOTO :EOF
:PRINT_STDOUT
echo %MESSAGE_NAME%=%MESSAGE_TEXT%
GOTO :EOF

:INCREMENT_STATUS
set /A STATUS=%STATUS% + %1%
rem
rem set SPACES to 1 or 2 spaces depending on the current status
rem value.  although quotes are unnessary, they make the reading
rem of the value easier
rem
set SPACES="  "
IF %STATUS% GTR 9 set SPACES=" "
rem strip quotes
set SPACES=%SPACES:"=%
echo %SPACES%%STATUS%%% %STATUSLINE% > %STATUSFILE%
GOTO :EOF

