@echo off

REM Name: setupmdb.bat 
REM
REM Description: 
REM         creates the MDB database
REM 
REM Syntax:
REM     See :USAGE_ERROR section at the end.
REM
REM History:
REM  27-jun-2006  mulwi01  created
REM  17-jul-2006  mulwi01  merged with SQLServer 1.0.4 script functionality
REM  27-jul-2006  mulwi01  added Oracle parameters
REM   2-aug-2006  mulwi01  added usage
REM   7-aug-2006  mulwi01  changed to mdbsetup.jar
REM  19-jan-2007  mulwi01  Star 15148450 Add more logging 
REM  25-jan-2007  mulwi01  fix previous regression  
REM   7-mar-2007  mulwi01  add optional JRE directory parameter
REM   7-mar-2007  hopst01  removed incorrect syntax comparision for quoted string mdbadmin
REM   1-apr-2007  hopst01  add java library path to allow for mssql dll inculsion
REM   7-jun-2007  hopst01  replace for loop with CD to set CURDIR.
REM                           causes empty log file when called through java
REM  11-sep-2007  lupmo01  removed PATH from java library path 
REM                             ( issue 16331864- PATH may contain invalid chars,
REM                               like quotes, that can prevent jvm to run )
REM   15-sep-2007  devjo01 STAR 16212586 changes
REM   19-sep-2007  johri05 b1486 1.0.5 in mdbsetup.bat,
REM                              also, set 19 into parens
REM   19-sep-2007  johri05 b1236 -pre-check environment option
REM   22-sep-2007  hopst01 fix pre-check copy, cleanup errors, dont delete target_dir if contains pre-check
REM   25-sep-2007  hopst01 handle case when pre-check dir is readonly. when running precheck handle case
REM                                source and target dirs are the same.
REM   27-sep-2007  hopst01 suppress output of precheck cleanup phase
REM    2-oct-2007  hopst01 b1627 - allow overwrite of files in target directory when copying target to source
REM   25-oct-2007  hopst01 b1630 - install fails when called from non-source directory 
REM                           and source and target are not the same
REM    1-nov-2007  hopst01 b1635 - use JRE on JRE_PATH if specified
REM    4-nov-2007  hopst01 b1619 - Java version check, allow versions greater than 1.5 (e.g. 2.0)
REM    7-nov-2007  hopst01 b1633 - Oracle default sys user
REM    8-nov-2007  hopst01 b1637 - Integrate windows script with status log 
REM   20-nov-2007  hopst01       - Remove quotes from around precheck values
REM   28-nov-2007  hopst01       - Correct typos.  Specify dbpassword when performing java precheck on Oracle
REM   29-nov-2007  hopst01       - Place return code last for precheck
REM   29-nov-2007  johri05  b1628 Consistency of logged messages time stamps
REM                      Add :GET_DT--Get Date/Time func to fmt logged timestamp
REM                      to match Java Utilities and LinuxUnix install scripts.
REM   30-nov-2007  hopst01       - set error code for java precheck failure, remove Ingres info from banner
REM    3-dec-2007  hopst01 b1661 - ORA FAILS SID MISMATCHED.  Add new parameter ORA_SERVICE_NAME to allow for
REM                                different service and net names.  Update usage info
REM    5-dec-2007  hopst01 b1635 - Quote path name, prevent blank path in error message
REM    6-dec-2007  hopst01       - Update scripts version to 20 
REM   14-dec-2007  johri05  b1628 Consistency of logged messages time stamps
REM                      Fix :GET_DT--"Get Date/Time" func, to always use 24-hr fmt
REM                      Also fix to use %TMPDIR% for temporary (writable) directory.
REM   21-dec-2007  hopst01  16587331 - only use default tblspace names if 
REM                                    both data and index aren't specified
REM   17-jan-2008  johri05  b1686-16705547-BETA: ORA Install Not Working
REM                      Tablespace path specified w/no tablespace names,
REM                      returns w/err=3 and no msg.
REM                      Add Oracle tablespace param tests.
REM    11-jan-2007 hopst01 b1658 - Batch script to install BFS
REM                              - Parse all command line arguments before throwing an error. 
REM                                Prevents need for TEMPDIR.
REM    07-feb-2008 hopst01 Add non-user command-line option: 
REM                        PRECEDENCE - will print the precedence number of this script
REM                                     NEVER SET PRECEDENCE TO 999 AS THIS IS BASE ERR RETURN CODE
REM                        LOG_DIR    - location of the install log and the message log
REM                        
REM                        Install now checks for MDB CCD in fixed location.  If found
REM                        atempts to install with scripts stored in CCD if newer. 
REM                        Regardless, xml, Cumulative patch are always copied to CCD prior
REM                        to install.  JRE is copied prior if no JRE is present in CCD.
REM                        
REM                        Full copy to CCD is performed upon successful install.  Logfile
REM                        with named date extention is also copied upon successful install.                          
REM    13-feb-2008 hospt01 make use of findcommon.bat to find/create MDB CCD
REM                        don't copy xml during precheck
REM                        mdb_common points to Windows sub-directory
REM                        pass source and target params into make104
REM                        remove space from copied log name stored in CCD
REM                        store msg file in log_dir if specified
REM    15-feb-2008 hopst01 ensure target_dir is deleted on precheck if created
REM                        PatchMDB, if patching look for XML tree in patchMDB specific location
REM                        use statusdir variable to ensure java created status file is in proper location
REM    16-feb-2008 hopst01 write out successful message to msg log upon successful install
REM                        don't write out msg log if failed install and err_message is blank. This is
REM                          the only way to persist the msg log created by make104 script
REM  19-feb-2008  hopst01  make sure that precheck doesn't create msg log.  Ensure that empty 
REM                           log is not written
REM  20-feb-2008  hopst01  add new command line parameter MDB_PRIVATE_DIR that overrides MDB CCD
REM                        ensure the vendor folder is copied to CCD if it doesn't already exist
REM  22-feb-2008  hopst01  quote password when passing to make104
REM                        change copy flags for xcopy command - mdb104 schema and jre
REM                        when apply mdb104 schema, only update mdb table if mdb104 driver 
REM                            succesfully installed. requires two java calls
REM  26-feb-2008  hopst01  update the version number of the scripts
REM  28-feb-2008  hopst01  change to the MDB_SOURCE_DIR before calling findcommon.bat
REM  07-apr-2008  hopst01  b1740 - quote oracle tablespace path when saving cmd line args, but leave value unquoted
REM  09-apr-2008  johri05  b1588r-16330943- 5-Sep pulka02  EA 1.5.19 M BETA:MDBTOOLS REQ PARMS
REM                         - (11feb08) (win) SqlSvr, DBUSER and DBPASSWORD parameters can be omitted for Windows Auth.
REM                         - (13mar) setupmdb "Usage" does NOT have COMPAT104, MDB_PRIVATE_DIR, MDB_COMMON_DIR options.
REM  15-apr-2008  hopst01  b1788 - MDB_COMMON s/b MDB_COMMON_DIR
REM  15-apr-2008  hopst01  Update precedence value to 3, change build number to 25
REM  18-apr-2008  hopst01  b1800 - Star: 17043818,BETA: QUOTES IN PATH ENV VAR
REM  18-apr-2008  hopst01  b1795 - Syncup make_MDB_104_compatible.bat
REM                         - create response file on successful install
REM                         - utilize '-batch' command line option when 
REM  06-may-2008  hopst01  b1822 - BETA: JRE PATH COMPAT104
REM  13-may-2008  hopst01  b1840 - New API to patchmdb
REM  17-jun-2008  hopst01  b1825 - Cryptic error if COMPAT104, and no MDB104 schema. 
REM                        Show error message when workspace/manifest missing
REM  14-jul-2008  hopst01  b1766 - PRECHECK FAILURE (Star 17044308). Always copy precheck files. 
REM                        Use /R when updating CCD, copying XML to CCD or copying files for precheck
REM  24-jul-2008  hopst01  Update mdb version
REM  18-jul-2008  hopst01  Star 17294367 DBPASSWORD WITH "," FAILED. Treat DBPASSWORD like 
REM                        MDB_ADMIN_PSWD.  Use temp variable when comparing it to an 
REM                        empty string.
REM  06-aug-2008  hopst01  STAR 17177592 - Conversion unnecessary.  If MDB 1.0.4 present only install
REM                          MDB Base 1.5 schema. For COMPAT104 and make_mdb_104_compatible do not
REM                          install schema if already 1.0.4 upgraded MDB.
REM  08-aug-2008  johri05  b1882-17149528-MDBTOOLS DATE TIME ERROR LOG
REM                        Fix to use only minimal short format to prefix logged messages.
REM                        Long format is to be used once in the install log header.
REM                           long fmt:  yyyy-MMM-dd,HH:mm:ss-zzzz -ex:2008-jul-04,15:30:00-0400
REM                           short fmt:       MM-dd,HH:mm:ss      -ex:      07-04,15:30:00
REM                        Del :GET_DT function
REM                        Chg calls to :GET_DT label to be calls to mdbFuncs with GET_DT argument
REM                        Add "CALL mdbFuncs GET_LONG_DT" to display long date format in log header.
REM  11-aug-2008  hopst01  b1882-17149528-MDBTOOLS DATE TIME ERROR LOG. Target Directory needs to 
REM                        be quoted. Don't pipe calls to mdbFuncs to nul as it makes debug impossible.
REM  11-aug-2008  hopst01  b1928 - STAR 17394032 Check MDB_TARGET_DIR to ensure that it is writeable.
REM  22-aug-2008  hopst01  STAR 17429282 - MDB_TARGET_DIR reported as readonly if folder 
REM                        does not pre-exist.
REM  27-aug-2008  hopst01  b1949 - Copy signature file and parameter file to CCD
REM  15-sep-2008  johri05  b1882-17149528-MDBTOOLS DATE TIME ERROR LOG-PRECHECK
REM                        Add calls to two new funcs -- GET_PRECHECK_DT and GET_PRECHECK_TM  
REM                           to set PRECHECK_DATE and PRECHECK_TM
REM                           with PRECHECK formats =>  yyyy-mm-dd  and HH:mm:ss.hh
REM  30-oct-2008  hopst01  STAR 17623061 - When CCD precedence is higher than source_dir precedence 
REM                        precheck looks for the files in the wrong location. Use MDB_SOURCE_DIR_ORG for precheck
REM   4-nov-2008  hopst01  STAR 17639530 - Setting MDB_SOURCE_DIR equal to CCD (ProgramFiles../MDB)  
REM                        will cause circular copy when updating the CCD after successful installation
REM  05-dec-2008  hopst01  17727007 - FOR LOOP W CMD NOT SUPPORTED. For loops that use 
REM                        a command to populate the for loop set should be replaced.
REM  09-jan-2009  lupmo01  17826194: read port number from registry for the local default instance of SQL Server 
REM			   in case DBPORT is not specified or it is zero
REM

setlocal

REM set DEBUGECHO=@ECHO on
set DEBUGECHO=@ECHO off
%DEBUGECHO%

rem
rem Version and build for log
rem
set MDB_VERSION=1.05.02 RE00043
set PRECEDENCE_VALUE=11

set CURDIR=%CD%
set SAVEPATH=%PATH%

REM
REM Set the default parameters
REM
set DBVENDOR=
set DBHOST=localhost
set DBNAME=MDB
set DBUSER=
set DBPASSWORD=
set DBDRIVER=
set WORKSPACE=
set WORKSPACE_FLD=
set II_MDB_SIZE=small
set /a II_MDB_SIZE_NUM=2
set II_MDB_CONFIG=yes
set PRECHECK=no
set PRECHECK_MSGDIR=
set PRECHECK_MSGFILE=
set TGTDIR_CREATED=no
set LOGDIR_CREATED=no
set DBPORT=
set DBPORT_ORIG=
set EXIT_SCRIPT=/B
set MDB_SOURCE_DIR=%CURDIR%
set MDB_TARGET_DIR=%CURDIR%
set MDB_PRIVATE_DIR=
set II_REINIT_CONFIG=FALSE
set JRE_DIR=
set STATUS=0
set STATUSFILE=mdbstatus.log
set STATUSLINE=(  0%% Tables,  0%% Constraints,  0%% Permissions,  0%% Views,  0%% Procedures,  0%% Triggers,  0%% Data)
rem
rem Tablespace names and sizes (in megabytes)
rem
set ORA_DATA_SIZE=
set ORA_INDEX_SIZE=
set MDBDATA_SIZE=400
set MDBINDEX_SIZE=100
set MDB_ADMIN_PSWD=

set CREATE_DATA_TBLSPACE=no
set CREATE_INDEX_TBLSPACE=no
set CREATE_MDBADMIN=no
set ORA_AS_SYSDBA= AS SYSDBA
set ORA_TBLSPACE_PATH=
set ORA_SERVICE_NAME=

set ERR_MESSAGE=
set LOGFILE=

set RC=999

set DB_CREATED=false
set DB_CREATED_INDEX=false
set DB_CREATED_USER=false

set SETUPMDB_COMMAND=setupmdb 
set COMPAT104=no
set MDB_COMMON=
set INVALID_ARG_NAME=
set IS_104_COMPATIBLE=no
set MDB_VERSION_EXIST=no
set IS_REC_CALL=no
rem used to store all other command line args, 
rem except driver, workspace, exit script, scr, trgt, mdb_common, precedence, log dir
set COMMAND_LINE_ARGS=

set PRECEDENCE=no
set COPY_TO_CCD=no
set EXTRA_PARAMS=
set LOG_NAME_EXT=
set STATUSDIR=
set MDB_SOURCE_DIR_ORG=

rem
rem Parse the command line parameters
rem
:PARSE_ARGS

IF "%~1" == ""   GOTO NO_MORE_ARGS

IF NOT "%~1" == "-DBVENDOR" GOTO NEXT_ARG1
SHIFT
set DBVENDOR=%~1
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -DBVENDOR=%1
set COMMAND_LINE_ARGS=%COMMAND_LINE_ARGS% -DBVENDOR=%~1
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG1
IF NOT "%~1" == "-DBUSER" GOTO NEXT_ARG2
SHIFT
set DBUSER=%~1
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -DBUSER=%1
set COMMAND_LINE_ARGS=%COMMAND_LINE_ARGS% -DBUSER=%~1
IF (%DBUSER%)==(sys) GOTO USR_SYSDBA
IF (%DBUSER%)==(SYS) GOTO USR_SYSDBA
set ORA_AS_SYSDBA=
:USR_SYSDBA
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG2
IF NOT "%~1" == "-DBPASSWORD" GOTO NEXT_ARG3
IF "%DBPASSSWORD%" EQU "" GOTO USE_CMD_LINE_DB
SHIFT
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -DBPASSWORD=***
set COMMAND_LINE_ARGS=%COMMAND_LINE_ARGS% -DBPASSWORD=%DBPASSWORD%
GOTO DONE_USE_CMD_LINE_DB

:USE_CMD_LINE_DB
SHIFT
set DBPASSWORD="%~1"
set DBPASSWORD_T=x%DBPASSWORD_SMDB%x
set DBPASSWORD_T="%DBPASSWORD_T:"=%"
IF (%DBPASSWORD_T%) EQU ("xx") GOTO CHECK_MDB104_PSWD_DB
set DBPASSWORD=%DBPASSWORD_SMDB%
GOTO USE_CMD_LINE_SET_VAL_DB

:CHECK_MDB104_PSWD_DB
set DBPASSWORD_T=x%DBPASSWORD_MAKE%x
set DBPASSWORD_T="%DBPASSWORD_T:"=%"
IF (%DBPASSWORD_T%) EQU ("xx") GOTO USE_CMD_LINE_SET_VAL_DB
set DBPASSWORD=%DBPASSWORD_MAKE%

:USE_CMD_LINE_SET_VAL_DB
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -DBPASSWORD=***
set COMMAND_LINE_ARGS=%COMMAND_LINE_ARGS% -DBPASSWORD=%DBPASSWORD%
rem Escape any reserved shell characters
set DBPASSWORD_DQ=%DBPASSWORD%
set DBPASSWORD=%DBPASSWORD:^=^^%
set DBPASSWORD=%DBPASSWORD:<=^<%
set DBPASSWORD=%DBPASSWORD:>=^>%
set DBPASSWORD=%DBPASSWORD:|=^|%
set DBPASSWORD=%DBPASSWORD:&=^&%
set DBPASSWORD=%DBPASSWORD:(=^(%
set DBPASSWORD=%DBPASSWORD:)=^)%
set DBPASSWORD=%DBPASSWORD:"=\"%
:DONE_USE_CMD_LINE_DB
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG3
IF NOT "%~1" == "-DBHOST" GOTO NEXT_ARG4
SHIFT
set DBHOST=%~1
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -DBHOST=%1
set COMMAND_LINE_ARGS=%COMMAND_LINE_ARGS% -DBHOST=%~1
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG4
IF NOT "%~1" == "-DBNAME" GOTO NEXT_ARG5
SHIFT
set DBNAME=%~1
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -DBNAME=%1
set COMMAND_LINE_ARGS=%COMMAND_LINE_ARGS% -DBNAME=%~1
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG5
IF NOT "%~1" == "-DBDRIVER" GOTO NEXT_ARG6
SHIFT
set DBDRIVER=%~1
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -DBDRIVER=%1
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG6
IF NOT "%~1" == "-WORKSPACE" GOTO NEXT_ARG7
SHIFT
set WORKSPACE=%~1
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -WORKSPACE=%1
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG7
IF NOT "%~1" == "-MDB_EXIT_SCRIPT" GOTO NEXT_ARG8
SHIFT
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -MDB_EXIT_SCRIPT=%1
IF NOT "%~1" == "yes" GOTO CHECK_NO
set EXIT_SCRIPT=/B
SHIFT
GOTO PARSE_ARGS
:CHECK_NO
IF NOT "%~1" == "no" GOTO XSCRIPT_ERR
set EXIT_SCRIPT=
SHIFT
GOTO PARSE_ARGS
:XSCRIPT_ERR
set ERR_MESSAGE=Invalid -MDB_EXIT_SCRIPT argument "%~1", expected yes or no
set RC=89
GOTO USAGE_ERROR

:NEXT_ARG8
IF NOT "%~1" == "-MDB_SOURCE_DIR" GOTO NEXT_ARG9
SHIFT
set MDB_SOURCE_DIR=%~1
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -MDB_SOURCE_DIR=%1
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG9
IF NOT "%~1" == "-MDB_TARGET_DIR" GOTO NEXT_ARG10
SHIFT
set MDB_TARGET_DIR=%~1
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -MDB_TARGET_DIR=%1
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG10
IF NOT "%~1" == "-debug" GOTO NEXT_ARG11
set DEBUGECHO=@ECHO ON
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -debug
set COMMAND_LINE_ARGS=%COMMAND_LINE_ARGS% -debug
%DEBUGECHO%
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG11
IF NOT "%~1" == "-II_REINIT_CONFIG" GOTO NEXT_ARG12
SHIFT
set II_REINIT_CONFIG=TRUE
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -II_REINIT_CONFIG
set COMMAND_LINE_ARGS=%COMMAND_LINE_ARGS% -II_REINIT_CONFIG
GOTO PARSE_ARGS

:NEXT_ARG12
IF NOT "%~1" == "-II_MDB_SIZE" GOTO NEXT_ARG12a
SHIFT
set II_MDB_SIZE=%~1
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -II_MDB_SIZE=%1
set COMMAND_LINE_ARGS=%COMMAND_LINE_ARGS% -II_MDB_SIZE=%~1
SHIFT

:TRY_SMALL
IF /i NOT "%II_MDB_SIZE%" == "small" GOTO TRY_MEDIUM
set II_MDB_SIZE=small
set /a II_MDB_SIZE_NUM=2
GOTO END_ARG11

:TRY_MEDIUM
IF /i NOT "%II_MDB_SIZE%" == "medium" GOTO TRY_LARGE
set II_MDB_SIZE=medium
set /a II_MDB_SIZE_NUM=3
GOTO END_ARG11

:TRY_LARGE
IF /i NOT "%II_MDB_SIZE%" == "large" GOTO TRY_INVALID
set II_MDB_SIZE=large
set /a II_MDB_SIZE_NUM=4
GOTO END_ARG11

:TRY_INVALID
set MDB_RC=99
set ERR_MESSAGE=Invalid II_MDB_SIZE argument "%II_MDB_SIZE%"
GOTO USAGE_ERROR
:END_ARG11
GOTO PARSE_ARGS

:NEXT_ARG12a
IF NOT "%~1" == "-DBPORT" GOTO NEXT_ARG13
SHIFT
set DBPORT=%~1
set DBPORT_ORIG=%DBPORT%
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -DBPORT=%1
set COMMAND_LINE_ARGS=%COMMAND_LINE_ARGS% -DBPORT=%~1
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG13
IF NOT "%~1" == "-II_MDB_CONFIG" GOTO NEXT_ARG13a
SHIFT
set II_MDB_CONFIG=%~1
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -II_MDB_CONFIG=%1
set COMMAND_LINE_ARGS=%COMMAND_LINE_ARGS% -II_MDB_CONFIG=%~1
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG13a
IF NOT "%~1" == "-PRECHECK" GOTO NEXT_ARG13b
set PRECHECK=yes
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -PRECHECK
set COMMAND_LINE_ARGS=%COMMAND_LINE_ARGS% -PRECHECK
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG13b
IF NOT "%~1" == "-PRECHECK_MSGDIR" GOTO NEXT_ARG14
SHIFT
set PRECHECK_MSGDIR=%~f1
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -PRECHECK_MSGDIR=%1
set COMMAND_LINE_ARGS=%COMMAND_LINE_ARGS% -PRECHECK_MSGDIR="%~f1%"
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG14
IF NOT "%~1" == "-ORA_TBLSPACE_PATH" GOTO NEXT_ARG15
SHIFT
set ORA_TBLSPACE_PATH=%~1
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -ORA_TBLSPACE_PATH="%1"
set COMMAND_LINE_ARGS=%COMMAND_LINE_ARGS% -ORA_TBLSPACE_PATH="%~1"
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG15
IF NOT "%~1" == "-ORA_DATA_SIZE" GOTO NEXT_ARG16
SHIFT
set MDBDATA_SIZE=%~1
set ORA_DATA_SIZE=%~1
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -ORA_DATA_SIZE=%1
set COMMAND_LINE_ARGS=%COMMAND_LINE_ARGS% -ORA_DATA_SIZE=%~1
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG16
IF NOT "%~1" == "-ORA_INDEX_SIZE" GOTO NEXT_ARG17
SHIFT
set MDBINDEX_SIZE=%~1
set ORA_INDEX_SIZE=%~1
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -ORA_INDEX_SIZE=%1
set COMMAND_LINE_ARGS=%COMMAND_LINE_ARGS% -ORA_INDEX_SIZE=%~1
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG17
IF NOT "%~1" == "-ORA_DATA_TBLSPACE" GOTO NEXT_ARG18
SHIFT
set ORA_DATA_TBLSPACE_NAME="%~1"
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -ORA_DATA_TBLSPACE=%1
set COMMAND_LINE_ARGS=%COMMAND_LINE_ARGS% -ORA_DATA_TBLSPACE=%~1
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG18
IF NOT "%~1" == "-ORA_INDEX_TBLSPACE" GOTO NEXT_ARG19
SHIFT
set ORA_INDEX_TBLSPACE_NAME="%~1"
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -ORA_INDEX_TBLSPACE=%1
set COMMAND_LINE_ARGS=%COMMAND_LINE_ARGS% -ORA_INDEX_TBLSPACE=%~1
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG19
IF NOT "%~1" == "-MDB_ADMIN_PSWD" GOTO NEXT_ARG20
IF "%MDB_ADMIN_PSWD%" EQU "" GOTO USE_CMD_LINE 
SHIFT
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -MDB_ADMIN_PSWD=***
set COMMAND_LINE_ARGS=%COMMAND_LINE_ARGS% -MDB_ADMIN_PSWD=%MDB_ADMIN_PSWD%
GOTO DONE_USE_CMD_LINE

:USE_CMD_LINE
SHIFT
set MDB_ADMIN_PSWD="%~1"
set MDB_ADMIN_PSWD_T=x%MDB_ADMIN_PSWD_SMDB%x
set MDB_ADMIN_PSWD_T="%MDB_ADMIN_PSWD_T:"=%"
IF (%MDB_ADMIN_PSWD_T%) EQU ("xx") GOTO CHECK_MDB104_PSWD
set MDB_ADMIN_PSWD=%MDB_ADMIN_PSWD_SMDB%
GOTO USE_CMD_LINE_SET_VAL

:CHECK_MDB104_PSWD
set MDB_ADMIN_PSWD_T=x%MDB_ADMIN_PSWD_MAKE%x
set MDB_ADMIN_PSWD_T="%MDB_ADMIN_PSWD_T:"=%"
IF (%MDB_ADMIN_PSWD_T%) EQU ("xx") GOTO USE_CMD_LINE_SET_VAL
set MDB_ADMIN_PSWD=%MDB_ADMIN_PSWD_MAKE%

:USE_CMD_LINE_SET_VAL
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -MDB_ADMIN_PSWD=***
set COMMAND_LINE_ARGS=%COMMAND_LINE_ARGS% -MDB_ADMIN_PSWD=%MDB_ADMIN_PSWD%
rem Escape any reserved shell characters
set MDB_ADMIN_PSWD_DQ=%MDB_ADMIN_PSWD%
set MDB_ADMIN_PSWD=%MDB_ADMIN_PSWD:^=^^%
set MDB_ADMIN_PSWD=%MDB_ADMIN_PSWD:<=^<%
set MDB_ADMIN_PSWD=%MDB_ADMIN_PSWD:>=^>%
set MDB_ADMIN_PSWD=%MDB_ADMIN_PSWD:|=^|%
set MDB_ADMIN_PSWD=%MDB_ADMIN_PSWD:&=^&%
set MDB_ADMIN_PSWD=%MDB_ADMIN_PSWD:(=^(%
set MDB_ADMIN_PSWD=%MDB_ADMIN_PSWD:)=^)%
set MDB_ADMIN_PSWD=%MDB_ADMIN_PSWD:"=\"%
:DONE_USE_CMD_LINE
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG20
IF NOT "%~1" == "-JRE_DIR" GOTO NEXT_ARG21
SHIFT
set JRE_DIR=%~1
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -JRE_DIR=%1
set COMMAND_LINE_ARGS=%COMMAND_LINE_ARGS% -JRE_DIR="%~1"
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG21
IF NOT "%~1" == "-MANIFEST" GOTO NEXT_ARG22
SHIFT
set DBDRIVER=%~1
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -MANIFEST=%1
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG22
IF NOT "%~1" == "-ORA_SERVICE_NAME" GOTO NEXT_ARG23
SHIFT
set ORA_SERVICE_NAME=%~1
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -ORA_SERVICE_NAME=%1
set COMMAND_LINE_ARGS=%COMMAND_LINE_ARGS% -ORA_SERVICE_NAME=%~1
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG23
IF NOT "%~1" == "-COMPAT104" GOTO NEXT_ARG24
SHIFT
set COMPAT104=yes
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -COMPAT104
GOTO PARSE_ARGS

:NEXT_ARG24
IF NOT "%~1" == "-MDB_COMMON_DIR" GOTO NEXT_ARG25
SHIFT
set MDB_COMMON=%~1
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -MDB_COMMON_DIR=%1
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG25
IF NOT "%~1" == "-PRECEDENCE" GOTO NEXT_ARG26
set PRECEDENCE=yes
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -PRECEDENCE=%1
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG26
IF NOT "%~1" == "-LOG_DIR" GOTO NEXT_ARG27
SHIFT
set LOG_DIR=%~1
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG27
IF NOT "%~1" == "-MDB_PRIVATE_DIR" GOTO NEXT_ARG28
SHIFT
set MDB_PRIVATE_DIR=%~1
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% -MDB_PRIVATE_DIR=%1
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG28
IF NOT "%~1" == "-rec" GOTO NEXT_ARG29
SHIFT
set IS_REC_CALL=yes
GOTO PARSE_ARGS

:NEXT_ARG29
IF NOT "%~1" == "-MDB_SOURCE_DIR_ORG" GOTO NEXT_ARG30
SHIFT
set MDB_SOURCE_DIR_ORG=%~1
SHIFT
GOTO PARSE_ARGS

:NEXT_ARG30
:INVALID_ARG
IF ("%INVALID_ARG_NAME%") EQU ("") SET INVALID_ARG_NAME=%~1
set SETUPMDB_COMMAND=%SETUPMDB_COMMAND% %1
SHIFT
GOTO PARSE_ARGS

:NO_MORE_ARGS

IF ("%INVALID_ARG_NAME%") EQU ("") GOTO NO_ARG_ERROR 
set ERR_MESSAGE=Usage: Invalid argument "%INVALID_ARG_NAME%"
set RC=90
GOTO USAGE_ERROR

:NO_ARG_ERROR

IF "%PRECEDENCE%" EQU "yes" GOTO PRINT_PRCDNC
GOTO DEF_INSTALL
:PRINT_PRCDNC
endlocal & exit /B %PRECEDENCE_VALUE% 

:DEF_INSTALL
REM
REM Check that the source directory exists.
REM
IF EXIST "%MDB_SOURCE_DIR%" GOTO SOURCE_DIR_EXIST
set ERR_MESSAGE=Non-existant install source directory -- "%MDB_SOURCE_DIR%"
set RC=95
GOTO USAGE_ERROR
:SOURCE_DIR_EXIST
chdir /D "%MDB_SOURCE_DIR%" >> nul 2>&1

rem
rem get MDB_COMMON or use MDB_PRIVATE_DIR if specified
rem
IF ("%MDB_PRIVATE_DIR%") EQU ("") GOTO FIND_CCD
set MDB_COMMON=%MDB_PRIVATE_DIR%
GOTO DEF_INSTALL_CHECK

:FIND_CCD
set FIND_CCD_ARGS=MDB_COMMON
IF ("%MDB_COMMON%") NEQ ("") SET FIND_CCD_ARGS=%FIND_CCD_ARGS% "%MDB_COMMON%" 
CALL findcommon %FIND_CCD_ARGS%
set RC=%ERRORLEVEL%
IF %RC% EQU 0 IF ("%MDB_COMMON%") NEQ ("") GOTO DEF_INSTALL_CHECK
IF "%PRECHECK%" == "yes" GOTO RUN_THIS_COPY
IF "%ERR_MESSAGE%" NEQ "" SET ERR_MESSAGE=Details: %ERR_MESSAGE%
SET ERR_MESSAGE=Unable to determine the Common Component Directory for the MDB. %ERR_MESSAGE%
GOTO FINI_ERROR 

:DEF_INSTALL_CHECK
IF NOT EXIST "%MDB_COMMON%\Windows\setupmdb.bat" GOTO RUN_THIS_COPY
for /f "delims=," %%a in ("%MDB_SOURCE_DIR%") do set MDB_SOURCE_FULL=%%~fa
IF ("%MDB_SOURCE_FULL%") EQU ("%MDB_COMMON%\Windows") GOTO RUN_THIS_COPY
CALL "%MDB_COMMON%\Windows\setupmdb.bat" -PRECEDENCE
set CCD_PRECEDENCE=%ERRORLEVEL%
%DEBUGECHO%
IF %CCD_PRECEDENCE% EQU 999 GOTO RUN_THIS_COPY  
IF %CCD_PRECEDENCE% GTR %PRECEDENCE_VALUE% GOTO RUN_DEF_INSTALL
GOTO RUN_THIS_COPY
:RUN_DEF_INSTALL
chdir /D "%MDB_COMMON%\Windows" > nul 2>&1
set RC=%ERRORLEVEL%
IF %RC% NEQ 0 GOTO RUN_THIS_COPY
rem
rem copy xml files, CumulativePatch and jre (if necessary)
rem
IF "%PRECHECK%" == "yes" GOTO DONE_COPY_XML
rem i switch added to enable copy of 1.5 patches
IF EXIST "%MDB_SOURCE_DIR%\%WORKSPACE%_%DBVENDOR%" xcopy "%MDB_SOURCE_DIR%\%WORKSPACE%_%DBVENDOR%" "%MDB_COMMON%\Windows\%WORKSPACE%_%DBVENDOR%" /E /Q /Y /I /R > nul 2>&1
IF NOT EXIST "%MDB_COMMON%\Windows\MDB104_mssql" IF EXIST "%MDB_SOURCE_DIR%\MDB104_mssql" xcopy "%MDB_SOURCE_DIR%\MDB104_mssql" "%MDB_COMMON%\Windows\MDB104_mssql" /E /Q /Y /I /R > nul 2>&1
IF NOT EXIST "%MDB_COMMON%\Windows\MDB104_oracle" IF EXIST "%MDB_SOURCE_DIR%\MDB104_oracle" xcopy "%MDB_SOURCE_DIR%\MDB104_oracle" "%MDB_COMMON%\Windows\MDB104_oracle" /E /Q /Y /I /R > nul 2>&1
:DONE_COPY_XML

IF EXIST "%MDB_COMMON%\Windows\%DBVENDOR%" GOTO COPY_CUM_PATCH_ONLY
xcopy "%MDB_SOURCE_DIR%\%DBVENDOR%" "%MDB_COMMON%\Windows\%DBVENDOR%" /E /Q /Y /I > nul 2>&1
GOTO DONE_COPY_VENDOR_FILES
:COPY_CUM_PATCH_ONLY
xcopy "%MDB_SOURCE_DIR%\%DBVENDOR%\CumulativePatch" "%MDB_COMMON%\Windows\%DBVENDOR%\CumulativePatch" /E /Q /Y /I > nul 2>&1
:DONE_COPY_VENDOR_FILES

rem IF ("%MDB_SOURCE_DIR%") NEQ ("") SET EXTRA_PARAMS=%EXTRA_PARAMS% -MDB_SOURCE_DIR="%MDB_SOURCE_DIR%"
rem IF ("%MDB_TARGET_DIR%") NEQ ("") SET EXTRA_PARAMS=%EXTRA_PARAMS% -MDB_TARGET_DIR="%MDB_TARGET_DIR%"
IF ("%MDB_PRIVATE_DIR%") NEQ ("") SET EXTRA_PARAMS=%EXTRA_PARAMS% -MDB_PRIVATE_DIR="%MDB_PRIVATE_DIR%"
IF ("%DBDRIVER%") NEQ ("") SET EXTRA_PARAMS=%EXTRA_PARAMS% -DBDRIVER=%DBDRIVER%
IF ("%WORKSPACE%") NEQ ("") SET EXTRA_PARAMS=%EXTRA_PARAMS% -WORKSPACE=%WORKSPACE%
IF ("%EXIT_SCRIPT%") NEQ ("") SET EXTRA_PARAMS=%EXTRA_PARAMS% -MDB_EXIT_SCRIPT=yes
rem IF ("%MDB_COMMON%") NEQ ("") SET EXTRA_PARAMS=%EXTRA_PARAMS% -MDB_COMMON_DIR="%MDB_COMMON%"
IF ("%LOG_DIR%") NEQ ("") SET EXTRA_PARAMS=%EXTRA_PARAMS% -LOG_DIR="%LOG_DIR%"
IF ("%LOG_DIR%") EQU ("") SET EXTRA_PARAMS=%EXTRA_PARAMS% -LOG_DIR="%MDB_TARGET_DIR%"
IF "%COMPAT104%" NEQ "no" SET EXTRA_PARAMS=%EXTRA_PARAMS% -COMPAT104
SET EXTRA_PARAMS=%EXTRA_PARAMS% -MDB_SOURCE_DIR_ORG="%MDB_SOURCE_DIR%"
IF EXIST "%MDB_COMMON%\Windows\jre" GOTO MAKE_DEF_CALL
IF NOT EXIST "%MDB_SOURCE%\jre" GOTO MAKE_DEF_CALL
xcopy "%MDB_SOURCE%\jre" "%MDB_COMMON%\Windows\jre" /E /Q /Y /I > nul 2>&1
:MAKE_DEF_CALL
SET DBPASSWORD_SMDB=%DBPASSWORD_DQ%
SET MDB_ADMIN_PSWD_SMDB=%MDB_ADMIN_PSWD_DQ%
CALL setupmdb %EXTRA_PARAMS% %COMMAND_LINE_ARGS% 
set RC=%ERRORLEVEL%
chdir /D "%CURDIR%" > nul 2>&1
GOTO EXIT_SETUPMDB


rem 
rem run local copy
rem
:RUN_THIS_COPY
set COPY_TO_CCD=yes

:CHECK_COMPAT
IF "%COMPAT104%" == "no" GOTO CONT_INSTALL
rem
rem Install the 1.04 Compatible
rem
IF ("%MDB_SOURCE_DIR%") NEQ ("") SET EXTRA_PARAMS=%EXTRA_PARAMS% -MDB_SOURCE_DIR="%MDB_SOURCE_DIR%"
IF ("%MDB_TARGET_DIR%") NEQ ("") SET EXTRA_PARAMS=%EXTRA_PARAMS% -MDB_TARGET_DIR="%MDB_TARGET_DIR%"
IF ("%MDB_PRIVATE_DIR%") NEQ ("") SET EXTRA_PARAMS=%EXTRA_PARAMS% -MDB_PRIVATE_DIR="%MDB_PRIVATE_DIR%"
IF ("%LOG_DIR%") NEQ ("") SET EXTRA_PARAMS=%EXTRA_PARAMS% -LOG_DIR="%LOG_DIR%"
IF ("%LOG_DIR%") EQU ("") SET EXTRA_PARAMS=%EXTRA_PARAMS% -LOG_DIR="%MDB_TARGET_DIR%"
IF ("%MDB_SOURCE_DIR_ORG%") NEQ ("") (
	SET EXTRA_PARAMS=%EXTRA_PARAMS% -MDB_SOURCE_DIR_ORG="%MDB_SOURCE_DIR_ORG%"
) ELSE (
	SET EXTRA_PARAMS=%EXTRA_PARAMS% -MDB_SOURCE_DIR_ORG="%MDB_SOURCE_DIR%"
)
IF EXIST make_MDB_104_compatible.bat GOTO MAKE_MDB_104_CALL
SET RC=113
set ERR_MESSAGE=make_MDB_104_compatible.bat is not found.
GOTO FINI
:MAKE_MDB_104_CALL
SET DBPASSWORD_SMDB=%DBPASSWORD_DQ%
SET MDB_ADMIN_PSWD_SMDB=%MDB_ADMIN_PSWD_DQ%
CALL make_MDB_104_compatible %COMMAND_LINE_ARGS% %EXTRA_PARAMS% -batch -MDB_EXIT_SCRIPT=yes 
set RC=%ERRORLEVEL%
IF %RC% EQU 0 SET IS_REC_CALL=yes
IF %RC% EQU 0 GOTO CONT_INSTALL
GOTO FINI

:CONT_INSTALL
rem Verify that the target directory is writeable
rem
SET VERIFY_WRITE_FILE="%MDB_TARGET_DIR%\wc_file.txt"
attrib "%MDB_TARGET_DIR%" | findstr /B /R .....R > nul
IF %ERRORLEVEL% EQU 0 GOTO READ_ONLY_TARGET_DIR
set TEMP_CREATE_TARGET=no
IF EXIST "%MDB_TARGET_DIR%" GOTO TEST_TARGET_WRITEABLE
set TEMP_CREATE_TARGET=yes
mkdir "%MDB_TARGET_DIR%"
:TEST_TARGET_WRITEABLE
echo. > %VERIFY_WRITE_FILE% 2>nul
IF EXIST %VERIFY_WRITE_FILE% GOTO WRITEABLE_TARGET_DIR
:READ_ONLY_TARGET_DIR
set RC=125
set ERR_MESSAGE=Unable to write to target directory %MDB_TARGET_DIR%. The directory must be writeable.
goto FINI_ERROR
:WRITEABLE_TARGET_DIR
IF EXIST %VERIFY_WRITE_FILE%  del %VERIFY_WRITE_FILE%                 >> nul 2>&1
IF "%TEMP_CREATE_TARGET%" EQU "yes" rmdir "%MDB_TARGET_DIR%" /S /Q > nul 2>&1 
IF NOT EXIST %VERIFY_WRITE_FILE%  goto VERIFY_FILE_GONE
set ERR_MESSAGE=Unable to modify target directory %MDB_TARGET_DIR%. Read/write access required.
set RC=126
goto FINI_ERROR
:VERIFY_FILE_GONE

rem
rem
rem If pre-check ("-PRECHECK") set?  Validate any given pre-check msg file.
rem Exit w/error if the pre-check msg file  pre-exists.
rem
IF /I NOT "%PRECHECK%" == "yes"    goto END_PRECHECK_PARAM_VALIDATE
rem
rem Call functions to set PRECHECK date and time variables.
rem
IF NOT EXIST "%MDB_TARGET_DIR%" mkdir "%MDB_TARGET_DIR%" 
call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_PRECHECK_DT  "%MDB_TARGET_DIR%"
call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_PRECHECK_TM  "%MDB_TARGET_DIR%"
IF "%TEMP_CREATE_TARGET%" EQU "yes" rmdir "%MDB_TARGET_DIR%" /S /Q > nul 2>&1

IF "%PRECHECK_MSGDIR%" == ""  goto  MSG_STDOUT
IF EXIST "%PRECHECK_MSGDIR%"  goto  MSGDIR_EXISTS
set ERR_MESSAGE=Missing pre-check message log directory.
set RC=60
goto FINI_ERROR

:MSGDIR_EXISTS
set PRECHECK_MSGFILE="%PRECHECK_MSGDIR%\mdb_precheck.log"
rem Check that msg file can be deleted/written.
attrib "%PRECHECK_MSGDIR%" | findstr /B /R .....R > nul
IF %ERRORLEVEL% EQU 0 GOTO READ_ONLY_DIR
echo. > %PRECHECK_MSGFILE% 2>nul
IF EXIST %PRECHECK_MSGFILE% GOTO WRITEABLE_DIR
:READ_ONLY_DIR
set RC=59
set ERR_MESSAGE=Unable to write to Pre-Check directory %PRECHECK_MSGDIR%
set PRECHECK_MSGDIR=
goto FINI_ERROR
:WRITEABLE_DIR
IF EXIST %PRECHECK_MSGFILE%  del %PRECHECK_MSGFILE%                 >> nul 2>&1
IF NOT EXIST %PRECHECK_MSGFILE%  goto MSGFILE_GONE
set ERR_MESSAGE=Failed to delete Pre-Check message log file.
set RC=61
set PRECHECK_MSGDIR=
goto FINI_ERROR
:MSGFILE_GONE
echo DATE=%PRECHECK_DT% >> %PRECHECK_MSGFILE%
echo TIME=%PRECHECK_TM% >> %PRECHECK_MSGFILE%
IF %ERRORLEVEL% EQU 0  goto END_PRECHECK_PARAM_VALIDATE
set ERR_MESSAGE=Failed to create Pre-Check message log file -- %PRECHECK_MSGFILE%
set PRECHECK_MSGDIR=
set RC=63
goto FINI_ERROR
:MSG_STDOUT

echo DATE=%PRECHECK_DT%
echo TIME=%PRECHECK_TM%
:END_PRECHECK_PARAM_VALIDATE




rem
rem Check required arguments.
rem
IF (%DBVENDOR%) NEQ () GOTO REQ_ARG1
set ERR_MESSAGE=Missing required argument: DBVENDOR 
set RC=91
GOTO USAGE_ERROR

:REQ_ARG1
IF (%WORKSPACE%) NEQ () GOTO REQ_ARG2
set ERR_MESSAGE=Missing required argument: WORKSPACE 
set RC=92
GOTO USAGE_ERROR

:REQ_ARG2
IF (%DBDRIVER%) NEQ () GOTO REQ_ARG3
set ERR_MESSAGE=Missing required argument: DBDRIVER 
set RC=93
GOTO USAGE_ERROR

:REQ_ARG3


REM
REM Check the specified database vendor(RDBMS)
REM
IF "%DBVENDOR%" == "ingres" GOTO VALID_VENDOR
IF "%DBVENDOR%" == "mssql"  GOTO VALID_VENDOR
IF "%DBVENDOR%" == "oracle" GOTO VALID_VENDOR_ORACLE
set ERR_MESSAGE=Invalid DBVENDOR: "%DBVENDOR%", expected ingres, mssql, or oracle 
set RC=94
GOTO USAGE_ERROR


:VALID_VENDOR_ORACLE
rem
rem Oracle: Check tablespace args -- if one given, both must be given.
rem
IF (%ORA_DATA_TBLSPACE_NAME%)  EQU ()  GOTO DATA_NAME_NULL
IF (%ORA_INDEX_TBLSPACE_NAME%) NEQ ()  GOTO BOTH_NAMES_GIVEN
   set ERR_MESSAGE=ORA_DATA_TBLSPACE specified but ORA_INDEX_TBLSPACE missing.
   set RC=67
   GOTO USAGE_ERROR

:DATA_NAME_NULL
IF (%ORA_INDEX_TBLSPACE_NAME%) EQU ()  GOTO BOTH_NAMES_NULL
   set ERR_MESSAGE=ORA_INDEX_TBLSPACE specified but ORA_DATA_TBLSPACE missing.
   set RC=67
   GOTO USAGE_ERROR

rem
rem Oracle: w/out given tablespace names => default is MDB_{DATA,INDEX}
rem
:BOTH_NAMES_NULL
   set ORA_DATA_TBLSPACE_NAME=MDB_DATA
   set ORA_INDEX_TBLSPACE_NAME=MDB_INDEX
:BOTH_NAMES_GIVEN

:VALID_VENDOR


set MDBTOOLS_HOME=%MDB_TARGET_DIR%
set MDBTOOLS_CLASSPATH="%MDBTOOLS_HOME%\lib;%MDBTOOLS_HOME%\lib\mdbsetup.jar;%MDBTOOLS_HOME%\lib\eTFWExplorer.jar;%MDBTOOLS_HOME%\lib\log4j-1.2.13.jar;%MDBTOOLS_HOME%\lib\sqljdbc.jar;%MDBTOOLS_HOME%\lib\iijdbc.jar;%MDBTOOLS_HOME%\lib\ojdbc7.jar"

REM
REM Check that the source directory exists.
REM
IF EXIST "%MDB_SOURCE_DIR%" GOTO SOURCE_DIR_EXIST
set ERR_MESSAGE=Non-existant install source directory -- "%MDB_SOURCE_DIR%"
set RC=95
GOTO USAGE_ERROR
:SOURCE_DIR_EXIST
chdir /D "%MDB_SOURCE_DIR%" >> nul 2>&1

REM
REM Validate parameter -II_MDB_CONFIG={yes|no}
REM
IF /I "%II_MDB_CONFIG%" == "yes"   GOTO VALID_CONFIG
IF /I "%II_MDB_CONFIG%" == "no"    GOTO VALID_CONFIG
set ERR_MESSAGE=Invalid II_MDB_CONFIG: "%II_MDB_CONFIG%", expected yes or no
set RC=96
GOTO USAGE_ERROR
:VALID_CONFIG


rem
rem Create the target directory if not there.
rem
IF /I "%MDB_TARGET_DIR%" == "%MDB_SOURCE_DIR%"  goto TGTDIR_EXIST
IF EXIST "%MDB_TARGET_DIR%" GOTO TGTDIR_EXIST
mkdir "%MDB_TARGET_DIR%"
IF %ERRORLEVEL% EQU 0  GOTO MADE_TGTDIR
set ERR_MESSAGE="Could not create target directory -- %MDB_TARGET_DIR%"
set RC=62
GOTO USAGE_ERROR

:MADE_TGTDIR
set TGTDIR_CREATED=yes
IF /I "%PRECHECK%" == "yes"   GOTO TGTDIR_EXIST
call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_DT  "%MDB_TARGET_DIR%"
echo %DT% Install created the mdb directory as "%MDB_TARGET_DIR%" 
:TGTDIR_EXIST

set SCRATCH_DIR=%MDB_TARGET_DIR%
IF ("%LOG_DIR%") EQU ("") GOTO LOGDIR_EXIST
IF /I "%LOG_DIR%" == "%MDB_TARGET_DIR%" GOTO LOGDIR_EXIST
mkdir "%LOG_DIR%" > nul 2>&1
set T_RC=%ERRORLEVEL%
set LOGDIR_CREATED=yes
IF %T_RC% NEQ 0 set LOGDIR_CREATED=no
set SCRATCH_DIR=%LOG_DIR%
IF EXIST "%LOG_DIR%" GOTO LOGDIR_EXIST
IF %T_RC% EQU 0 GOTO LOGDIR_EXIST
set ERR_MESSAGE="Could not create directory -- %LOG_DIR%"
set RC=122
GOTO USAGE_ERROR

:LOGDIR_EXIST
set STATUSFILE="%SCRATCH_DIR%\mdbstatus.log"
IF EXIST %STATUSFILE% DEL %STATUSFILE% >> nul 2>&1

rem
rem Pre-check ordered?  -- Run the pre-script to check connections an short out.
rem Limited src-->tgt copy of pre-install script(w/any support files required)
rem Can the pre-script in pre-check mode be run directly out of the source?
rem (if so this copy section can be removed)
rem
IF /I NOT "%PRECHECK%" == "yes"   goto COPY_IMAGE
chdir /D "%MDB_TARGET_DIR%\%DBVENDOR%"                               >> nul 2>&1
IF /I "%MDB_TARGET_DIR%" == "%MDB_SOURCE_DIR%" goto JAVA_VALIDATION
xcopy "%MDB_SOURCE_DIR%\%DBVENDOR%\setupmdb_pre.bat" "%MDB_TARGET_DIR%" /Q /Y /R > nul 2>&1
xcopy "%MDB_SOURCE_DIR%\%DBVENDOR%\ddl" "%MDB_TARGET_DIR%\ddl" /E /Q /I /Y /R > nul 2>&1
xcopy "%MDB_SOURCE_DIR%\lib"            "%MDB_TARGET_DIR%\lib" /E /Q /I /Y /R > nul 2>&1
xcopy "%MDB_SOURCE_DIR%\MDBLog4j.xml" "%MDB_TARGET_DIR%"  /Q /Y /R > nul 2>&1
IF NOT EXIST "%MDB_SOURCE_DIR%\jre" GOTO VERIFY_COPY 
IF NOT EXIST "%MDB_TARGET_DIR%\jre"  xcopy "%MDB_SOURCE_DIR%\jre" "%MDB_TARGET_DIR%\jre" /E /Q /I /Y > nul 2>&1

rem
rem dont fail if file already existed
rem
:VERIFY_COPY
IF NOT EXIST "%MDB_TARGET_DIR%\setupmdb_pre.bat" GOTO COPY_ERR
IF NOT EXIST "%MDB_TARGET_DIR%\ddl"              GOTO COPY_ERR
IF NOT EXIST "%MDB_TARGET_DIR%\lib"              GOTO COPY_ERR
IF NOT EXIST "%MDB_TARGET_DIR%\MDBLog4j.xml"     GOTO COPY_ERR
IF NOT EXIST "%MDB_SOURCE_DIR%\jre"              GOTO LIM_COPY_END
IF NOT EXIST "%MDB_TARGET_DIR%\jre"              GOTO COPY_ERR
goto LIM_COPY_END
:COPY_ERR
set ERR_MESSAGE=Failed to copy files from  "%MDB_SOURCE_DIR%"  to  "%MDB_TARGET_DIR%"
set RC=97
goto FINI_ERROR
:LIM_COPY_END
chdir /D "%MDB_TARGET_DIR%"                                          >> nul 2>&1
GOTO JAVA_VALIDATION

REM
REM Copy the install image
REM
:COPY_IMAGE
CALL :INCREMENT_STATUS 1
IF /I "%MDB_TARGET_DIR%" == "%MDB_SOURCE_DIR%" GOTO COPY_END
call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_DT  "%MDB_TARGET_DIR%"
echo %DT% Copying "%MDB_SOURCE_DIR%" to "%MDB_TARGET_DIR%"
xcopy "%MDB_SOURCE_DIR%" "%MDB_TARGET_DIR%" /E /Q /Y /R
if %ERRORLEVEL% EQU 0 GOTO COPY_END
set ERR_MESSAGE=Failed to copy files from "%MDB_SOURCE_DIR%" to "%MDB_TARGET_DIR%" (errorlevel=%ERRORLEVEL%)
set RC=97
GOTO FINI_ERROR
:COPY_END
CALL :INCREMENT_STATUS 1
chdir /D "%MDB_TARGET_DIR%"                                          >> nul 2>&1

REM
REM JAVA Validation
REM
:JAVA_VALIDATION
SET JAVA_CMD=java
IF ("%JRE_DIR%") NEQ ("") GOTO CHECK_JRE_DIR
IF EXIST "%MDB_TARGET_DIR%\jre\bin" set PATH=%MDB_TARGET_DIR%\jre\bin;%PATH%
GOTO JRE_JAVA_EXIST

:CHECK_JRE_DIR
set JAVA_CMD="%JRE_DIR%\bin\java.exe"
IF EXIST "%JRE_DIR%"  GOTO JRE_DIR_EXIST
set ERR_MESSAGE=Invalid JRE_DIR directory: "%JRE_DIR%" does not exist
set RC=98
GOTO USAGE_ERROR
:JRE_DIR_EXIST

IF EXIST "%JRE_DIR%\bin\java.exe" GOTO JRE_JAVA_EXIST
set ERR_MESSAGE=Invalid JRE_DIR directory: JRE file "%JRE_DIR%\bin\java.exe" does not exist
set RC=99
GOTO USAGE_ERROR

:JRE_JAVA_EXIST

set JRE_VER_FILE="%MDB_TARGET_DIR%\jver.log"

IF EXIST %JRE_VER_FILE%   DEL %JRE_VER_FILE%
%JAVA_CMD% -version   >> %JRE_VER_FILE% 2>&1
findstr "1\.[5-9]\." %JRE_VER_FILE% > nul 2>&1
if %ERRORLEVEL% EQU 0 GOTO FOUND_JRE
findstr "[2-9]\.[0-9]\." %JRE_VER_FILE% > nul 2>&1
if %ERRORLEVEL% EQU 0 GOTO FOUND_JRE
IF /I NOT "%PRECHECK%" == "yes" type %JRE_VER_FILE%
IF EXIST %JRE_VER_FILE% DEL %JRE_VER_FILE%
set ERR_MESSAGE=Error: Java JRE
IF ("%JRE_DIR%") NEQ ("") SET ERR_MESSAGE=%ERR_MESSAGE% in "%JRE_DIR%"
SET ERR_MESSAGE=%ERR_MESSAGE% must be version 1.5 or higher
set RC=121
GOTO USAGE_ERROR
:FOUND_JRE
IF EXIST %JRE_VER_FILE% DEL %JRE_VER_FILE%


:END_JRE_DIR_EXIST

rem
rem set the command line options for java
rem 

call :SET_MDBTOOLS_OPT

set JAVA_CNCT_DB=%DBNAME%
IF NOT "%DBVENDOR%" == "oracle" GOTO SET_AUTHENTICATION
IF (%ORA_SERVICE_NAME%) NEQ () set JAVA_CNCT_DB=%ORA_SERVICE_NAME% 
rem
rem authentication values for precheck will be different as mdbadmin
rem user is not guaranteed to exist
rem
:SET_AUTHENTICATION
set DBAUTH=
SET T_DBPASSWORD=x%DBPASSWORD%x
SET T_DBPASSWORD="%T_DBPASSWORD:"=%"
IF "%DBVENDOR%" == "oracle" GOTO ORA_AUTH
IF "%DBVENDOR%" == "ingres" GOTO ING_AUTH
IF (%DBUSER%) NEQ () set DBAUTH=%DBAUTH% -dbuser %DBUSER%
IF (%T_DBPASSWORD%) NEQ ("xx") set DBAUTH=%DBAUTH% -dbpassword %DBPASSWORD_DQ%
GOTO END_AUTH
:ORA_AUTH
IF (%DBUSER%) NEQ () GOTO ORA_AUTH_USER_SET
set DBUSER=SYS
set ORA_AS_SYSDBA= AS SYSDBA
:ORA_AUTH_USER_SET
IF /I "%PRECHECK%" == "yes" GOTO ORA_PRECHECK_AUTH
set DBAUTH=%DBAUTH% -dbuser mdbadmin -dbpassword %MDB_ADMIN_PSWD_DQ%  
set DBAUTH=%DBAUTH% -dbprivuser %DBUSER%
IF (%T_DBPASSWORD%) NEQ ("xx") set DBAUTH=%DBAUTH% -dbprivpassword %DBPASSWORD_DQ%
GOTO END_AUTH
:ORA_PRECHECK_AUTH
set DBAUTH=%DBAUTH% -dbuser %DBUSER%
IF (%T_DBPASSWORD%) NEQ ("xx") set DBAUTH=%DBAUTH% -dbpassword %DBPASSWORD_DQ%
GOTO END_AUTH

:ING_AUTH
IF /I "%PRECHECK%" == "yes" GOTO ING_PRECHECK_AUTH
set DBAUTH=%DBAUTH% -dbuser mdbadmin
IF /I (%DBUSER%) EQU (mdbadmin) IF (%T_DBPASSWORD%) NEQ ("xx") GOTO PSWD_EXISTS
IF (%MDB_ADMIN_PSWD%) NEQ () set DBAUTH=%DBAUTH% -dbpassword %MDB_ADMIN_PSWD%
GOTO END_AUTH
:PSWD_EXISTS
set DBAUTH=%DBAUTH% -dbpassword %DBPASSWORD%
GOTO END_AUTH
:ING_PRECHECK_AUTH
IF (%DBUSER%) EQU () GOTO END_AUTH
set DBAUTH=%DBAUTH% -dbuser %DBUSER%
IF (%T_DBPASSWORD%) NEQ ("xx") set DBAUTH=%DBAUTH% -dbpassword %DBPASSWORD%
GOTO END_AUTH

:END_AUTH

rem
rem PatchMDB, if patching look for XML tree in patchMDB specific location
rem
set WORKSPACE_FLD=%WORKSPACE%
IF "%MDB_PATCH%" NEQ "" set WORKSPACE_FLD=patches\%DBDRIVER%\%WORKSPACE%
set SCHEMADIR="%MDBTOOLS_HOME%\%WORKSPACE_FLD%_%DBVENDOR%"

set STATUSDIR=%MDB_TARGET_DIR%
IF ("%LOG_DIR%") NEQ ("") set STATUSDIR=%LOG_DIR%

IF /I NOT "%PRECHECK%" == "yes"   goto NOT_PRECHECK 
rem
rem PRECHECK RDBMS via RDBMS specific setupmdb_pre script.
rem Report only pre-check success.  ERR_MESSAGE and MDB_RC
rem      are set by the setupmdb_pre script. 
rem This call depends on the proper chg to the current dir above.
rem
:PRECHECK_PRE_SCRIPT
call setupmdb_pre
set RC=%ERRORLEVEL%
if %RC% NEQ 0 goto PRINT_PRECHECK_RESULT

rem
rem run the java command with the additional -precheck parameter
rem
chdir /D "%MDB_TARGET_DIR%"                                          >> nul 2>&1
IF EXIST java_output.txt del java_output.txt
rem
rem we cannot gaurantee that the database has been created
rem so use known database names for mssql and ingres
rem
IF "%DBVENDOR%" == "mssql" set JAVA_CNCT_DB=master
IF "%DBVENDOR%" == "ingres" set JAVA_CNCT_DB=iidbdb
call :SET_MDBTOOLS_OPT
%JAVA_CMD% -Dfile.encoding=UTF-8 -Djava.library.path=".\lib" -Xmx256m -classpath %MDBTOOLS_CLASSPATH% com.ca.mdbtools.Utilities -install -apply -verbose -driver %DBDRIVER% -schemadir %SCHEMADIR% -dbvendor %DBVENDOR% -dbhost %DBHOST% -db %JAVA_CNCT_DB%  %DBAUTH% %MDBTOOLS_OPT% -statusdir "%STATUSDIR%" -statusstart %STATUS% -precheck >> java_output.txt 2>&1

set RC=%ERRORLEVEL%
IF %RC% EQU 0  GOTO PRCHK_VERIFY_MANIFEST

set RC=17
set MDB_RC=%RC%
set ERR_MESSAGE=Error encountered while using java to connect to the database %DBNAME%
IF EXIST java_output.txt GOTO JAVA_PRECHECK_ERR
GOTO PRINT_PRECHECK_RESULT

:JAVA_PRECHECK_ERR
if exist java_output_temp.txt del java_output_temp.txt
findstr /B /L /C:ERR_MESSAGE= java_output.txt > java_output_temp.txt
for /F "usebackq tokens=1,2* delims==" %%i in (java_output_temp.txt) do IF (%%j) NEQ () set ERR_MESSAGE=%%j
IF (%PRECHECK_MSGFILE%) EQU () GOTO JAVA_PRECHECK_INFO_STDOUT
findstr /B /L /C:ERR_INFO= java_output.txt >> %PRECHECK_MSGFILE%
GOTO PRINT_PRECHECK_RESULT
:JAVA_PRECHECK_INFO_STDOUT
findstr /B /L /C:ERR_INFO= java_output.txt
GOTO PRINT_PRECHECK_RESULT

:PRCHK_VERIFY_MANIFEST
SET INSTALL_DRIVER=%DBDRIVER%
IF "%DBDRIVER%" EQU "mdb104" IF "%IS_104_COMPATIBLE%" EQU "yes" SET INSTALL_DRIVER=mdbBaseOnly
IF "%IS_104_COMPATIBLE%" EQU "yes" IF "%MDB_VERSION_EXIST%" EQU "yes" IF "%IS_REC_CALL%" EQU "yes" IF "%WORKSPACE%" EQU "MDB104" GOTO TOOLS_END

REM
REM Verify that the manifest exists before installing
REM
CALL :LOCATE_MANIFEST
IF %T_RC% EQU 1 SET RC=123 && GOTO FINI_ERROR
IF %T_RC% EQU 2 SET RC=124 && GOTO FINI_ERROR
GOTO PRINT_PRECHECK_RESULT

:PRINT_PRECHECK_RESULT
IF EXIST java_output.txt del java_output.txt
REM 
REM print the precheck result
REM
GOTO FINI_ERROR


:NOT_PRECHECK


REM
REM Validate pre-script existance.
REM
IF EXIST "%MDB_TARGET_DIR%\%DBVENDOR%\setupmdb_pre.bat" GOTO PRE_SCRIPT_OK
set ERR_MESSAGE=Missing pre-install script: "%MDB_TARGET_DIR%\%DBVENDOR%\setupmdb_pre.bat" 
set RC=100
GOTO FINI_ERROR
:PRE_SCRIPT_OK


REM
REM Set the install log file
REM
set LOGFILE=%MDB_TARGET_DIR%
IF ("%LOG_DIR%") NEQ ("") SET LOGFILE=%LOG_DIR%
set LOGFILE="%LOGFILE%\install_%DBNAME%.log"

call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_DT  "%MDB_TARGET_DIR%"
echo %DT% setupmdb mdb log file in %LOGFILE%
echo . >> %LOGFILE% 2>&1
echo . >> %LOGFILE% 2>&1
echo . >> %LOGFILE% 2>&1
call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_LONG_DT  "%MDB_TARGET_DIR%"
echo . Install Date:  %LONG_DT%                                                  >> %LOGFILE% 2>&1
call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_DT  "%MDB_TARGET_DIR%"
echo %DT% ====================================================================== >> %LOGFILE% 2>&1
echo %DT% ============================= %MDB_VERSION% =========================== >> %LOGFILE% 2>&1
echo %DT% ====================================================================== >> %LOGFILE% 2>&1
echo %DT% setupmdb mdb log file in %LOGFILE% >> %LOGFILE% 2>&1

set LOG_NAME_EXT=%DT%
set LOG_NAME_EXT=%LOG_NAME_EXT:@=_% 
set LOG_NAME_EXT=%LOG_NAME_EXT::=_%
set LOG_NAME_EXT=%LOG_NAME_EXT: =%

REM 
REM Log the full command
REM
call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_DT  "%MDB_TARGET_DIR%"
echo %DT% SetupMDB command: %SETUPMDB_COMMAND% >> %LOGFILE% 2>&1


REM
REM Change to the target directory
REM
chdir /D "%MDB_TARGET_DIR%" >> %LOGFILE% 2>&1



REM
REM Execute the vendor specific pre-processing script
REM
IF EXIST "%MDB_TARGET_DIR%\%DBVENDOR%\setupmdb_pre.bat" GOTO PRE_SCRIPT
set ERR_MESSAGE=Missing pre-install script: "%MDB_TARGET_DIR%\%DBVENDOR%\setupmdb_pre.bat" 
set RC=100
GOTO FINI_ERROR
:PRE_SCRIPT

call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_DT  "%MDB_TARGET_DIR%"
echo %DT% Executing pre-process: "%MDB_TARGET_DIR%\%DBVENDOR%\setupmdb_pre.bat" >>%LOGFILE%
chdir /D "%MDB_TARGET_DIR%\%DBVENDOR%" >> %LOGFILE% 2>&1
call setupmdb_pre  
if %ERRORLEVEL% EQU 0  GOTO PRE_END
set ERR_MESSAGE=Pre-process failed (errorlevel=%ERRORLEVEL%)
set RC=110
chdir /D "%MDB_TARGET_DIR%" >> %LOGFILE% 2>&1
CALL :DB_CLEANUP
GOTO FINI_ERROR
:PRE_END

IF /I (%II_REINIT_CONFIG%)  EQU (TRUE) GOTO FINI
chdir /D "%MDB_TARGET_DIR%" >> %LOGFILE% 2>&1

SET INSTALL_DRIVER=%DBDRIVER%
IF "%DBDRIVER%" EQU "mdb104" IF "%IS_104_COMPATIBLE%" EQU "yes" SET INSTALL_DRIVER=mdbBaseOnly
IF "%IS_104_COMPATIBLE%" EQU "yes" IF "%MDB_VERSION_EXIST%" EQU "yes" IF "%IS_REC_CALL%" EQU "yes" IF "%WORKSPACE%" EQU "MDB104" GOTO TOOLS_END   

REM
REM Verify that the manifest exists before installing
REM
CALL :LOCATE_MANIFEST
IF %T_RC% EQU 1 SET RC=123 && GOTO INSTALL_CLEANUP
IF %T_RC% EQU 2 SET RC=124 && GOTO INSTALL_CLEANUP

REM
REM install using mdbtools...
REM
call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_DT  "%MDB_TARGET_DIR%"
echo %DT% Loading %DBNAME%...
echo %DT% >> %LOGFILE% 2>&1
echo %DT% Loading %DBNAME%...>> %LOGFILE% 2>&1
echo                        using %INSTALL_DRIVER%
echo                        using %INSTALL_DRIVER% >> %LOGFILE% 2>&1

call :SET_MDBTOOLS_OPT
echo %DT% %JAVA_CMD% -Dfile.encoding=UTF-8 -Djava.library.path=^".\lib^" -Xmx256m -classpath %MDBTOOLS_CLASSPATH% com.ca.mdbtools.Utilities -install -apply -verbose -driver %INSTALL_DRIVER% -schemadir %SCHEMADIR% -dbvendor %DBVENDOR% -dbhost %DBHOST% -db %JAVA_CNCT_DB% %MDBTOOLS_OPT% -statusdir "%STATUSDIR%" -statusstart %STATUS% ... >> %LOGFILE% 2>&1
%JAVA_CMD% -Dfile.encoding=UTF-8 -Djava.library.path=".\lib" -Xmx256m -classpath %MDBTOOLS_CLASSPATH% com.ca.mdbtools.Utilities -install -apply -verbose -driver %INSTALL_DRIVER% -schemadir %SCHEMADIR% -dbvendor %DBVENDOR% -dbhost %DBHOST% -db %JAVA_CNCT_DB%  %DBAUTH% %MDBTOOLS_OPT% -statusdir "%STATUSDIR%" -statusstart %STATUS% >> %LOGFILE% 2>&1
set RC=%ERRORLEVEL%
IF %RC% EQU 0 IF "%INSTALL_DRIVER%" NEQ "mdb104" GOTO TOOLS_END
IF %RC% EQU 0 IF "%INSTALL_DRIVER%" EQU "mdb104" GOTO UPDATE_MDB_DATA
GOTO INSTALL_ERR

:UPDATE_MDB_DATA
rem
rem Update the mdb table only if installation of mdb104 was successful
rem
IF "%WORKSPACE_FLD%" NEQ "MDB104" GOTO TOOLS_END
echo %DT% %JAVA_CMD% -Dfile.encoding=UTF-8 -Djava.library.path=^".\lib^" -Xmx256m -classpath %MDBTOOLS_CLASSPATH% com.ca.mdbtools.Utilities -install -apply -verbose -driver mdbtable_data -schemadir %SCHEMADIR% -dbvendor %DBVENDOR% -dbhost %DBHOST% -db %JAVA_CNCT_DB% %MDBTOOLS_OPT% -statusdir "%STATUSDIR%" -statusstart %STATUS% ... >> %LOGFILE% 2>&1
%JAVA_CMD% -Dfile.encoding=UTF-8 -Djava.library.path=".\lib" -Xmx256m -classpath %MDBTOOLS_CLASSPATH% com.ca.mdbtools.Utilities -install -apply -verbose -driver mdbtable_data -schemadir %SCHEMADIR% -dbvendor %DBVENDOR% -dbhost %DBHOST% -db %JAVA_CNCT_DB%  %DBAUTH% %MDBTOOLS_OPT% -statusdir "%STATUSDIR%" -statusstart %STATUS% >> %LOGFILE% 2>&1
set RC=%ERRORLEVEL%
IF %RC% EQU 0 GOTO TOOLS_END
:INSTALL_ERR
set ERR_MESSAGE=MDBTools install error=%RC%
:INSTALL_CLEANUP
call :DB_CLEANUP
GOTO FINI_ERROR
:TOOLS_END



REM
REM execute the vendor specific post-processing script
REM

IF EXIST "%MDB_TARGET_DIR%\%DBVENDOR%\setupmdb_post.bat" GOTO POST_SCRIPT
set ERR_MESSAGE=Missing post-install script: "%MDB_TARGET_DIR%\%DBVENDOR%\setupmdb_post.bat"
set RC=102
GOTO FINI_ERROR

:POST_SCRIPT
call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_DT  "%MDB_TARGET_DIR%"
echo %DT% Executing post-process: "%MDB_TARGET_DIR%\%DBVENDOR%\setupmdb_post.bat" >> %LOGFILE%
chdir /D "%MDB_TARGET_DIR%\%DBVENDOR%" >> %LOGFILE% 2>&1
call setupmdb_post  
if %ERRORLEVEL% EQU 0 GOTO POST_END
set ERR_MESSAGE=Post-process failed (errorlevel=%ERRORLEVEL%)
set RC=120
chdir /D "%MDB_TARGET_DIR%" >> %LOGFILE% 2>&1
GOTO FINI_ERROR
:POST_END
CALL :GEN_RSP_FILE
CALL :COPY_TO_CCD
CALL :SET_STATUS_DONE
chdir /D "%MDB_TARGET_DIR%" >> %LOGFILE% 2>&1




set ERR_MESSAGE=MDB setup completed successfully
call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_DT  "%MDB_TARGET_DIR%"
echo %DT% %ERR_MESSAGE% >> %LOGFILE% 2>&1
set RC=0
GOTO :FINI



:DB_CLEANUP
IF "%DB_CREATED%" == "false" IF "%DB_CREATED_USER%" == "false" GOTO END_CLEANUP
call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_DT  "%MDB_TARGET_DIR%"
echo %DT% %ERR_MESSAGE%
echo %DT% %ERR_MESSAGE% >> %LOGFILE% 2>&1
IF NOT EXIST "%MDB_TARGET_DIR%\%DBVENDOR%\setupmdb_post.bat" GOTO END_CLEANUP

set SAVE_MSG=%ERR_MESSAGE%
call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_DT  "%MDB_TARGET_DIR%"
echo %DT% Executing post-process cleanup: "%MDB_TARGET_DIR%\%DBVENDOR%\setupmdb_post.bat" >> %LOGFILE%
chdir /D "%MDB_TARGET_DIR%\%DBVENDOR%" >> %LOGFILE% 2>&1
call setupmdb_post cleanup 
chdir /D "%MDB_TARGET_DIR%" >> %LOGFILE% 2>&1
set ERR_MESSAGE=%SAVE_MSG%

:END_CLEANUP
GOTO :EOF



:USAGE_ERROR

IF /I  "%PRECHECK%" == "yes"   goto FINI_ERROR

call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_DT  "%MDB_TARGET_DIR%"
echo %DT% SetupMDB command: %SETUPMDB_COMMAND%

echo setupmdb usage:
echo   ^|  setupmdb -DBVENDOR={mssql^|oracle}   [-JRE_DIR=?]
echo   ^|          [-MDB_SOURCE_DIR=?]   [-MDB_PRIVATE_DIR=?]
echo   ^|          [-MDB_TARGET_DIR=?]   [-MDB_COMMON_DIR=?]
echo   ^|          [-DBNAME=?]  [-ORA_SERVICE_NAME=?]  [-DBHOST=?]  [-DBPORT=?]
echo   ^|           -DBUSER=?    -DBPASSWORD=?
echo   ^|           -MANIFEST=?  -WORKSPACE=?  [-COMPAT104]
echo   ^|          [-ORA_DATA_TBLSPACE=?]      [-ORA_DATA_SIZE=?]
echo   ^|          [-ORA_INDEX_TBLSPACE=?]     [-ORA_INDEX_SIZE=?]
echo   ^|          [-ORA_TBLSPACE_PATH=?] -MDB_ADMIN_PSWD=?
echo   ^|          [-PRECHECK]  [-PRECHECK_MSGDIR=?]  [-PRECEDENCE]
echo   ^|          [-MDB_EXIT_SCRIPT={yes,no}] [-debug]
echo   ^| 
echo   ^|  -DBVENDOR           dbms {mssql or oracle}           (required)
echo   ^|  -JRE_DIR            JRE directory                    (opt)
echo   ^|  -MDB_SOURCE_DIR     install media source             (def: cur dir)
echo   ^|  -MDB_PRIVATE_DIR    private MDB destination dir      (def: cur dir)
echo   ^|  -MDB_TARGET_DIR     MDb target dir for logs          (def: cur dir)
echo   ^|  -MDB_COMMON_DIR     common dir(def:C:\"Program Files"\CA\SC or existing)
echo   ^|  -DBNAME             Database (Oracle connect ID)     (def: mdb)
echo   ^|  -ORA_SERVICE_NAME   Oracle service name              (def: DBNAME)
echo   ^|  -DBHOST             database server name             (def: localhost)
echo   ^|  -DBPORT             mdbtools JDBC port               (opt)
echo   ^|  -DBUSER             dbms user for mssql              (required)
echo   ^|                      SYS user for oracle              (def: SYS)
echo   ^|  -DBPASSWORD         dbms user password               (required)
echo   ^|                      SYS user password for oracle     (required)
echo   ^|  -MANIFEST           mdbtools driver name             (required)
echo   ^|  -WORKSPACE          mdbtools workspace name          (required)
echo   ^|  -COMPAT104          Add full MDB 1.0.4 schema        (def: unset)
echo   ^|  -ORA_DATA_TBLSPACE  data tablespace name for oracle  (def: MDB_DATA)
echo   ^|  -ORA_DATA_SIZE      data tablespace size for oracle  (def: 400)
echo   ^|  -ORA_INDEX_TBLSPACE index tablespace name for oracle (def: MDB_INDEX)
echo   ^|  -ORA_INDEX_SIZE     index tablespace size for oracle (def: 100)
echo   ^|  -ORA_TBLSPACE_PATH  tablespace path for oracle       (opt w/omf)
echo   ^|                      required if creating w/out OMF
echo   ^|  -MDB_ADMIN_PSWD     mdbadmin password for oracle     (required)
echo   ^|  -PRECHECK           validate without doing anything  (def: unset)
echo   ^|  -PRECHECK_MSGDIR    validation status msg file       (def: console)
echo   ^|  -MDB_EXIT_SCRIPT    exit script using /B {yes,no}    (def: yes)
echo   ^|  -debug              Run with "echo on"               (def: unset)
echo   ^| 
echo   ^| 
echo   ^|  SQLServer example:
echo   ^|    setupmdb -DBVENDOR=mssql  -DBUSER=sa -DBPASSWORD=sa
echo   ^|             -MANIFEST=mstest -WORKSPACE=mstest  
echo   ^| 
echo   ^|  Oracle example:
echo   ^|    setupmdb -DBVENDOR=oracle -DBNAME=ORCL -DBUSER=sys -DBPASSWORD=sys
echo   ^|             -ORA_TBLSPACE_PATH=d:\oracle\product\10.2.0\oradata\orcl
echo   ^|             -MDB_ADMIN_PSWD=mdbadmin -MANIFEST=oratest -WORKSPACE=oratest  
echo   ^|
echo  End setupmdb usage

GOTO FINI_ERROR



:FINI_ERROR
REM
REM Handle precheck output
REM
IF /I  NOT "%PRECHECK%" == "yes"  goto NORM_FINI
IF ("%PRECHECK_MSGDIR%") EQU ("") goto PRECHK_STDOUT
IF (%PRECHECK_MSGFILE%) EQU ()    goto PRECHK_STDOUT
IF NOT EXIST %PRECHECK_MSGFILE%   goto PRECHK_STDOUT
echo ERR_MESSAGE=%ERR_MESSAGE%  >> %PRECHECK_MSGFILE%
echo RC=%RC%                      >> %PRECHECK_MSGFILE%
GOTO FINI
:PRECHK_STDOUT
echo ERR_MESSAGE=%ERR_MESSAGE%
echo RC=%RC%
GOTO FINI_PRECHECK 

:NORM_FINI
call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_DT  "%MDB_TARGET_DIR%"
echo %DT% %ERR_MESSAGE%

IF (%LOGFILE%) == () goto NO_LOG
call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_DT  "%MDB_TARGET_DIR%"
echo %DT% %ERR_MESSAGE% >> %LOGFILE% 2>&1
echo %DT% Setupmdb exit %EXIT_SCRIPT% return code=%RC% >> %LOGFILE% 2>&1
:NO_LOG

call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_DT  "%MDB_TARGET_DIR%"
echo %DT% Setupmdb exit %EXIT_SCRIPT% return code=%RC%


:FINI
IF EXIST %STATUSFILE% DEL %STATUSFILE% > nul 2>&1
set MSGFILE=%MDB_TARGET_DIR%
IF ("%LOG_DIR%") NEQ ("") SET MSGFILE=%LOG_DIR%
SET T_ERR_MESSAGE=x%ERR_MESSAGE%x
SET T_ERR_MESSAGE="%T_ERR_MESSAGE:"=%"
IF %RC% EQU 0 IF (%T_ERR_MESSAGE%) NEQ ("xx") SET ERR_MESSAGE=MDB setup completed successfully
IF %RC% NEQ 0 IF "%COMPAT104%" EQU "yes" IF (%T_ERR_MESSAGE%) EQU ("xx") GOTO MSGLOG_WRITTEN
echo %DT% %ERR_MESSAGE% > "%MSGFILE%\install_%DBNAME%_msg.log"
:MSGLOG_WRITTEN
:FINI_PRECHECK
IF NOT "%PRECHECK%" == "yes"  goto LEAVE_LOG_DIR
IF ("%CURDIR%")   NEQ ("") chdir /D "%CURDIR%" > nul 2>&1
IF NOT "%TGTDIR_CREATED%" == "yes"  goto LEAVE_TGT
IF NOT EXIST "%MDB_TARGET_DIR%"  goto LEAVE_TGT
for /f "delims=," %%a in ("%MDB_TARGET_DIR%") do set MDB_TARGET_DIR_TEMP_VAR=%%~fa
IF "%MDB_TARGET_DIR_TEMP_VAR%" == "%PRECHECK_MSGDIR%" GOTO LEAVE_TGT
rmdir "%MDB_TARGET_DIR%" /S /Q > nul 2>&1
:LEAVE_TGT

IF NOT "%LOGDIR_CREATED%" == "yes" goto LEAVE_LOG_DIR
IF "%LOG_DIR%" NEQ "" IF EXIST "%LOG_DIR%" rmdir "%LOG_DIR%" /S /Q > nul 2>&1
:LEAVE_LOG_DIR

:EXIT_SETUPMDB
IF %RC% EQU 0 IF NOT "%PRECHECK%" == "yes" CALL :COPY_LOG_TO_CCD
SET T_SAVEPATH=%SAVEPATH:"=%
IF ("%T_SAVEPATH%") NEQ ("") set PATH=%SAVEPATH%
IF ("%CURDIR%")   NEQ ("") chdir /D "%CURDIR%" > nul 2>&1 
endlocal & exit %EXIT_SCRIPT% %RC%

:INCREMENT_STATUS
set /A STATUS=%STATUS% + %1%
rem
rem set SPACES to 1 or 2 spaces depending on the current status
rem value.  although quotes are unnessary, they make the reading
rem of the value easier
rem
set SPACES="  "
IF %STATUS% GTR 9 set SPACES=" "
rem strip quotes
set SPACES=%SPACES:"=%
echo %SPACES%%STATUS%%% %STATUSLINE% > %STATUSFILE%
GOTO :EOF

:SET_STATUS_DONE
IF ("%WORKSPACE%") EQU ("MDB104_%DBVENDOR%") IF "%DBDRIVER%" EQU "mdb104" GOTO END_SET_STATUS
:WRITE_STATUS
echo 100%% MDB created > %STATUSFILE%
:END_SET_STATUS
GOTO :EOF

:WRITE_RESPONSE_FILE
GOTO :EOF

rem
rem If the install from the install image was successful
rem copy scripts to common component directory (CCD)
rem
:COPY_TO_CCD
for /f "delims=," %%a in ("%MDB_SOURCE_DIR%") do set MDB_SOURCE_FULL=%%~fa
rem if our source dir is the ccd, there's nothing to copy
IF ("%MDB_SOURCE_FULL%") EQU ("%MDB_COMMON%\Windows") GOTO DONE_COPY_TO_CCD 
IF "%COPY_TO_CCD%" EQU "no" GOTO DONE_COPY_TO_CCD
IF "%PRECHECK%" == "yes"  goto DONE_COPY_TO_CCD
call "%MDB_SOURCE_DIR%\mdbFuncs"  GET_DT  "%MDB_TARGET_DIR%"
echo %DT% Updating the MDB Common Component Directory >> %LOGFILE% 2>&1
echo %DT% Copying the MDB resources to '%MDB_COMMON%\Windows' >> %LOGFILE% 2>&1
echo %DT% Updating the MDB Common Component Directory 
echo %DT% Copying the MDB resources to '%MDB_COMMON%\Windows'
IF NOT EXIST "%MDB_COMMON%\Windows" mkdir "%MDB_COMMON%\Windows" > nul 2>&1
IF EXIST "%MDB_COMMON%\Windows" GOTO COPY_ITEMS_DIR
echo Warning: Unable to update MDB resources to '%MDB_COMMON%\Windows'
echo Warning: Unable to update MDB resources to '%MDB_COMMON%\Windows' >> %LOGFILE% 2>&1
GOTO DONE_COPY_TO_CCD

:COPY_ITEMS_DIR
REM
REM Copy the files (non directories) in the source dir
REM
SET T_CURDIR=%CD%
chdir /D "%MDB_SOURCE_DIR%" >> nul 2>&1
FOR %%f IN (*) DO CALL :COPY_ITEM %%f
chdir /D "%T_CURDIR%" >> nul 2>&1

REM
REM Copy the directories in the source dir. If the current directory is the 
REM CCD parent directory, then don't copy the Windows directory. STAR 17623061
REM
SET T_CURDIR=%CD%
chdir /D "%MDB_SOURCE_DIR%" >> nul 2>&1
FOR /D %%f IN (*) DO CALL :COPY_DIR %%f
chdir /D "%T_CURDIR%" >> nul 2>&1

rem
rem Copy install file and parameter file
rem
:COPY_PARAM_FILE
set T_SIGFILE=%SIGFILE%
set T_SIGFILE=x%T_SIGFILE%x
set T_SIGFILE=%T_SIGFILE:"=%
IF "%T_SIGFILE%" NEQ "xx" GOTO SIGFILE_DEFINED
rem redundant for furture redefinition
IF "%DBVENDOR%" EQU "oracle" SET SIGFILE="%MDB_TARGET_DIR%\%DBNAME%_signature.txt"
IF "%DBVENDOR%" EQU "mssql"  SET SIGFILE="%MDB_TARGET_DIR%\%DBNAME%_signature.txt"
:SIGFILE_DEFINED
SET CCD_SIG_FILE="%MDB_COMMON%\Windows"
IF EXIST %SIGFILE% xcopy %SIGFILE% %CCD_SIG_FILE% /Q /Y /R > nul 2>&1
IF %RSP_FILE% NEQ "" IF EXIST %RSP_FILE% xcopy %RSP_FILE% "%MDB_COMMON%\Windows\%DBVENDOR%" /Q /Y /R > nul 2>&1 
:DONE_COPY_TO_CCD
GOTO :EOF

rem
rem Copy the Logfile to the CCD
rem
:COPY_LOG_TO_CCD
set COPY_LOGFILE=install_%DBNAME%_%LOG_NAME_EXT%.log
IF NOT EXIST "%MDB_COMMON%\Windows\logs" mkdir "%MDB_COMMON%\Windows\logs" > nul 2>&1 
copy %LOGFILE% "%MDB_COMMON%\Windows\logs\%COPY_LOGFILE%" /Y  > nul 2>&1
IF %ERRORLEVEL% NEQ 0 GOTO DONE_COPY_LOG_TO_CCD
echo. >> %LOGFILE% 2>&1
echo logfile copied to "%MDB_COMMON%\Windows\logs\%COPY_LOGFILE%" >> %LOGFILE% 2>&1
echo. >> %LOGFILE% 2>&1
:DONE_COPY_LOG_TO_CCD
GOTO :EOF

rem
rem Generate the Response file that will be used for make_mdb_104_compat
rem
:GEN_RSP_FILE
set RSP_FILE="%MDB_TARGET_DIR%\%DBVENDOR%\last_install.param"
IF ("%DBVENDOR%")                NEQ ("")    echo DBVENDOR=%DBVENDOR%>                           %RSP_FILE% 2>nul 
IF ("%DBHOST%")                  NEQ ("")    echo DBHOST=%DBHOST%>>                              %RSP_FILE% 2>nul
IF ("%DBNAME%")                  NEQ ("")    echo DBNAME=%DBNAME%>>                              %RSP_FILE% 2>nul
IF ("%MDB_TARGET_DIR%")          NEQ ("")    echo MDB_TARGET_DIR="%MDB_TARGET_DIR%">>            %RSP_FILE% 2>nul
IF ("%DBPORT_ORIG%")             NEQ ("")    echo DBPORT=^%DBPORT_ORIG%>>                   %RSP_FILE% 2>nul
IF ("%ORA_TBLSPACE_PATH%")       NEQ ("")    echo ORA_TBLSPACE_PATH="%ORA_TBLSPACE_PATH%">>      %RSP_FILE% 2>nul
IF ("%ORA_DATA_SIZE%")           NEQ ("")    echo ORA_DATA_SIZE=%ORA_DATA_SIZE%>>                %RSP_FILE% 2>nul
IF ("%ORA_INDEX_SIZE%")          NEQ ("")    echo ORA_INDEX_SIZE=%ORA_INDEX_SIZE%>>              %RSP_FILE% 2>nul
IF ("%ORA_DATA_TBLSPACE_NAME%")  NEQ ("")    echo ORA_DATA_TBLSPACE=%ORA_DATA_TBLSPACE_NAME%>>   %RSP_FILE% 2>nul
IF ("%ORA_INDEX_TBLSPACE_NAME%") NEQ ("")    echo ORA_INDEX_TBLSPACE=%ORA_INDEX_TBLSPACE_NAME%>> %RSP_FILE% 2>nul
IF ("%JRE_DIR%")                 NEQ ("")    echo JRE_DIR="%JRE_DIR%">>                          %RSP_FILE% 2>nul
IF ("%ORA_SERVICE_NAME%")        NEQ ("")    echo ORA_SERVICE_NAME=%ORA_SERVICE_NAME%>>          %RSP_FILE% 2>nul
IF ("%EXIT_SCRIPT%")             NEQ ("")    echo MDB_EXIT_SCRIPT=yes>>                          %RSP_FILE% 2>nul
IF ("%EXIT_SCRIPT%")             EQU ("")    echo MDB_EXIT_SCRIPT=no>>                           %RSP_FILE% 2>nul
IF ("%WORKSPACE%")               NEQ ("")    echo WORKSPACE=%WORKSPACE%>>                        %RSP_FILE% 2>nul
IF ("%DBDRIVER%")                NEQ ("")    echo MANIFEST=%DBDRIVER%>>                          %RSP_FILE% 2>nul
IF ("%MDB_SOURCE_DIR%")          NEQ ("")    echo MDB_SOURCE_DIR="%MDB_SOURCE_DIR%">>            %RSP_FILE% 2>nul
IF ("%MDB_COMMON_DIR%")          NEQ ("")    echo MDB_COMMON_DIR="%MDB_COMMON_DIR%">>            %RSP_FILE% 2>nul
IF ("%MDB_PRIVATE_DIR%")         NEQ ("")    echo MDB_PRIVATE_DIR="%MDB_PRIVATE_DIR%">>          %RSP_FILE% 2>nul
IF ("%COMPAT104%")               EQU ("yes") echo COMPAT104=%COMPAT104%>>                        %RSP_FILE% 2>nul
GOTO :EOF

:LOCATE_MANIFEST
SET T_RC=0
SET COMPAT_STR=""
IF "%DBDRIVER%" EQU "mdb104" IF ("%WORKSPACE_FLD%") EQU ("MDB104") SET COMPAT_STR="COMPAT104 "
SET COMPAT_STR=%COMPAT_STR:"=%
SET XML_FOLDER=%MDBTOOLS_HOME%\%WORKSPACE_FLD%_%DBVENDOR%
IF "%PRECHECK%" == "yes" SET XML_FOLDER=%MDB_SOURCE_DIR%\%WORKSPACE_FLD%_%DBVENDOR%
IF "%PRECHECK%" == "yes" IF "%MDB_SOURCE_DIR_ORG%" NEQ "" SET XML_FOLDER=%MDB_SOURCE_DIR_ORG%\%WORKSPACE_FLD%_%DBVENDOR%
IF NOT EXIST "%XML_FOLDER%" GOTO WRKSPC_MISSING
dir > nul 2>&1
type "%XML_FOLDER%\DBDriver\%INSTALL_DRIVER%.xml" > nul 2>&1
SET T_RC=%ERRORLEVEL%
IF %T_RC% NEQ 0 GOTO MANIFEST_UNREADABLE
GOTO LOCATE_MANIFEST_END

:WRKSPC_MISSING
SET T_RC=1 && SET ERR_MESSAGE=Cannot access %COMPAT_STR%workspace dir-"%XML_FOLDER%"
GOTO LOCATE_MANIFEST_END

:MANIFEST_UNREADABLE
SET T_RC=2 && SET ERR_MESSAGE=Cannot find or read %COMPAT_STR%manifest %XML_FOLDER%\DBDriver\%INSTALL_DRIVER%.xml
GOTO LOCATE_MANIFEST_END

:LOCATE_MANIFEST_END
GOTO :EOF

:COPY_ITEM
IF EXIST "%MDB_SOURCE_DIR%\%1" xcopy "%MDB_SOURCE_DIR%\%1" "%MDB_COMMON%\Windows" /Q /Y /R > nul 2>&1
GOTO :EOF

:COPY_DIR
IF "%1" EQU "Windows" IF "%MDB_COMMON%" EQU "%MDB_SOURCE_DIR%" GOTO END_COPY_DIR
IF EXIST "%MDB_SOURCE_DIR%\%1" xcopy "%MDB_SOURCE_DIR%\%1" "%MDB_COMMON%\Windows\%1" /E /Q /I /Y /R > nul 2>&1
:END_COPY_DIR
GOTO :EOF

:SET_MDBTOOLS_OPT
set MDBTOOLS_OPT=
IF (%ORA_DATA_TBLSPACE_NAME%) NEQ () set MDBTOOLS_OPT=%MDBTOOLS_OPT% -dataTableSpace %ORA_DATA_TBLSPACE_NAME%
IF (%ORA_INDEX_TBLSPACE_NAME%) NEQ () set MDBTOOLS_OPT=%MDBTOOLS_OPT% -indexTableSpace %ORA_INDEX_TBLSPACE_NAME%
IF (%DBPORT%) NEQ () set MDBTOOLS_OPT=%MDBTOOLS_OPT% -dbport %DBPORT%
GOTO :EOF
